/*{ GPIO.c ***********************************************
 *
 * Written by Gregor Rebel 2010-2012
 *
 * Low Level Driver for General Purpose Input Output (GPIO) on EM357/EM357 architecture.
 * 
}*/

#include "em357_gpio.h"

#ifdef EXTENSION_200_cpu_stm32w1xx

GPIOMode_TypeDef em357_map_Type(ttc_gpio_mode_e Type) {
    switch (Type) { // find best matching type for stm32-architecture

    case tgm_analog_in:
      return GPIO_Mode_AN | GPIO_Mode_IN;
    case tgm_input_floating:
      return GPIO_Mode_IN;
    case tgm_input_pull_down:
      return GPIO_Mode_IN_PUD;
    case tgm_input_pull_up:
      return GPIO_Mode_IN_PUD;
    case tgm_output_open_drain:
      return GPIO_Mode_OUT_OD;
    case tgm_output_push_pull:
      return GPIO_Mode_OUT_PP;
    case tgm_alternate_function_push_pull:
      return GPIO_Mode_AF_PP;
    case tgm_alternate_function_open_drain:
      return GPIO_Mode_AF_OD;
    case tgm_alternate_function_SPECIAL_SCLK:
      return GPIO_Mode_AF_PP_SPI ;

    default: Assert(0, ec_InvalidArgument);
    }

    return 0;
}
#else // using defines provided by SimpleMAC-Library <= v2.0.0
GPIOMode_TypeDef em357_map_Type(ttc_gpio_mode_e Type) {
    switch (Type) { // find best matching type for stm32-architecture

    case tgm_analog_in:
      return GPIOCFG_ANALOG;
    case tgm_input_floating:
      return GPIOCFG_IN;
    case tgm_input_pull_down:
      return GPIOCFG_IN_PUD;
    case tgm_input_pull_up:
      return GPIOCFG_IN_PUD;
    case tgm_output_open_drain:
      return GPIOCFG_OUT_OD;
    case tgm_output_push_pull:
      return GPIOCFG_OUT_mode;
    case tgm_alternate_function_push_pull:
      return GPIOCFG_OUT_ALT;
    case tgm_alternate_function_open_drain:
      return GPIOCFG_OUT_ALT_OD;
    case tgm_alternate_function_SPECIAL_SCLK:
      return GPIOCFG_OUT_ALT_SPECIAL_SCLK ;

    default: Assert(0, ec_InvalidArgument);
    }

    return 0;
}
#endif

GPIOSpeed_TypeDef em357_map_Speed(ttc_gpio_speed_e Speed) {
    switch (Speed) { // find best matching speed for stm32-architecture
    case tgs_Min:
    case tgs_2MHz:
    case tgs_10MHz:
    case tgs_50MHz:
    case tgs_Max:   return 0;
    default: Assert(0, ec_InvalidArgument);
    }

    return 0;
}
inline void em357_loadPort_variable(em357_Port_t* Port, GPIO_TypeDef* GPIOx, u8_t Pin) {
        Port->GPIOx=GPIOx;
        Port->Pin  =Pin;
}
void em357_initPort_variable(em357_Port_t* Port, GPIOMode_TypeDef Type) {
  em357_gpio_init_variable(Port, Type, tgs_Max);
}
void em357_gpio_init_variable(em357_Port_t* Port, GPIOMode_TypeDef Type, GPIOSpeed_TypeDef Speed) {
    em357_gpio_init(Port->GPIOx, Port->Pin, Type, Speed);
}
void em357_initPort(GPIO_TypeDef* GPIOx, u8_t Pin, GPIOMode_TypeDef Type) {
    em357_gpio_init(GPIOx, Pin, Type, tgs_Max);
}
void em357_gpio_init(GPIO_TypeDef* GPIOx, u8_t Pin, GPIOMode_TypeDef Type, GPIOSpeed_TypeDef Speed) {

       switch (Type) {

       case GPIOCFG_OUT_mode: {
           switch ( (u32_t) GPIOx) { // enable peripheral clock to this port
             case (u32_t) GPIOA:  halGpioConfig(PORTA_PIN(Pin), GPIOCFG_OUT_mode); break;
             case (u32_t) GPIOB:  halGpioConfig(PORTB_PIN(Pin), GPIOCFG_OUT_mode); break;
             case (u32_t) GPIOC:  halGpioConfig(PORTC_PIN(Pin), GPIOCFG_OUT_mode); break;

             default: Assert(0, ec_InvalidArgument); break; // ERROR: unknown GPIO given!
           }
              break;
           }
       case GPIOCFG_OUT_OD_mode: {
       break;
    }
       case GPIOCFG_OUT_ALT_mode: {
       break;
    }
       case GPIOCFG_OUT_ALT_OD_mode: {
       break;
    }
       case GPIOCFG_ANALOG_mode: {
       break;
    }
       case GPIOCFG_IN_mode: {
       break;
    }
       case GPIOCFG_IN_PUD_mode: {
       break;
    }
       case GPIOCFG_OUT_ALT_SPECIAL_SCLK_mode: {

           switch ( (u32_t) GPIOx) { // enable peripheral clock to this port
             case (u32_t) GPIOA:  halGpioConfig(PORTA_PIN(Pin), GPIOCFG_OUT_ALT_SPECIAL_SCLK); break;
             case (u32_t) GPIOB:  halGpioConfig(PORTB_PIN(Pin), GPIOCFG_OUT_ALT_SPECIAL_SCLK); break;
             case (u32_t)  GPIOC:  halGpioConfig(PORTC_PIN(Pin), GPIOCFG_OUT_ALT_SPECIAL_SCLK); break;

             default: Assert(0, ec_InvalidArgument); break; // ERROR: unknown GPIO given!
           }

       break;
    }
       default: Assert(0, ec_InvalidArgument);

      }

}
inline BOOL em357_gpio_get_variable(em357_Port_t* Port) {
    return em357_gpio_get(Port->GPIOx, Port->Pin);
}
BOOL em357_gpio_get(GPIO_TypeDef* GPIOx, u8_t Pin) {

#ifdef EXTENSION_255_cpu_stm32w1xx_std_peripherals_stm32w108xx_gpio
    return GPIO_ReadInputDataBit(GPIOx, Pin);
#else
    u32_t Mask = (u32_t) 1 << Pin;

    //must use Registers, because hal Library does not support reading GPIOs
    switch ( (u32_t) GPIOx) {
    case (u32_t) GPIOA:  {
        if ( ( (u32_t) GPIO_PAIN_REG) &  Mask)
            return TRUE;
        else
            return FALSE;
        break;
    }
    case (u32_t) GPIOB:  {
        if ( ( (u32_t) GPIO_PBIN_REG) &  Mask)
            return TRUE;
        else
            return FALSE;
        break;
    }
    case (u32_t) GPIOC:  {
        if ( ( (u32_t) GPIO_PCIN_REG) &  Mask)
            return TRUE;
        else
            return FALSE;
        break;
    }

    default: Assert(0, ec_InvalidArgument); break; // ERROR: unknown GPIO given!
    }
    return 0;
#endif
}
inline void em357_gpio_set_variable(em357_Port_t* Port) {
    em357_gpio_set(Port->GPIOx, Port->Pin);
}
void em357_gpio_set(GPIO_TypeDef* GPIOx, u8_t Pin) {

#ifdef EXTENSION_255_cpu_stm32w1xx_std_peripherals_stm32w108xx_gpio
    GPIO_WriteBit(GPIOx, Pin, 1);
#else
     switch ( (u32_t) GPIOx) { // enable peripheral clock to this port
     case (u32_t) GPIOA:  halGpioSet(PORTA_PIN(Pin), 1); break;
     case (u32_t) GPIOB:  halGpioSet(PORTB_PIN(Pin), 1); break;
     case (u32_t) GPIOC:  halGpioSet(PORTC_PIN(Pin), 1); break;

     default: Assert(0, ec_InvalidArgument); break; // ERROR: unknown GPIO given!
     }
#endif
}
inline void em357_gpio_clr_variable(em357_Port_t* Port) {
    em357_gpio_clr(Port->GPIOx, Port->Pin);
}
void em357_gpio_clr(GPIO_TypeDef* GPIOx, u8_t Pin) {

#ifdef EXTENSION_255_cpu_stm32w1xx_std_peripherals_stm32w108xx_gpio
    GPIO_WriteBit(GPIOx, Pin, 0);
#else
    switch ( (u32_t) GPIOx) { // enable peripheral clock to this port
    case (u32_t) GPIOA:  halGpioSet(PORTA_PIN(Pin), 0); break;
    case (u32_t) GPIOB:  halGpioSet(PORTB_PIN(Pin), 0); break;
    case (u32_t)   GPIOC:  halGpioSet(PORTC_PIN(Pin), 0); break;

    default: Assert(0, ec_InvalidArgument); break; // ERROR: unknown GPIO given!
  }
#endif
}
u8_t GPIO_create_index8(ttc_GPIO_BANK GPIOx, u8_t Pin) {

    switch ( (Base_t) GPIOx) {

#ifdef GPIO_BANK_A
    case (Base_t) GPIO_BANK_A: return 0x10 | (0x0f & Pin);
#endif
#ifdef GPIO_BANK_B
    case (Base_t) GPIO_BANK_B: return 0x20 | (0x0f & Pin);
#endif
#ifdef GPIO_BANK_C
    case (Base_t) GPIO_BANK_C: return 0x30 | (0x0f & Pin);
#endif

    default: break;
    }

    return 0;
}
void GPIO_from_index8(u8_t PhysicalIndex, ttc_GPIO_BANK* GPIOx, u8_t* Pin) {

    switch (PhysicalIndex & 0xf0) {

#ifdef GPIO_BANK_A
    case 0x10: *GPIOx = GPIO_BANK_A; break;
#endif
#ifdef GPIO_BANK_B
    case 0x20: *GPIOx = GPIO_BANK_B; break;
#endif
#ifdef GPIO_BANK_C
    case 0x30: *GPIOx = GPIO_BANK_C; break;
#endif

    default: *GPIOx = NULL;
    }

    *Pin = PhysicalIndex & 0x0f;
}
