#ifndef GFX_IMAGE_H
#define GFX_IMAGE_H
/*{ gfx_image.h ************************************************

  Image manipulation 

}*/

//{ Includes *************************************************************
#include "ttc_basic.h"
//} Includes

//{ Defines/ TypeDefs ****************************************************

//} Defines

//{ Structures/ Enums ****************************************************

typedef enum enumcolorDepth { Monochrom = 1, Color8Bit = 8, RealColor= 16, TrueColor = 24} ColorDepth;

typedef struct struct_image {
    u16_t Width;
    u16_t Height;
    ColorDepth Depth;
    unsigned char *Data;
} gfxImage;

//} Structures/ Enums

//{ Global Variables *****************************************************
//} Global Variables

//{ Function prototypes **************************************************

/** delivers single pixel from monochrome image
  *
  * u32_t gfx_image_get_pixel_24bit(u16_t X, u16_t Y, gfxImage *Image)
  *
  * @return  24-bit color value of single pixel
  */
#define gfx_image_get_pixel_24bit(X, Y, Image)  ( (u32_t) Image->Data[3*(Y*(Image->Width)+X)] << 16 ) + ( (u32_t) Image->Data[3*(Y*(Image->Width)+X)+1] << 8 ) + ( (u32_t) Image->Data[3*(Y*(Image->Width)+X)+2] )

/** delivers single pixel from monochrome image
  *
  * u16_t gfx_image_get_pixel_16bit(u16_t X, u16_t Y, gfxImage *Image)
  *
  * @return  16-bit color value of single pixel
  */
#define gfx_image_get_pixel_16bit(X, Y, Image)  (u16_t) Image->Data[( Y*Image->Width+X) * sizeof(u16_t)]  // ToDo: correct?

/** delivers single pixel from monochrome image
  *
  * u8_t gfx_image_get_pixel_8bit(u16_t X, u16_t Y, gfxImage *Image)
  *
  * @return  8-bit color value of single pixel
  */
#define gfx_image_get_pixel_8bit(X, Y, Image)  (u8_t) Image->Data[ (Y*Image->Width+X) * sizeof(u8_t)]  // ToDo: correct?

/** delivers single pixel from monochrome image
  *
  * u8_t gfx_image_get_pixel_1bit(u16_t X, u16_t Y, gfxImage *Image)
  *
  * @return  == 0: pixel not set; != 0: pixel is set
  */
#define gfx_image_get_pixel_1bit(X, Y, Image)  (u8_t) Image->Data[(Y*Image->Width/8 + X/8)] & (X & 7) // ToDo: correct?

//} Function prototypes

#endif
