#include "input_ads7843.h"


// different chip revisions deliver different coordinates for same point
tit_Pen_Holder_t* ia_Pen_Point = NULL;

void input_ads7843_ADS_Write_Byte(u8_t num)
{
    u8_t count=0;                   //declaration of the count variable
    for(count=0;count<8;count++)    //beginning of the for loop, every bit of num will be checked and transmitted
    {
        if(num&0x80)ttc_gpio_set(TP_MOSI);     //the bit on the very left is checked first, in the following the bits will be shifted
        else ttc_gpio_clr(TP_MOSI);            //and also transmitted
        num<<=1;                //shifting
        ttc_gpio_clr(TP_SCLK);                 //after every transmission the clock (TCLK) must be reseted
        ttc_gpio_set(TP_SCLK);
    }
}
u16_t input_ads7843_ADS_Read_AD(u8_t CMD)
{
    u8_t i;                         //declaration of the count variable i
    u8_t count=0;                   //declaration of the count variable count
    u16_t Num=0;                    //declaration of the memory variable Num
    ttc_gpio_clr(TP_SCLK);                         //set clock to 0
    ttc_gpio_clr(TP_CS);                          //chip conversation mode "on"
    input_ads7843_ADS_Write_Byte(CMD);            //sending of the command byte
    for(i=100;i>0;i--);

    ttc_gpio_set(TP_SCLK);                         //reset of the clock
    ttc_gpio_clr(TP_SCLK);
    for(count=0;count<16;count++)
    {
        Num<<=1;                    //Bitshift
        ttc_gpio_clr(TP_SCLK);                     //Clock-Reset
        ttc_gpio_set(TP_SCLK);
        if(ttc_gpio_get(TP_MISO))Num++; 		    //if TOUT =1, the first bit of Num will be set
    }
    Num>>=4;                        //the touch-driver sends 16 bit, but the last 4 are always empty, so bitshifting of 4
    ttc_gpio_set(TP_CS);                          //chip conversation mode "off"
    return(Num);
}
u16_t input_ads7843_ADS_Read_XY(u8_t xy)
{
    u16_t i, j;                      //declaration of the count variable i
    u16_t buf[READ_TIMES];           //declaration of the BUFFER-Array buf
    u16_t sum=0;                     //declaration of the variable sum, which is responsible for calculate the average value
    u16_t temp;                      //declaration of the variable temp, which is responsible for calculate the average value

    for(i=0;i<READ_TIMES;i++)
    {
        buf[i]=input_ads7843_ADS_Read_AD(xy);	    //filling the buffer with the read data of the driver
    }

    for(i=0;i<READ_TIMES-1; i++)
    {
        for(j=i+1;j<READ_TIMES;j++)
        {
            if(buf[i]>buf[j])
            {
                temp=buf[i];
                buf[i]=buf[j];          //bubble-sort for sort values of the array, who are empty to the very right
                buf[j]=temp;
            }
        }
    }
    sum=0;
    for(i=LOST_VAL;i<READ_TIMES-LOST_VAL;i++)sum+=buf[i];    //akkumulation of the data of the buffer in the variable sum
    temp=sum/(READ_TIMES-2*LOST_VAL);                        //caluculating of the average value
    return temp;                                             //return of the calculated value
}
BOOL input_ads7843_read_position(u16_t *x,u16_t *y)
{
    Assert(ia_Pen_Point, ec_NULL);

    u16_t xtemp,ytemp;                              //declaration of the two temp. variables

    if (ia_Pen_Point->ChipVersion==1)
    {
        xtemp=input_ads7843_ADS_Read_XY(CMD_RDX);            //reading of the x,y position
        ytemp=input_ads7843_ADS_Read_XY(CMD_RDY);
        if(xtemp<100||ytemp<100)return 0;                    //check, value in range, if out of range return 0
        xtemp=(xtemp*ia_Pen_Point->xfac+ia_Pen_Point->xoff);       //save the values to given adresses
        ytemp=(ytemp*ia_Pen_Point->yfac+ia_Pen_Point->yoff);
    }
    else
    {
        xtemp=input_ads7843_ADS_Read_XY(CMD_RDY);            //reading of the x,y position
        ytemp=input_ads7843_ADS_Read_XY(CMD_RDX);
        if(xtemp<100||ytemp<100)return 0;                    //check, value in range, if out of range return 0
        xtemp=240-(xtemp*ia_Pen_Point->xfac+ia_Pen_Point->xoff);   //save the values to given adresses
        ytemp=320-(ytemp*ia_Pen_Point->yfac+ia_Pen_Point->yoff);
    }
    if( (0<xtemp) && (xtemp<240) && (0<ytemp) && (ytemp<340) ) { // touch data valid
        *x = xtemp;
        *y = ytemp;

        return TRUE;
    }
    else // no touch data abailable
    {
        return FALSE;
    }
}
void input_ads7843_configuration()
{
    //GPIO_InitTypeDef GPIO_InitStructure; //generate initStructure
    //Enables the High Speed APB (APB2) peripheral clock for the Ports GPI0A, GPI0C, AFI0
    //The AFIO ist neccesary for the EXTI
    //RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA | RCC_APB2Periph_GPIOC | RCC_APB2Periph_AFIO
    //                       , ENABLE );
    //ttc-configuration of the LCD-touch-driver
    //outputs
    ttc_gpio_init(TP_SCLK, tgm_output_push_pull);
    ttc_gpio_init(TP_MOSI, tgm_output_push_pull);
    ttc_gpio_init(TP_CS, tgm_output_push_pull);
    //inputs
    ttc_gpio_init(TP_MISO, tgm_input_floating);
    ttc_gpio_init(TP_INT,tgm_input_floating);

    //old implementation
    /**************************************************************************

    //Configure PA5&7 pin: TP_SCLK(5) pin TP_MOSI(7) pin
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_5 | GPIO_Pin_7;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOA,&GPIO_InitStructure);

    //Configure PA6 pin: TP_MISO pin
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU;
    GPIO_Init(GPIOA,&GPIO_InitStructure);

    //Configure PA4 pin: TP_CS pin
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_4;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOA,&GPIO_InitStructure);

    //Configure PC13 pin: TP_INT pin
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_13;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU;
    GPIO_Init(GPIOC,&GPIO_InitStructure);

***************************************************************************/
}
void input_ads7843_init(tit_Pen_Holder_t* Pen_Point) {
    Assert(Pen_Point, ec_NULL);

    input_ads7843_configuration();
    input_ads7843_read_position(&Pen_Point->X,&Pen_Point->Y);

  /*  if (1) { // ToDo: replace by ttc_interrupt-call
         Connect PEN EXTI Line to Key Button GPIO Pin
        GPIO_EXTILineConfig(GPIO_PortSourceGPIOC, GPIO_PinSource13); //FixMe

         Configure PEN EXTI Line to generate an interrupt on falling edge
        EXTI_InitTypeDef EXTI_InitStructure;
        EXTI_InitStructure.EXTI_Line = EXTI_Line13; //PC13      //FixMe
        EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
        EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Falling;//FixMe
        EXTI_InitStructure.EXTI_LineCmd = ENABLE;
        EXTI_Init(&EXTI_InitStructure);

        Generate software interrupt: simulate a falling edge applied on PEN EXTI line
        EXTI_GenerateSWInterrupt(EXTI_Line13);//FixMe
    }
    */
    //Values for calibrating the touch screen, optional, function for autoadjust
    Pen_Point->xoff = -10;
    Pen_Point->yoff = -10;
    Pen_Point->xfac = 0.13;
    Pen_Point->yfac = 0.18;

    Pen_Point->ChipVersion = 1; // ToDo: correct?

    ia_Pen_Point = ia_Pen_Point;
}
