/*{ lcd_ili9320.c *******************************

  Low-level driver for LCD Controller ILI9320.

  written by Patrick von Poblotzki, Gregor Rebel 2012

  Benchmark v1.0.50(5)
  benchmark_Clear(5)        4975 ms
  benchmark_Circle(5)      34086 ms
  benchmark_Line_ortho(5)  34297 ms
  benchmark_Line(5)        35196 ms
  benchmark_Rect(5)        31492 ms
  benchmark_Text(5)        22577 ms
  benchmark_SetPixel(5)    33842 ms
  benchmark_RectFill(5)    31159 ms
  sum:                     227624 ms

  Benchmark v1.0.50(5)
  changes: vert and hori lines are drawn with only one adress command
           command pins are controled by adress
           debug compile
  benchmark_Clear(5)       1251  ms
  benchmark_Circle(5)      8852  ms
  benchmark_Line_ortho(5)  8258  ms
  benchmark_Line(5)        9426  ms
  benchmark_Rect(5)        8209  ms
  benchmark_Text(5)        2359  ms
  benchmark_SetPixel(5)    8880  ms
  benchmark_RectFill(5)    8140  ms
  sum:                     55375 ms

  Benchmark v1.0.50(5)
  changes smallest compile
  benchmark_Clear(5)       1181  ms
  benchmark_Circle(5)      8549  ms
  benchmark_Line_ortho(5)  1478  ms
  benchmark_Line(5)        9543  ms
  benchmark_Rect(5)        1497  ms
  benchmark_Text(5)        2040  ms
  benchmark_SetPixel(5)    9032  ms
  benchmark_RectFill(5)    1176  ms
  sum:                     34888 ms

  Benchmark v1.0.50(5)
  register inline
  benchmark_Clear(5)       1181  ms
  benchmark_Circle(5)      8594  ms
  benchmark_Line_ortho(5)  1506  ms
  benchmark_Line(5)        9173  ms
  benchmark_Rect(5)        1533  ms
  benchmark_Text(5)        2023  ms
  benchmark_SetPixel(5)    8666  ms
  benchmark_RectFill(5)    1173  ms
  sum:                     33849 ms


}*/

#include "lcd_ili9320.h"

// define parallel banks to use for parallel output in this example
#ifdef TTC_PARALLEL16_1_BANK_
#define PARALLEL16_BANK TTC_PARALLEL16_1_BANK
#else
// using 8-bit ports if no 16-bit port available
#ifdef TTC_PARALLEL08_1_BANK
#define PARALLEL08_1_BANK     TTC_PARALLEL08_1_BANK
#define PARALLEL08_1_FIRSTPIN TTC_PARALLEL08_1_FIRSTPIN
#endif
#ifdef TTC_PARALLEL08_2_BANK
#define PARALLEL08_2_BANK     TTC_PARALLEL08_2_BANK
#define PARALLEL08_2_FIRSTPIN TTC_PARALLEL08_2_FIRSTPIN
#endif
#endif

// minimal interface for ttc_gfx.c (must be implemented)
void ili9320_get_configuration() {

    if (! ttc_gfx_CurrentDisplay->Width)  ttc_gfx_CurrentDisplay->Width  = 320;
    if (! ttc_gfx_CurrentDisplay->Height) ttc_gfx_CurrentDisplay->Height = 240;
    if (! ttc_gfx_CurrentDisplay->Depth)  ttc_gfx_CurrentDisplay->Depth  = 24;

    // fill in mandatory function pointers
    ttc_gfx_CurrentDisplay->function_set_color_fg24 = ili9320_set_color_fg24;
    ttc_gfx_CurrentDisplay->function_set_color_bg24 = ili9320_set_color_bg24;

    // fill in optional function pointers
    ttc_gfx_CurrentDisplay->function_set_backlight  = ili9320_set_backlight;
    ttc_gfx_CurrentDisplay->function_clear          = ili9320_clear;
    ttc_gfx_CurrentDisplay->function_line_horizontal= ili9320_line_horizontal;
    ttc_gfx_CurrentDisplay->function_line_vertical  = ili9320_line_vertical;
    ttc_gfx_CurrentDisplay->function_rect_fill      = ili9320_rect_fill;

#ifdef EXTENSION_500_ttc_font  // font attributes
    ttc_gfx_CurrentDisplay->function_put_char       = ili9320_put_char;
    ttc_gfx_CurrentDisplay->function_put_char_solid = ili9320_put_char_solid;
#endif
}
void ili9320_init() {
    static u16_t DeviceCode=0x9325; //int for the Device Code of the TFT-drive

    _ttc_gpio_init_u8_pin(PARALLEL08_1_BANK,PARALLEL08_1_FIRSTPIN,tgm_output_push_pull,tgs_Max); // ToDo: Do not use private function!
    _ttc_gpio_init_u8_pin(PARALLEL08_2_BANK,PARALLEL08_2_FIRSTPIN,tgm_output_push_pull,tgs_Max); // ToDo: Do not use private function!


    ttc_gpio_init(PIN_PC8, tgm_output_push_pull);
    ttc_gpio_init(PIN_PC9, tgm_output_push_pull);
    ttc_gpio_init(PIN_PC10, tgm_output_push_pull);
    ttc_gpio_init(PIN_PC11, tgm_output_push_pull);


#ifdef PARALLEL08_1_BANK
    // initialize 8-bit wide output port

    for (int Bit = 0; Bit < 8; Bit++) {
        ttc_gpio_init2(PARALLEL08_1_BANK, Bit + PARALLEL08_1_FIRSTPIN,  tgm_output_push_pull, tgs_Max);
    }

#define USE_BANK8_1
#endif

#ifdef PARALLEL08_2_BANK
    // initialize 8-bit wide output port
    for (int Bit = 0; Bit < 8; Bit++) {
        ttc_gpio_init2(PARALLEL08_2_BANK, Bit + PARALLEL08_2_FIRSTPIN,  tgm_output_push_pull, tgs_Max);
    }

#define USE_BANK8_2
#endif

    ili9320_write_register(0x0000,0x0001); //command for reading the register
    ili9320_delay(5);                 // LCD_Delay 5 ms
    DeviceCode = ili9320_read_register(0x0000); //save Readreg to var. DeviceCode
     ili9320_delay(5);
     ili9320_write_register(0x0000,0x0001);
    DeviceCode = ili9320_read_register(0x0000); //save Readreg to var. DeviceCode
    ili9320_delay(5);

    //power configuration, not optimized, for more information ->manual
    if (DeviceCode==0x9325 || DeviceCode==0x9328)//ILI9325
    {

        ili9320_write_register(0x0001,0x0100);       //Driver Output Contral.
        ili9320_write_register(0x0002,0x0700);       //LCD Driver Waveform Contral.
        ili9320_write_register(0x0003,0x1030);       //Entry Mode Set.
        ili9320_write_register(0x0004,0x0000);       //Scalling Control.
        ili9320_write_register(0x0007,0x0133);       //Display Control.
        ili9320_write_register(0x0008,0x0202);       //Display Contral 2.(0x0207)
        ili9320_write_register(0x0009,0x0000);       //Display Contral 3.(0x0000)
        ili9320_write_register(0x000a,0x0000);       //Frame Cycle Control.(0x0000)
        ili9320_write_register(0x000c,0x0001);       //Extern Display Interface Control 1.(0x0000) = 16 bit
        ili9320_write_register(0x000d,0x0000);       //Frame Maker Position.
        ili9320_write_register(0x000f,0x0000);       //Extern Display Interface Control 2.
        ili9320_write_register(0x0010,0x1690);       //Power Control 1.(0x16b0)
        ili9320_write_register(0x0011,0x0227);       //Power Control 2.(0x0001)
        ili9320_write_register(0x0012,0x009d);       //Power Control 3.(0x0138)
        ili9320_write_register(0x0013,0x1900);       //Power Control 4.
        ili9320_write_register(0x0020,0x0000);
        ili9320_write_register(0x0021,0x0000);
        ili9320_write_register(0x0029,0x0025);
        ili9320_write_register(0x002b,0x000d);
        ili9320_write_register(0x0030,0x0007);
        ili9320_write_register(0x0031,0x0303);
        ili9320_write_register(0x0032,0x0003);
        ili9320_write_register(0x0035,0x0206);
        ili9320_write_register(0x0036,0x0008);
        ili9320_write_register(0x0037,0x0406);
        ili9320_write_register(0x0038,0x0304);
        ili9320_write_register(0x0039,0x0007);
        ili9320_write_register(0x003c,0x0602);
        ili9320_write_register(0x003d,0x0008);
        ili9320_write_register(0x0050,0x0000);       //Set X - Window Start
        ili9320_write_register(0x0051,0x00ef);       //Set X - Window END
        ili9320_write_register(0x0052,0x0000);       //Set Y - Window Start
        ili9320_write_register(0x0053,0x013f);       //Set Y - Window END
        ili9320_write_register(0x0060,0xa700);
        ili9320_write_register(0x0061,0x0001);
        ili9320_write_register(0x006a,0x0000);
        ili9320_write_register(0x0080,0x0000);
        ili9320_write_register(0x0081,0x0000);
        ili9320_write_register(0x0082,0x0000);
        ili9320_write_register(0x0083,0x0000);
        ili9320_write_register(0x0084,0x0000);
        ili9320_write_register(0x0085,0x0000);
        ili9320_write_register(0x0090,0x0010);
        ili9320_write_register(0x0092,0x0600);
        ili9320_write_register(0x00e5,0x78F0);       //LCD_WriteReg(0x0000,0x0001);

    }
    else if (DeviceCode==0x9320 || DeviceCode==0x9300|| DeviceCode==0x9324||DeviceCode==0x1320||DeviceCode==0x9125)
    {
        ili9320_write_register(0x00,0x0000);
        ili9320_write_register(0x01,0x0100);	//Driver Output Contral.
        ili9320_write_register(0x02,0x0700);	//LCD Driver Waveform Contral.
        ili9320_write_register(0x03,0x1030);    //Entry Mode Set.
        ili9320_write_register(0x04,0x0000);	//Scalling Control.
        ili9320_write_register(0x08,0x0202);	//Display Contral 2.(0x0207)
        ili9320_write_register(0x09,0x0000);	//Display Contral 3.(0x0000)
        ili9320_write_register(0x0a,0x0000);	//Frame Cycle Contal.(0x0000)
        ili9320_write_register(0x0c,(1<<0));	//Extern Display Interface Control 1.(0x0000) = 16 bit
        ili9320_write_register(0x0d,0x0000);	//Frame Maker Position.
        ili9320_write_register(0x0f,0x0000);	//Extern Display Interface Control 2.
        ili9320_delay(10);
        ili9320_write_register(0x07,0x0101);	//Display Control.
        ili9320_delay(10);
        ili9320_write_register(0x10,(1<<12)|(0<<8)|(1<<7)|(1<<6)|(0<<4));                         	//Power Control 1.(0x16b0)
        ili9320_write_register(0x11,0x0007);								//Power Control 2.(0x0001)
        ili9320_write_register(0x12,(1<<8)|(1<<4)|(0<<0));                        			//Power Control 3.(0x0138)
        ili9320_write_register(0x13,0x0b00);								//Power Control 4.
        ili9320_write_register(0x29,0x0000);								//Power Control 7.
        ili9320_write_register(0x2b,(1<<14)|(1<<4));
        ili9320_write_register(0x50,0);         //Set X - Window Start
        ili9320_write_register(0x51,239);       //Set X - Windows End
        ili9320_write_register(0x52,0);   	    //Set Y - Windows Start
        ili9320_write_register(0x53,319);       //Set Y - Window End
        ili9320_write_register(0x60,0x2700);	//Driver Output Control.
        ili9320_write_register(0x61,0x0001);	//Driver Output Control.
        ili9320_write_register(0x6a,0x0000);	//Vertical Srcoll Control.
        ili9320_write_register(0x80,0x0000);	//Display Position? Partial Display 1.
        ili9320_write_register(0x81,0x0000);	//RAM Address Start? Partial Display 1.
        ili9320_write_register(0x82,0x0000);	//RAM Address End-Partial Display 1.
        ili9320_write_register(0x83,0x0000);	//Displsy Position? Partial Display 2.
        ili9320_write_register(0x84,0x0000);	//RAM Address Start? Partial Display 2.
        ili9320_write_register(0x85,0x0000);	//RAM Address End? Partial Display 2.
        ili9320_write_register(0x90,(0<<7)|(16<<0));	//Frame Cycle Contral.(0x0013)
        ili9320_write_register(0x92,0x0000);	//Panel Interface Contral 2.(0x0000)
        ili9320_write_register(0x93,0x0001);	//Panel Interface Contral 3.
        ili9320_write_register(0x95,0x0110);	//Frame Cycle Contral.(0x0110)
        ili9320_write_register(0x97,(0<<8));	//
        ili9320_write_register(0x98,0x0000);	//Frame Cycle Contral.
        ili9320_write_register(0x07,0x0173);	//(0x0173)
        ili9320_delay(10);
    }

    //first cleanup of the display
    ili9320_clear();
}

inline void ili9320_clr_pixel() {
    // set pixel
    ili9320_write_command(0x22);             // Command for write dot a activ position
    ili9320_write_data(ttc_gfx_CurrentColorBg);    // Color-data for the point
}
void ili9320_set_color_fg24(u32_t Color) {
    ttc_gfx_CurrentDisplay->ColorFg = ttc_gfx_CurrentColorFg = ( (((Color>>3)&0x1F) |( (Color>>5)&0x7E0 )| ((Color>>8)&0xF800)) );
}
void ili9320_set_color_bg24(u32_t Color) {
    ttc_gfx_CurrentDisplay->ColorBg = ttc_gfx_CurrentColorBg = ( (((Color>>3)&0x1F) |( (Color>>5)&0x7E0 )| ((Color>>8)&0xF800)) );
}
void ili9320_set_cursor(u16_t X, u16_t Y) {
    ili9320_write_register(0x20, X);
    ili9320_write_register(0x21, Y);
}

// extended interface for ttc_gfx.c (can be implemented)
void ili9320_clear() {
    GFX_STATIC u16_t ForegroundColor; ForegroundColor = ttc_gfx_CurrentDisplay->ColorBg;


    ili9320_write_register(0x50,0);           // Set X - Window Start
    ili9320_write_register(0x51,ttc_gfx_CurrentDisplay->Width -1);         // Set X - Windows End
    ili9320_write_register(0x52,0);   	      // Set Y - Windows Start
    ili9320_write_register(0x53,ttc_gfx_CurrentDisplay->Height - 1);         // Set Y - Window End
    ili9320_write_command(0x22);              // prepare GRAM for writing

    //mass sending off data to the driver, runtime shorten by mass-orders of 120x (ili9320_write_data(Color);)
     Base_t PixelCount = ttc_gfx_CurrentDisplay->Height * ttc_gfx_CurrentDisplay->Width;

     while (PixelCount & 0xf) { // make it dividable by 16
         ili9320_write_data(ForegroundColor);
         PixelCount--;
     }
     while (PixelCount > 0) {
         ili9320_write_data(ForegroundColor);
         ili9320_write_data(ForegroundColor);
         ili9320_write_data(ForegroundColor);
         ili9320_write_data(ForegroundColor);
         ili9320_write_data(ForegroundColor);
         ili9320_write_data(ForegroundColor);
         ili9320_write_data(ForegroundColor);
         ili9320_write_data(ForegroundColor);
         ili9320_write_data(ForegroundColor);
         ili9320_write_data(ForegroundColor);
         ili9320_write_data(ForegroundColor);
         ili9320_write_data(ForegroundColor);
         ili9320_write_data(ForegroundColor);
         ili9320_write_data(ForegroundColor);
         ili9320_write_data(ForegroundColor);
         ili9320_write_data(ForegroundColor);
         PixelCount -= 16;
     }
}

void ili9320_set_backlight(u8_t Level) {
    (void) Level;
}
void ili9320_rect_fill(u16_t X, u16_t Y, s16_t W, s16_t H) {

    ili9320_write_register(0x50,X);           //Set X - Window Start
    ili9320_write_register(0x51,X+W-1);         //Set X - Windows End
    ili9320_write_register(0x52,Y);   	 //Set Y - Windows Start
    ili9320_write_register(0x53,Y+H-1);         //Set Y - Window End
    ili9320_write_command(0x22);
    ili9320_set_cursor(X,Y);
    ili9320_write_command(0x22);
    u16_t ForegroundColor = ttc_gfx_CurrentDisplay->ColorFg;

    Base_t PixelCount = W*H;

    while (PixelCount & 0xf) { // make it dividable by 16
        ili9320_write_data(ForegroundColor);
        PixelCount--;
    }
    while (PixelCount > 0) {
        ili9320_write_data(ForegroundColor);
        ili9320_write_data(ForegroundColor);
        ili9320_write_data(ForegroundColor);
        ili9320_write_data(ForegroundColor);
        ili9320_write_data(ForegroundColor);
        ili9320_write_data(ForegroundColor);
        ili9320_write_data(ForegroundColor);
        ili9320_write_data(ForegroundColor);
        ili9320_write_data(ForegroundColor);
        ili9320_write_data(ForegroundColor);
        ili9320_write_data(ForegroundColor);
        ili9320_write_data(ForegroundColor);
        ili9320_write_data(ForegroundColor);
        ili9320_write_data(ForegroundColor);
        ili9320_write_data(ForegroundColor);
        ili9320_write_data(ForegroundColor);
        PixelCount -= 16;
    }

    ili9320_write_register(0x50,0);           //Set X - Window Start
    ili9320_write_register(0x51,239);         //Set X - Windows End
    ili9320_write_register(0x52,0);   	 //Set Y - Windows Start
    ili9320_write_register(0x53,359);         //Set Y - Window End
}

#ifdef EXTENSION_500_ttc_font  // font support available
void ili9320_put_char(u8_t Char) {
    static u16_t BitLine;
    static  u8_t LineNo;
    static u16_t X;
    static u16_t Y;              Y = ttc_gfx_CurrentDisplay->CursorY;
    static const u16_t* Font; Font = ttc_gfx_CurrentDisplay->Font->Characters + (Char - 32) * 24;

    for (LineNo = 0; LineNo < 24; LineNo++)
    {
        BitLine = *Font++;

        if (BitLine) {
            X = ttc_gfx_CurrentDisplay->CursorX;
            do {
                if (BitLine & 1) {
                    ili9320_write_register(0x20, X);
                    ili9320_write_register(0x21, Y);           // Set cursor on the LCD
                    ili9320_put_pixel();
                }
                X++;
                BitLine /= 2;
            } while (BitLine);
        }
        Y++;
    }
}
void ili9320_put_char_solid(u8_t Character) {
    u16_t k,line;
    u16_t CurrentX = ttc_gfx_CurrentDisplay->CursorX;
    u16_t CurrentY = ttc_gfx_CurrentDisplay->CursorY;

    ili9320_set_cursor(CurrentX, CurrentY);
    ili9320_write_register(0x50, CurrentX);           // Set X - Window Start
    ili9320_write_register(0x51, CurrentX+15);        // Set X - Windows End
    ili9320_write_register(0x52, CurrentY);   	     // Set Y - Windows Start
    ili9320_write_register(0x53, CurrentY+23);        // Set Y - Window End

    ili9320_write_command(0x22); //
    u16_t BackgroundColor    = ttc_gfx_CurrentDisplay->ColorBg;
    u16_t ForegroundColor    = ttc_gfx_CurrentDisplay->ColorFg;
    const u16_t* CurrentFont = ttc_gfx_CurrentDisplay->Font->Characters;

    for (k=0; k<21; k++) {
        line = CurrentFont[k+((Character-32)*24)];

        for ( u32_t BitMask = 1; BitMask < 0x10000; BitMask = BitMask << 1) {
            if (line & BitMask) {
                ili9320_write_data(ForegroundColor);
            }
            else {
                ili9320_write_data(BackgroundColor);
            }
        }
    }

    ili9320_write_register(0x50,0);           //Set X - Window Start
    ili9320_write_register(0x51,239);         //Set X - Window End
    ili9320_write_register(0x52,0);   	      //Set Y - Window Start
    ili9320_write_register(0x53,319);         //Set Y - Window End
}
#endif


// private support  functions
void ili9320_set_display_window(u8_t Xpos, u16_t Ypos, u8_t Height, u16_t LI9320_WIDTH) {
    ili9320_write_register(0x50, Xpos);	   	   	//Partial Image 1 Display Position to X-pos
    ili9320_write_register(0x51, Xpos+Height);                 //Partial Image 1 Area (Start Line) to Xpos+Height
    ili9320_write_register(0x52, Ypos);		  	//Partial Image 1 Area (End Line) to Ypos
    ili9320_write_register(0x53, Ypos+LI9320_WIDTH);                  //Partial Image 2 Display Position Ypos+LI9320_WIDTH

    ili9320_set_cursor(Xpos, Ypos);			//Set Cursor on xpos, ypos
}
u16_t ili9320_read_register(u16_t LCD_Reg) {
    u16_t temp; //initialize the variable temp which will be returned
    ili9320_write_command(LCD_Reg);  // order the driver to send data

    //init BANKS as input
    _ttc_gpio_init_u8_pin(PARALLEL08_1_BANK,PARALLEL08_1_FIRSTPIN,tgm_input_floating,tgs_Max);
    _ttc_gpio_init_u8_pin(PARALLEL08_2_BANK,PARALLEL08_2_FIRSTPIN,tgm_input_floating,tgs_Max);


    CLR_LCD_CS;                                         //pull LCD_CS to zero -> beginning open the read mode
    SET_LCD_RS;                                         //push LCD_RS to iovcc
    CLR_LCD_RD;                                          //pull LCD_RD to zero -> now the driver sends data


    temp=(_ttc_gpio_get_u8_pin(GPIO_BANK_C,0) )| (_ttc_gpio_get_u8_pin(GPIO_BANK_B,8)<<8);//saves sended data to temp



    SET_LCD_RD;
    SET_LCD_CS;                                           //closing of the read-mode of the TFT-driver

    //init BANKS as output
    _ttc_gpio_init_u8_pin(PARALLEL08_1_BANK,PARALLEL08_1_FIRSTPIN,tgm_output_push_pull,tgs_Max);
    _ttc_gpio_init_u8_pin(PARALLEL08_2_BANK,PARALLEL08_2_FIRSTPIN,tgm_output_push_pull,tgs_Max);


    return temp;   //return of the Reg-data
}
void ili9320_write_command(u16_t LCD_Dat) {

    CLR_LCD_CS;   // pull LCD-Chip-Select to zero -> chip-select is activated
    CLR_LCD_RS;   // pull LCD_Register-Select to zero -> now the driver expect data which he will interprent as command

    //  GPIOC->ODR = (GPIOC->ODR&0xff00)|(LCD_Reg&0x00ff);          //save the 8 high pins of C and the 8 low pins of LCD_Reg to the output register
    //  GPIOB->ODR = (GPIOB->ODR&0x00ff)|(LCD_Reg&0xff00);         //save the 8 high pins of C and the 8 low pins of LCD_Reg to the output register

    //sending the Command-Data to the driver
    ttc_gpio_set_u8_pin3(PARALLEL08_2_BANK, PARALLEL08_2_FIRSTPIN, (0x00ff & LCD_Dat));
    ttc_gpio_set_u8_pin3(PARALLEL08_1_BANK, PARALLEL08_1_FIRSTPIN, (0xff00 & LCD_Dat)>>8);

    CLR_LCD_WR;    //renew the Write mode -> clock count
    SET_LCD_WR;
    SET_LCD_CS;    //push LCD_CS to vcc -> Chip-Select is deactivated
}
void ili9320_write_data(u16_t LCD_Dat) {
    CLR_LCD_CS;//pull LCD-Chip-Select to zero -> chip-select is activated
    SET_LCD_RS;//push LCD_Register-Select to vcc -> now the driver expect data which he will interprent as data

    //ili9320_parallelOutput(LCD_Dat);
    // GPIOC->ODR = (GPIOC->ODR&0xff00)|(LCD_Dat&0x00ff);          //save the 8 high pins of C and the 8 low pins of LCD_Reg to the output register
    //GPIOB->ODR = (GPIOB->ODR&0x00ff)|(LCD_Dat&0xff00);         //save the 8 high pins of C and the 8 low pins of LCD_Reg to the output register

    //sending the Data to the driver
    ttc_gpio_set_u8_pin3(PARALLEL08_2_BANK, PARALLEL08_2_FIRSTPIN, (0x00ff & LCD_Dat));
    ttc_gpio_set_u8_pin3(PARALLEL08_1_BANK, PARALLEL08_1_FIRSTPIN, (0xff00 & LCD_Dat)>>8);

    CLR_LCD_WR; //renew the Write mode -> clock count
    SET_LCD_WR;
    SET_LCD_CS; //push LCD_CS to vcc -> Chip-Select is deactivated
}
void ili9320_delay(u32_t nCount) {
    //Sleep function
    mSleep(nCount);
}
void ili9320_line_horizontal(u16_t X,u16_t Y,s16_t Lenght)
{
    GFX_STATIC u32_t Count;

 ili9320_set_cursor(X,Y);
 for(Count=0;Count<=Lenght;Count++)
 {
    ili9320_write_data(ttc_gfx_CurrentDisplay->ColorFg);
 }
}

void ili9320_line_vertical(u16_t X,u16_t Y,s16_t Lenght)
{
    GFX_STATIC u32_t Count;
 ili9320_write_register(0x03,0x1038);
 ili9320_set_cursor(X,Y);
 for(Count=0;Count<=Lenght;Count++)
 {
    ili9320_write_data(ttc_gfx_CurrentDisplay->ColorFg);
 }
 ili9320_write_register(0x03,0x1030);
}

