/*{ lcd_ili9320.h *******************************

  Low-level driver for LCD Controller ILI9320.

}*/

#ifndef IL9320
#define IL9320

//{ Includes 1 ***********************************************************

#include "ttc_basic.h"

//}Includes 1
//{ Defines 1 *************************************************************

//}Defines 1
//{ Includes *************************************************************

#include "ttc_gpio.h"
#include "ttc_task.h"
#include "ttc_interrupt.h"
#include "ttc_memory.h"
#include "ttc_gfx_types.h"

//X #include "ttc_watchdog.h" // ToDo: required?
//X #include "pin_configuration.h"
//X #include "lcd_gfximage.h"
//X #include "lcd_color.h"
//X #include "ttc_ads7843.h"
//X #include <stdio.h>

//}Includes
//{ externals provided by ttc_gfx.c *************************************

extern ttc_gfx_generic_t* ttc_gfx_CurrentDisplay;
extern u16_t ttc_gfx_CurrentColorFg;
extern u16_t ttc_gfx_CurrentColorBg;

//}externals
//{ Defines *************************************************************

//LCD measurements

#ifndef LCD_LI9320_WIDTH
#define LCD_LI9320_WIDTH 240
#endif
#ifndef LCD_LI9320_HEIGHT
#define LCD_LI9320_HEIGHT 320
#endif
#ifndef LCD_LI9320_DEPTH
#define LCD_LI9320_DEPTH 16 //in the 16-bit 1-transfer mode, 16-bit 2-transfer mode initialisation possible
#endif


#define BUTTONHEIGHT 18
#define BUTTONWIDTH BUTTONHEIGHT*3
#define MEDIUMLENGHT 10

#define ili9320_write_register(LCD_Reg,LCD_Dat) ili9320_write_command(LCD_Reg);ili9320_write_data(LCD_Dat)
#define ili9320_put_pixel()   ili9320_write_command(0x22);ili9320_write_data(ttc_gfx_CurrentColorFg)



#ifdef EXTENSION_100_board_mini_stm32_rev20
#ifdef TARGET_ARCHITECTURE_STM32
#define CLR_LCD_CS *(stm32_calc_peripheral_BitBandAddress( (void*) &(((GPIO_TypeDef *) GPIOC)->BRR),  8)) = 1  //LCD Chip Select
#define CLR_LCD_RS *(stm32_calc_peripheral_BitBandAddress( (void*) &(((GPIO_TypeDef *) GPIOC)->BRR),  9)) = 1  //LCD Reset Command
#define CLR_LCD_WR *(stm32_calc_peripheral_BitBandAddress( (void*) &(((GPIO_TypeDef *) GPIOC)->BRR),  10)) = 1 //LCD Write Command Bit
#define CLR_LCD_RD *(stm32_calc_peripheral_BitBandAddress( (void*) &(((GPIO_TypeDef *) GPIOC)->BRR),  11)) = 1 //LCD Read Command Bit
#define SET_LCD_CS *(stm32_calc_peripheral_BitBandAddress( (void*) &(((GPIO_TypeDef *) GPIOC)->BSRR), 8)) = 1  //LCD Chip Select
#define SET_LCD_RS *(stm32_calc_peripheral_BitBandAddress( (void*) &(((GPIO_TypeDef *) GPIOC)->BSRR), 9)) = 1  //LCD Reset Command
#define SET_LCD_WR *(stm32_calc_peripheral_BitBandAddress( (void*) &(((GPIO_TypeDef *) GPIOC)->BSRR), 10)) = 1 //LCD Write Command Bit
#define SET_LCD_RD *(stm32_calc_peripheral_BitBandAddress( (void*) &(((GPIO_TypeDef *) GPIOC)->BSRR), 11)) = 1 //LCD Read Command Bit
#else
#define CLR_LCD_CS ttc_gpio_clr(PIN_PC8)  //LCD Chip Select
#define CLR_LCD_RS ttc_gpio_clr(PIN_PC9)  //LCD Reset Command
#define CLR_LCD_WR ttc_gpio_clr(PIN_PC10) //LCD Write Command Bit
#define CLR_LCD_RD ttc_gpio_clr(PIN_PC11) //LCD Read Command Bit
#define SET_LCD_CS ttc_gpio_set(PIN_PC8)  //LCD Chip Select
#define SET_LCD_RS ttc_gpio_set(PIN_PC9)  //LCD Reset Command
#define SET_LCD_WR ttc_gpio_set(PIN_PC10) //LCD Write Command Bit
#define SET_LCD_RD ttc_gpio_set(PIN_PC11) //LCD Read Command Bit
#endif
#endif

#ifdef EXTENSION_100_board_mini_stm32_rev30
#ifdef TARGET_ARCHITECTURE_STM32
#define CLR_LCD_CS *(stm32_calc_peripheral_BitBandAddress( (void*) &(((GPIO_TypeDef *) GPIOC)->BRR),  8)) = 1  //LCD Chip Select
#define CLR_LCD_RS *(stm32_calc_peripheral_BitBandAddress( (void*) &(((GPIO_TypeDef *) GPIOC)->BRR),  9)) = 1  //LCD Reset Command
#define CLR_LCD_WR *(stm32_calc_peripheral_BitBandAddress( (void*) &(((GPIO_TypeDef *) GPIOC)->BRR),  10)) = 1 //LCD Write Command Bit
#define CLR_LCD_RD *(stm32_calc_peripheral_BitBandAddress( (void*) &(((GPIO_TypeDef *) GPIOC)->BRR),  11)) = 1 //LCD Read Command Bit
#define SET_LCD_CS *(stm32_calc_peripheral_BitBandAddress( (void*) &(((GPIO_TypeDef *) GPIOC)->BSRR), 8)) = 1  //LCD Chip Select
#define SET_LCD_RS *(stm32_calc_peripheral_BitBandAddress( (void*) &(((GPIO_TypeDef *) GPIOC)->BSRR), 9)) = 1  //LCD Reset Command
#define SET_LCD_WR *(stm32_calc_peripheral_BitBandAddress( (void*) &(((GPIO_TypeDef *) GPIOC)->BSRR), 10)) = 1 //LCD Write Command Bit
#define SET_LCD_RD *(stm32_calc_peripheral_BitBandAddress( (void*) &(((GPIO_TypeDef *) GPIOC)->BSRR), 11)) = 1 //LCD Read Command Bit
#else
#define CLR_LCD_CS ttc_gpio_clr(PIN_PC8)  //LCD Chip Select
#define CLR_LCD_RS ttc_gpio_clr(PIN_PC9)  //LCD Reset Command
#define CLR_LCD_WR ttc_gpio_clr(PIN_PC10) //LCD Write Command Bit
#define CLR_LCD_RD ttc_gpio_clr(PIN_PC11) //LCD Read Command Bit
#define SET_LCD_CS ttc_gpio_set(PIN_PC8)  //LCD Chip Select
#define SET_LCD_RS ttc_gpio_set(PIN_PC9)  //LCD Reset Command
#define SET_LCD_WR ttc_gpio_set(PIN_PC10) //LCD Write Command Bit
#define SET_LCD_RD ttc_gpio_set(PIN_PC11) //LCD Read Command Bit
#endif
#endif

#ifdef EXTENSION_100_board_mini_stm32_rev40
#ifdef TARGET_ARCHITECTURE_STM32
#define CLR_LCD_CS *(stm32_calc_peripheral_BitBandAddress( (void*) &(((GPIO_TypeDef *) GPIOC)->BRR),  9)) = 1  //LCD Chip Select
#define CLR_LCD_RS *(stm32_calc_peripheral_BitBandAddress( (void*) &(((GPIO_TypeDef *) GPIOC)->BRR),  8)) = 1  //LCD Reset Command
#define CLR_LCD_WR *(stm32_calc_peripheral_BitBandAddress( (void*) &(((GPIO_TypeDef *) GPIOC)->BRR),  10)) = 1 //LCD Write Command Bit
#define CLR_LCD_RD *(stm32_calc_peripheral_BitBandAddress( (void*) &(((GPIO_TypeDef *) GPIOC)->BRR),  11)) = 1 //LCD Read Command Bit
#define SET_LCD_CS *(stm32_calc_peripheral_BitBandAddress( (void*) &(((GPIO_TypeDef *) GPIOC)->BSRR), 9)) = 1  //LCD Chip Select
#define SET_LCD_RS *(stm32_calc_peripheral_BitBandAddress( (void*) &(((GPIO_TypeDef *) GPIOC)->BSRR), 8)) = 1  //LCD Reset Command
#define SET_LCD_WR *(stm32_calc_peripheral_BitBandAddress( (void*) &(((GPIO_TypeDef *) GPIOC)->BSRR), 10)) = 1 //LCD Write Command Bit
#define SET_LCD_RD *(stm32_calc_peripheral_BitBandAddress( (void*) &(((GPIO_TypeDef *) GPIOC)->BSRR), 11)) = 1 //LCD Read Command Bit
#else
#define CLR_LCD_CS ttc_gpio_clr(PIN_PC9)  //LCD Chip Select
#define CLR_LCD_RS ttc_gpio_clr(PIN_PC8)  //LCD Reset Command
#define CLR_LCD_WR ttc_gpio_clr(PIN_PC10) //LCD Write Command Bit
#define CLR_LCD_RD ttc_gpio_clr(PIN_PC11) //LCD Read Command Bit
#define SET_LCD_CS ttc_gpio_set(PIN_PC9)  //LCD Chip Select
#define SET_LCD_RS ttc_gpio_set(PIN_PC8)  //LCD Reset Command
#define SET_LCD_WR ttc_gpio_set(PIN_PC10) //LCD Write Command Bit
#define SET_LCD_RD ttc_gpio_set(PIN_PC11) //LCD Read Command Bit
#endif
#endif

//}Defines
//{ Function prototypes **************************************************

// initialize the LCD , white Background
void ili9320_init();

/* Send the information to the LCD-driver which Reg is asked */
// Write command to the TFT-driver which paramter will be read
void ili9320_write_command(u16_t LCD_Reg);

// Reads the Reg-ID of the TFT-driver
u16_t ili9320_read_register(u16_t LCD_Reg);

//void ili9320_write_register(u16_t LCD_Reg,u16_t LCD_Dat);

void ili9320_delay (uint32_t nCount);

void ili9320_clear();

void ili9320_write_data(uint16_t LCD_Dat);

void ili9320_set_color_fg8(u8_t Color);

void ili9320_set_color_fg24(u32_t Color);

void ili9320_set_color_bg24(u32_t Color);

/** sets single pixel at current cursor position in forgeground color */


/** sets single pixel at current cursor position in background color */
void ili9320_clr_pixel();


void ili9320_put_string_solid(const char *String);

/** sets level of backlight
  *
  * @param Config  configuration data of display to use
  * @param Level   Brightness level (0 = off, 255 = max)
  */
void ili9320_set_backlight(u8_t Level);

void ili9320_set_cursor(u16_t X,u16_t Y);

void ili9320_set_display_window(uint8_t Xpos, uint16_t Ypos, uint8_t Height, uint16_t Width);

void ili9320_rect_fill(u16_t X, u16_t Y, s16_t W, s16_t H);

void ili9320_get_configuration();

void ili9320_line_horizontal(u16_t X, u16_t Y, s16_t Lenght);

void ili9320_line_vertical(u16_t X,u16_t Y,s16_t Lenght);

#ifdef EXTENSION_500_ttc_font  // font support available

void ili9320_put_char(u8_t Char);
void ili9320_put_char_solid(u8_t Character);

#endif

//}Function prototypes
//{ Macro support for ttc_gfx.h

#define _ttc_gfx_driver_set_pixel_at(X,Y)  ili9320_set_cursor(X, Y); ili9320_put_pixel()
#define _ttc_gfx_driver_clr_pixel_at(X,Y)  ili9320_set_cursor(X, Y); ili9320_clr_pixel()
#define _ttc_gfx_driver_set_cursor(X,Y)    ili9320_set_cursor(X, Y)

//}Macros
#endif // IL9320
