/*{ radio_cc1120_registers.h ********************************************************

 This source is published under the GNU LESSER GENERAL PUBLIC LICENSE (LGPL).
 See file LEGAL.txt in this software package for details.

 Authors: <Sascha Poggemann>, <Gregor Rebel>

 Description: Register definitions and structures of spi-controlled cc1120 radio transceiver

}*/
#ifndef RADIO_CC1120_REGISTERS_H
#define RADIO_CC1120_REGISTERS_H

//{ register address definitions ***************************************************

//{ Configuration Registers
typedef enum {

    //ToDo: due to 0x2Fxx very large enum maybe change to defines.

    /* configuration registers */
    rcr_CC112X_IOCFG3             =  0x0000,
    rcr_CC112X_IOCFG2             =  0x0001,
    rcr_CC112X_IOCFG1             =  0x0002,
    rcr_CC112X_IOCFG0             =  0x0003,
    rcr_CC112X_SYNC3              =  0x0004,
    rcr_CC112X_SYNC2              =  0x0005,
    rcr_CC112X_SYNC1              =  0x0006,
    rcr_CC112X_SYNC0              =  0x0007,
    rcr_CC112X_SYNC_CFG1          =  0x0008,
    rcr_CC112X_SYNC_CFG0          =  0x0009,
    rcr_CC112X_DEVIATION_M        =  0x000A,
    rcr_CC112X_MODCFG_DEV_E       =  0x000B,
    rcr_CC112X_DCFILT_CFG         =  0x000C,
    rcr_CC112X_PREAMBLE_CFG1      =  0x000D,
    rcr_CC112X_PREAMBLE_CFG0      =  0x000E,
    rcr_CC112X_FREQ_IF_CFG        =  0x000F,
    rcr_CC112X_IQIC               =  0x0010,
    rcr_CC112X_CHAN_BW            =  0x0011,
    rcr_CC112X_MDMCFG1            =  0x0012,
    rcr_CC112X_MDMCFG0            =  0x0013,
    rcr_CC112X_DRATE2             =  0x0014,
    rcr_CC112X_DRATE1             =  0x0015,
    rcr_CC112X_DRATE0             =  0x0016,
    rcr_CC112X_AGC_REF            =  0x0017,
    rcr_CC112X_AGC_CS_THR         =  0x0018,
    rcr_CC112X_AGC_GAIN_ADJUST    =  0x0019,
    rcr_CC112X_AGC_CFG3           =  0x001A,
    rcr_CC112X_AGC_CFG2           =  0x001B,
    rcr_CC112X_AGC_CFG1           =  0x001C,
    rcr_CC112X_AGC_CFG0           =  0x001D,
    rcr_CC112X_FIFO_CFG           =  0x001E,
    rcr_CC112X_DEV_ADDR           =  0x001F,
    rcr_CC112X_SETTLING_CFG       =  0x0020,
    rcr_CC112X_FS_CFG             =  0x0021,
    rcr_CC112X_WOR_CFG1           =  0x0022,
    rcr_CC112X_WOR_CFG0           =  0x0023,
    rcr_CC112X_WOR_EVENT0_MSB     =  0x0024,
    rcr_CC112X_WOR_EVENT0_LSB     =  0x0025,
    rcr_CC112X_PKT_CFG2           =  0x0026,
    rcr_CC112X_PKT_CFG1           =  0x0027,
    rcr_CC112X_PKT_CFG0           =  0x0028,
    rcr_CC112X_RFEND_CFG1         =  0x0029,
    rcr_CC112X_RFEND_CFG0         =  0x002A,
    rcr_CC112X_PA_CFG2            =  0x002B,
    rcr_CC112X_PA_CFG1            =  0x002C,
    rcr_CC112X_PA_CFG0            =  0x002D,
    rcr_CC112X_PKT_LEN            =  0x002E,

    /* Extended Configuration Registers */
    rcr_CC112X_IF_MIX_CFG         =  0x2F00,
    rcr_CC112X_FREQOFF_CFG        =  0x2F01,
    rcr_CC112X_TOC_CFG            =  0x2F02,
    rcr_CC112X_MARC_SPARE         =  0x2F03,
    rcr_CC112X_ECG_CFG            =  0x2F04,
    rcr_CC112X_SOFT_TX_DATA_CFG   =  0x2F05,
    rcr_CC112X_EXT_CTRL           =  0x2F06,
    rcr_CC112X_RCCAL_FINE         =  0x2F07,
    rcr_CC112X_RCCAL_COARSE       =  0x2F08,
    rcr_CC112X_RCCAL_OFFSET       =  0x2F09,
    rcr_CC112X_FREQOFF1           =  0x2F0A,
    rcr_CC112X_FREQOFF0           =  0x2F0B,
    rcr_CC112X_FREQ2              =  0x2F0C,
    rcr_CC112X_FREQ1              =  0x2F0D,
    rcr_CC112X_FREQ0              =  0x2F0E,
    rcr_CC112X_IF_ADC2            =  0x2F0F,
    rcr_CC112X_IF_ADC1            =  0x2F10,
    rcr_CC112X_IF_ADC0            =  0x2F11,
    rcr_CC112X_FS_DIG1            =  0x2F12,
    rcr_CC112X_FS_DIG0            =  0x2F13,
    rcr_CC112X_FS_CAL3            =  0x2F14,
    rcr_CC112X_FS_CAL2            =  0x2F15,
    rcr_CC112X_FS_CAL1            =  0x2F16,
    rcr_CC112X_FS_CAL0            =  0x2F17,
    rcr_CC112X_FS_CHP             =  0x2F18,
    rcr_CC112X_FS_DIVTWO          =  0x2F19,
    rcr_CC112X_FS_DSM1            =  0x2F1A,
    rcr_CC112X_FS_DSM0            =  0x2F1B,
    rcr_CC112X_FS_DVC1            =  0x2F1C,
    rcr_CC112X_FS_DVC0            =  0x2F1D,
    rcr_CC112X_FS_LBI             =  0x2F1E,
    rcr_CC112X_FS_PFD             =  0x2F1F,
    rcr_CC112X_FS_PRE             =  0x2F20,
    rcr_CC112X_FS_REG_DIV_CML     =  0x2F21,
    rcr_CC112X_FS_SPARE           =  0x2F22,
    rcr_CC112X_FS_VCO4            =  0x2F23,
    rcr_CC112X_FS_VCO3            =  0x2F24,
    rcr_CC112X_FS_VCO2            =  0x2F25,
    rcr_CC112X_FS_VCO1            =  0x2F26,
    rcr_CC112X_FS_VCO0            =  0x2F27,
    rcr_CC112X_GBIAS6             =  0x2F28,
    rcr_CC112X_GBIAS5             =  0x2F29,
    rcr_CC112X_GBIAS4             =  0x2F2A,
    rcr_CC112X_GBIAS3             =  0x2F2B,
    rcr_CC112X_GBIAS2             =  0x2F2C,
    rcr_CC112X_GBIAS1             =  0x2F2D,
    rcr_CC112X_GBIAS0             =  0x2F2E,
    rcr_CC112X_IFAMP              =  0x2F2F,
    rcr_CC112X_LNA                =  0x2F30,
    rcr_CC112X_RXMIX              =  0x2F31,
    rcr_CC112X_XOSC5              =  0x2F32,
    rcr_CC112X_XOSC4              =  0x2F33,
    rcr_CC112X_XOSC3              =  0x2F34,
    rcr_CC112X_XOSC2              =  0x2F35,
    rcr_CC112X_XOSC1              =  0x2F36,
    rcr_CC112X_XOSC0              =  0x2F37,
    rcr_CC112X_ANALOG_SPARE       =  0x2F38,
    rcr_CC112X_PA_CFG3            =  0x2F39,
    rcr_CC112X_IRQ0M              =  0x2F3F,
    rcr_CC112X_IRQ0F              =  0x2F40,

    /* Status Registers */
    rcr_CC112X_WOR_TIME1          =  0x2F64,
    rcr_CC112X_WOR_TIME0          =  0x2F65,
    rcr_CC112X_WOR_CAPTURE1       =  0x2F66,
    rcr_CC112X_WOR_CAPTURE0       =  0x2F67,
    rcr_CC112X_BIST               =  0x2F68,
    rcr_CC112X_DCFILTOFFSET_I1    =  0x2F69,
    rcr_CC112X_DCFILTOFFSET_I0    =  0x2F6A,
    rcr_CC112X_DCFILTOFFSET_Q1    =  0x2F6B,
    rcr_CC112X_DCFILTOFFSET_Q0    =  0x2F6C,
    rcr_CC112X_IQIE_I1            =  0x2F6D,
    rcr_CC112X_IQIE_I0            =  0x2F6E,
    rcr_CC112X_IQIE_Q1            =  0x2F6F,
    rcr_CC112X_IQIE_Q0            =  0x2F70,
    rcr_CC112X_RSSI1              =  0x2F71,
    rcr_CC112X_RSSI0              =  0x2F72,
    rcr_CC112X_MARCSTATE          =  0x2F73,
    rcr_CC112X_LQI_VAL            =  0x2F74,
    rcr_CC112X_PQT_SYNC_ERR       =  0x2F75,
    rcr_CC112X_DEM_STATUS         =  0x2F76,
    rcr_CC112X_FREQOFF_EST1       =  0x2F77,
    rcr_CC112X_FREQOFF_EST0       =  0x2F78,
    rcr_CC112X_AGC_GAIN3          =  0x2F79,
    rcr_CC112X_AGC_GAIN2          =  0x2F7A,
    rcr_CC112X_AGC_GAIN1          =  0x2F7B,
    rcr_CC112X_AGC_GAIN0          =  0x2F7C,
    rcr_CC112X_SOFT_RX_DATA_OUT   =  0x2F7D,
    rcr_CC112X_SOFT_TX_DATA_IN    =  0x2F7E,
    rcr_CC112X_ASK_SOFT_RX_DATA   =  0x2F7F,
    rcr_CC112X_RNDGEN             =  0x2F80,
    rcr_CC112X_MAGN2              =  0x2F81,
    rcr_CC112X_MAGN1              =  0x2F82,
    rcr_CC112X_MAGN0              =  0x2F83,
    rcr_CC112X_ANG1               =  0x2F84,
    rcr_CC112X_ANG0               =  0x2F85,
    rcr_CC112X_CHFILT_I2          =  0x2F86,
    rcr_CC112X_CHFILT_I1          =  0x2F87,
    rcr_CC112X_CHFILT_I0          =  0x2F88,
    rcr_CC112X_CHFILT_Q2          =  0x2F89,
    rcr_CC112X_CHFILT_Q1          =  0x2F8A,
    rcr_CC112X_CHFILT_Q0          =  0x2F8B,
    rcr_CC112X_GPIO_STATUS        =  0x2F8C,
    rcr_CC112X_FSCAL_CTRL         =  0x2F8D,
    rcr_CC112X_PHASE_ADJUST       =  0x2F8E,
    rcr_CC112X_PARTNUMBER         =  0x2F8F,
    rcr_CC112X_PARTVERSION        =  0x2F90,
    rcr_CC112X_SERIAL_STATUS      =  0x2F91,
    rcr_CC112X_RX_STATUS          =  0x2F92,
    rcr_CC112X_TX_STATUS          =  0x2F93,
    rcr_CC112X_MARC_STATUS1       =  0x2F94,
    rcr_CC112X_MARC_STATUS0       =  0x2F95,
    rcr_CC112X_PA_IFAMP_TEST      =  0x2F96,
    rcr_CC112X_FSRF_TEST          =  0x2F97,
    rcr_CC112X_PRE_TEST           =  0x2F98,
    rcr_CC112X_PRE_OVR            =  0x2F99,
    rcr_CC112X_ADC_TEST           =  0x2F9A,
    rcr_CC112X_DVC_TEST           =  0x2F9B,
    rcr_CC112X_ATEST              =  0x2F9C,
    rcr_CC112X_ATEST_LVDS         =  0x2F9D,
    rcr_CC112X_ATEST_MODE         =  0x2F9E,
    rcr_CC112X_XOSC_TEST1         =  0x2F9F,
    rcr_CC112X_XOSC_TEST0         =  0x2FA0 ,

    rcr_CC112X_RXFIRST            =  0x2FD2,
    rcr_CC112X_TXFIRST            =  0x2FD3,
    rcr_CC112X_RXLAST             =  0x2FD4,
    rcr_CC112X_TXLAST             =  0x2FD5,
    rcr_CC112X_NUM_TXBYTES        =  0x2FD6,  /* Number of bytes in TXFIFO */
    rcr_CC112X_NUM_RXBYTES        =  0x2FD7,  /* Number of bytes in RXFIFO */
    rcr_CC112X_FIFO_NUM_TXBYTES   =  0x2FD8,
    rcr_CC112X_FIFO_NUM_RXBYTES   =  0x2FD9

} radio_cc1120_register_e;






/* DATA FIFO Access */
#define CC112X_SINGLE_TXFIFO            0x003F      /*  TXFIFO  - Single accecss to Transmit FIFO */
#define CC112X_BURST_TXFIFO             0x007F      /*  TXFIFO  - Burst accecss to Transmit FIFO  */
#define CC112X_SINGLE_RXFIFO            0x00BF      /*  RXFIFO  - Single accecss to Receive FIFO  */
#define CC112X_BURST_RXFIFO             0x00FF      /*  RXFIFO  - Busrrst ccecss to Receive FIFO  */

#define CC112X_LQI_CRC_OK_BM            0x80
#define CC112X_LQI_EST_BM               0x7F



/* Command strobe registers */
#define CC112X_SRES                     0x30      /*  SRES    - Reset chip. */
#define CC112X_SFSTXON                  0x31      /*  SFSTXON - Enable and calibrate frequency synthesizer. */
#define CC112X_SXOFF                    0x32      /*  SXOFF   - Turn off crystal oscillator. */
#define CC112X_SCAL                     0x33      /*  SCAL    - Calibrate frequency synthesizer and turn it off. */
#define CC112X_SRX                      0x34      /*  SRX     - Enable RX. Perform calibration if enabled. */
#define CC112X_STX                      0x35      /*  STX     - Enable TX. If in RX state, only enable TX if CCA passes. */
#define CC112X_SIDLE                    0x36      /*  SIDLE   - Exit RX / TX, turn off frequency synthesizer. */
#define CC112X_SWOR                     0x38      /*  SWOR    - Start automatic RX polling sequence (Wake-on-Radio) */
#define CC112X_SPWD                     0x39      /*  SPWD    - Enter power down mode when CSn goes high. */
#define CC112X_SFRX                     0x3A      /*  SFRX    - Flush the RX FIFO buffer. */
#define CC112X_SFTX                     0x3B      /*  SFTX    - Flush the TX FIFO buffer. */
#define CC112X_SWORRST                  0x3C      /*  SWORRST - Reset real time clock. */
#define CC112X_SNOP                     0x3D      /*  SNOP    - No operation. Returns status byte. */
#define CC112X_AFC                      0x37      /*  AFC     - Automatic Frequency Correction */

/* Chip states returned in status byte */
#define CC112X_STATE_IDLE               0x00
#define CC112X_STATE_RX                 0x10
#define CC112X_STATE_TX                 0x20
#define CC112X_STATE_FSTXON             0x30
#define CC112X_STATE_CALIBRATE          0x40
#define CC112X_STATE_SETTLING           0x50
#define CC112X_STATE_RXFIFO_ERROR       0x60
#define CC112X_STATE_TXFIFO_ERROR       0x70


//} register content definitions

#endif //RADIO_CC1120_REGISTERS_H
