#ifndef STM32_GPIO_H
#define STM32_GPIO_H

/*{ stm32_gpio.h ***********************************************
 
 * Written by Gregor Rebel 2010-2012
 *
 * Low Level Driver for General Purpose Input Output (GPIO) on STM32 architecture.
 * 
}*/
//{ includes1

#ifndef EXTENSION_250_stm_std_peripherals
  #error EXTENSION_250_stm_std_peripherals not defined -> activate.200_cpu_stm32f10x_std_peripherals.sh
#endif

#include "ttc_basic.h"
#include "stm32_registers.h"
#include "stm32f10x_gpio.h"
#include "stm32f10x_rcc.h"

//}includes
//{ compatibility interface for ttc_gpio.h

/* Macro Arguments
 *
 * PORT      pointer to type ttc_Port_t
 * BANK      pointer to type ttc_GPIO_BANK
 * PIN       0..15 pin number in GPIO bank
 * FIRST_PIN 0..8 pin number of first pin of am 8-bit wide parallel port
 * TYPE      type to configure pin to  (one from ttc_gpio_types.h:ttc_gpio_mode_e)
 * SPEED     speed to configure pin to (one from ttc_gpio_types.h:ttc_gpio_speed_e)
 * BANK_PIN  BANK,PIN  (allows use of PIN_Pxn definitions like PIN_A1)
 * INDEX     u8_t Value storing GPIO-Bank index (0..15) and Pin index (0..15) in high- and low-nibble
 *
 */

#define ttc_gpio_get_variable(PORT)                        stm32_gpio_get_variable(PORT)
#define ttc_gpio_set_variable(PORT)                        stm32_gpio_set_variable(PORT)
#define ttc_gpio_clr_variable(PORT)                        stm32_gpio_clr_variable(PORT)
#define ttc_gpio_variable(PORT,BANK_PIN)                   stm32_gpio_variable(PORT,BANK_PIN)
#define ttc_gpio_set(BANK_PIN)                             stm32_gpio_set(BANK_PIN)
#define ttc_gpio_get(BANK_PIN)                             stm32_gpio_get(BANK_PIN)
#define ttc_gpio_clr(BANK_PIN)                             stm32_gpio_clr(BANK_PIN)
#define ttc_gpio_set2(BANK,PIN)                            stm32_gpio_set(BANK,PIN)
#define ttc_gpio_get2(BANK,PIN)                            stm32_gpio_get(BANK,PIN)
#define ttc_gpio_clr2(BANK,PIN)                            stm32_gpio_clr(BANK,PIN)
#define ttc_gpio_create_index8(BANK_PIN)                   stm32_gpio_create_index8(BANK_PIN)
#define ttc_gpio_from_index8(INDEX,BANK_PIN)               stm32_gpio_from_index8(INDEX,BANK_PIN)

//#define ttc_gpio_init_u8_pin(BANK_PIN,TYPE,SPEED)          stm32_gpio_init_u8_pin(BANK_PIN,TYPE,SPEED)
//#define ttc_gpio_init_u8_pin4(BANK,FIRST_PIN,TYPE,SPEED)   stm32_gpio_init_u8_pin4(BANK,FIRST_PIN,TYPE,SPEED)
#define ttc_gpio_get_u8_pin(BANK_PIN)                      stm32_gpio_get_u8_pin(BANK_PIN)
#define ttc_gpio_get_u8_pin2(BANK,FIRST_PIN)               stm32_gpio_get_u8_pin2(BANK,FIRST_PIN)
#define ttc_gpio_set_u8_pin(BANK_PIN,VALUE)                stm32_gpio_set_u8_pin(BANK_PIN,VALUE)
#define ttc_gpio_set_u8_pin3(BANK,FIRST_PIN,VALUE)         stm32_gpio_set_u8_pin3(BANK,FIRST_PIN,VALUE)
//#define ttc_gpio_init_u16(BANK,TYPE,SPEED)                 stm32_gpio_init_u16(BANK,TYPE,SPEED)
#define ttc_gpio_get_u16(BANK)                             stm32_gpio_get_u16(BANK)
#define ttc_gpio_set_u16(BANK,VALUE)                       stm32_gpio_set_u16(BANK,VALUE)

//}compatibility interface for ttc_gpio.h
//{ Structures/ Enums

typedef enum {
    stm32_GPIO_None,
    stm32_GPIOA = (u32_t) GPIOA,
    stm32_GPIOB = (u32_t) GPIOB,
    stm32_GPIOC = (u32_t) GPIOC,
    stm32_GPIOD = (u32_t) GPIOD,
    stm32_GPIOE = (u32_t) GPIOE,
    stm32_GPIOF = (u32_t) GPIOF,
    stm32_GPIOG = (u32_t) GPIOG,
    stm32_GPIO_NotImplemented
} stm32_GPIO_e;
typedef struct { // stm32_Port_t
    stm32_GPIO_e GPIOx;
    u8_t Pin;
    volatile u32_t* BSRR;  // bit-band address of BSRR register of this port bit
    volatile u32_t* BRR;   // bit-band address of BRR register of this port bit
    volatile u32_t* IDR;   // bit-band address of IDR register of this port bit
} __attribute__((__packed__)) stm32_Port_t;

//}Structures/ Enums
//{ Architecture-Defines (required by ttc_gpio_types.h)

// amount of gpio banks (A..G)
#define TTC_GPIO_MAX_AMOUNT 7

// amount of port pins per bank
#define TTC_GPIO_MAX_PINS 16

#define ttc_GPIO_BANK  stm32_GPIO_e
#define ttc_Port_t     stm32_Port_t

#define GPIO_BANK_A stm32_GPIOA
#define GPIO_BANK_B stm32_GPIOB
#define GPIO_BANK_C stm32_GPIOC
#define GPIO_BANK_D stm32_GPIOD
#define GPIO_BANK_E stm32_GPIOE
#ifdef GPIOF
#define GPIO_BANK_F stm32_GPIOF
#endif
#ifdef GPIOG
#define GPIO_BANK_G stm32_GPIOG
#endif

//{ Define shortcuts for all pins of GPIO Bank A
#define PIN_PA0  GPIO_BANK_A,0
#define PIN_PA1  GPIO_BANK_A,1
#define PIN_PA2  GPIO_BANK_A,2
#define PIN_PA3  GPIO_BANK_A,3
#define PIN_PA4  GPIO_BANK_A,4
#define PIN_PA5  GPIO_BANK_A,5
#define PIN_PA6  GPIO_BANK_A,6
#define PIN_PA7  GPIO_BANK_A,7
#define PIN_PA8  GPIO_BANK_A,8
#define PIN_PA9  GPIO_BANK_A,9
#define PIN_PA10 GPIO_BANK_A,10
#define PIN_PA11 GPIO_BANK_A,11
#define PIN_PA12 GPIO_BANK_A,12
#define PIN_PA13 GPIO_BANK_A,13
#define PIN_PA14 GPIO_BANK_A,14
#define PIN_PA15 GPIO_BANK_A,15
//}BankA
//{ Define shortcuts for all pins of GPIO Bank B
#define PIN_PB0  GPIO_BANK_B,0
#define PIN_PB1  GPIO_BANK_B,1
#define PIN_PB2  GPIO_BANK_B,2
#define PIN_PB3  GPIO_BANK_B,3
#define PIN_PB4  GPIO_BANK_B,4
#define PIN_PB5  GPIO_BANK_B,5
#define PIN_PB6  GPIO_BANK_B,6
#define PIN_PB7  GPIO_BANK_B,7
#define PIN_PB8  GPIO_BANK_B,8
#define PIN_PB9  GPIO_BANK_B,9
#define PIN_PB10 GPIO_BANK_B,10
#define PIN_PB11 GPIO_BANK_B,11
#define PIN_PB12 GPIO_BANK_B,12
#define PIN_PB13 GPIO_BANK_B,13
#define PIN_PB14 GPIO_BANK_B,14
#define PIN_PB15 GPIO_BANK_B,15
//}BankB
//{ Define shortcuts for all pins of GPIO Bank C
#define PIN_PC0  GPIO_BANK_C,0
#define PIN_PC1  GPIO_BANK_C,1
#define PIN_PC2  GPIO_BANK_C,2
#define PIN_PC3  GPIO_BANK_C,3
#define PIN_PC4  GPIO_BANK_C,4
#define PIN_PC5  GPIO_BANK_C,5
#define PIN_PC6  GPIO_BANK_C,6
#define PIN_PC7  GPIO_BANK_C,7
#define PIN_PC8  GPIO_BANK_C,8
#define PIN_PC9  GPIO_BANK_C,9
#define PIN_PC10 GPIO_BANK_C,10
#define PIN_PC11 GPIO_BANK_C,11
#define PIN_PC12 GPIO_BANK_C,12
#define PIN_PC13 GPIO_BANK_C,13
#define PIN_PC14 GPIO_BANK_C,14
#define PIN_PC15 GPIO_BANK_C,15
//}BankC
//{ Define shortcuts for all pins of GPIO Bank D
#define PIN_PD0  GPIO_BANK_D,0
#define PIN_PD1  GPIO_BANK_D,1
#define PIN_PD2  GPIO_BANK_D,2
#define PIN_PD3  GPIO_BANK_D,3
#define PIN_PD4  GPIO_BANK_D,4
#define PIN_PD5  GPIO_BANK_D,5
#define PIN_PD6  GPIO_BANK_D,6
#define PIN_PD7  GPIO_BANK_D,7
#define PIN_PD8  GPIO_BANK_D,8
#define PIN_PD9  GPIO_BANK_D,9
#define PIN_PD10 GPIO_BANK_D,10
#define PIN_PD11 GPIO_BANK_D,11
#define PIN_PD12 GPIO_BANK_D,12
#define PIN_PD13 GPIO_BANK_D,13
#define PIN_PD14 GPIO_BANK_D,14
#define PIN_PD15 GPIO_BANK_D,15
//}BankD
//{ Define shortcuts for all pins of GPIO Bank E
#define PIN_PE0  GPIO_BANK_E,0
#define PIN_PE1  GPIO_BANK_E,1
#define PIN_PE2  GPIO_BANK_E,2
#define PIN_PE3  GPIO_BANK_E,3
#define PIN_PE4  GPIO_BANK_E,4
#define PIN_PE5  GPIO_BANK_E,5
#define PIN_PE6  GPIO_BANK_E,6
#define PIN_PE7  GPIO_BANK_E,7
#define PIN_PE8  GPIO_BANK_E,8
#define PIN_PE9  GPIO_BANK_E,9
#define PIN_PE10 GPIO_BANK_E,10
#define PIN_PE11 GPIO_BANK_E,11
#define PIN_PE12 GPIO_BANK_E,12
#define PIN_PE13 GPIO_BANK_E,13
#define PIN_PE14 GPIO_BANK_E,14
#define PIN_PE15 GPIO_BANK_E,15
//}BankE
#ifdef GPIO_BANK_F
//{ Define shortcuts for all pins of GPIO Bank F
#define PIN_PF0  GPIO_BANK_F,0
#define PIN_PF1  GPIO_BANK_F,1
#define PIN_PF2  GPIO_BANK_F,2
#define PIN_PF3  GPIO_BANK_F,3
#define PIN_PF4  GPIO_BANK_F,4
#define PIN_PF5  GPIO_BANK_F,5
#define PIN_PF6  GPIO_BANK_F,6
#define PIN_PF7  GPIO_BANK_F,7
#define PIN_PF8  GPIO_BANK_F,8
#define PIN_PF9  GPIO_BANK_F,9
#define PIN_PF10 GPIO_BANK_F,10
#define PIN_PF11 GPIO_BANK_F,11
#define PIN_PF12 GPIO_BANK_F,12
#define PIN_PF13 GPIO_BANK_F,13
#define PIN_PF14 GPIO_BANK_F,14
#define PIN_PF15 GPIO_BANK_F,15
//}BankF
#endif
#ifdef GPIO_BANK_G
//{ Define shortcuts for all pins of GPIO Bank G
#define PIN_PG0  GPIO_BANK_G,0
#define PIN_PG1  GPIO_BANK_G,1
#define PIN_PG2  GPIO_BANK_G,2
#define PIN_PG3  GPIO_BANK_G,3
#define PIN_PG4  GPIO_BANK_G,4
#define PIN_PG5  GPIO_BANK_G,5
#define PIN_PG6  GPIO_BANK_G,6
#define PIN_PG7  GPIO_BANK_G,7
#define PIN_PG8  GPIO_BANK_G,8
#define PIN_PG9  GPIO_BANK_G,9
#define PIN_PG10 GPIO_BANK_G,10
#define PIN_PG11 GPIO_BANK_G,11
#define PIN_PG12 GPIO_BANK_G,12
#define PIN_PG13 GPIO_BANK_G,13
#define PIN_PG14 GPIO_BANK_G,14
#define PIN_PG15 GPIO_BANK_G,15
//}BankG
#endif


//}Defines
//{ includes2

#include "ttc_gpio_types.h"  // must be included AFTER defining some ttc_XXXX thingies
//X #include <stdlib.h>
#include "stm32f10x.h"
#include "core_cm3.h"
#include "stm32f10x_conf.h"
#include "stm32f10x_gpio.h"

// FreeRTOS
#include <string.h>
#include "stddef.h"

//}includes2
//{ Macros

//}Macros
//{ Prototypes

/** converts from architecture independent pin type to stm32 pendant
 * @param    Type   one of tgm_analog_in, tgm_input_floating, tgm_input_pull_down, tgm_input_pull_up, tgm_output_open_drain, tgm_output_push_pull, tgm_alternate_function_push_pull, tgm_alternate_function_open_drain
 * @return   one of GPIO_Mode_AIN, GPIO_Mode_IN_FLOATING, GPIO_Mode_IPD, GPIO_Mode_IPU, GPIO_Mode_Out_OD, GPIO_Mode_Out_PP, GPIO_Mode_Out_OD, GPIO_Mode_AF_PP, GPIO_Mode_AF_OD
 */
GPIOMode_TypeDef stm32_map_Type(ttc_gpio_mode_e Type);

/** converts from architecture independent pin speed to stm32 pendant
 * @param    Type   one of tgs_Min, tgs_2MHz, tgs_10MHz, tgs_50MHz, tgs_Max
 * @return   one of GPIO_Speed_2MHz, GPIO_Speed_10MHz, GPIO_Speed_50MHz
 */
GPIOSpeed_TypeDef stm32_map_Speed(ttc_gpio_speed_e Speed);

/** initializes single port bit for use as input/ output
 * @param Port  filled out struct as being returned by stm32_gpio_variable()
 * @param Type   one of GPIO_Mode_AIN, GPIO_Mode_IN_FLOATING, GPIO_Mode_IPD, GPIO_Mode_IPU, GPIO_Mode_Out_OD, GPIO_Mode_Out_PP, GPIO_Mode_Out_OD, GPIO_Mode_AF_PP, GPIO_Mode_AF_OD
 * @param Speed  one of GPIO_Speed_2MHz, GPIO_Speed_10MHz, GPIO_Speed_50MHz
 */
void stm32_gpio_init_variable(stm32_Port_t* Port, ttc_gpio_mode_e Type, ttc_gpio_speed_e Speed);

/** initializes single port bit for use as input/ output
 * @param GPIOx  one of GPIO_BANK_A, GPIO_BANK_B, GPIO_BANK_C, GPIO_BANK_D, GPIO_BANK_E, GPIO_BANK_F
 * @param Pin    0..15
 * @param Type   one of GPIO_Mode_AIN, GPIO_Mode_IN_FLOATING, GPIO_Mode_IPD, GPIO_Mode_IPU, GPIO_Mode_Out_OD, GPIO_Mode_Out_PP, GPIO_Mode_Out_OD, GPIO_Mode_AF_PP, GPIO_Mode_AF_OD
 * @param Speed  one of GPIO_Speed_2MHz, GPIO_Speed_10MHz, GPIO_Speed_50MHz
 */
void stm32_gpio_init(stm32_GPIO_e GPIOx, u8_t Pin, ttc_gpio_mode_e Type, ttc_gpio_speed_e Speed);

/** allows to load struct from list type definition
 *
 * @param Port   points to structure to be filled with given data
 * @param GPIOx  one of GPIO_BANK_A, GPIO_BANK_B, GPIO_BANK_C, GPIO_BANK_D, GPIO_BANK_E, GPIO_BANK_F
 * @param Pin    0..15
 *
 * Example:
 * #define PB_LED1=GPIO_BANK_C,6
 * stm32_Port_t Port;
 * stm32_gpio_variable(&Port, PB_LED1);
 * stm32_gpio_init_variable(&Port, GPIO_Mode_Out_PP);
 * stm32_gpio_set_variable(&Port);
 */
void stm32_gpio_variable(stm32_Port_t* Port, stm32_GPIO_e GPIOx, u8_t Pin);

/** read current value of input pin
 * @param Port  filled out struct as being returned by stm32_gpio_variable()
 */
bool stm32_gpio_get_variable(stm32_Port_t* Port);

/** set output pin to logical one
 * @param Port  filled out struct as being returned by stm32_gpio_variable()
 */
void stm32_gpio_set_variable(stm32_Port_t* Port);

/** set output pin to logical zero
 * @param Port  filled out struct as being returned by stm32_gpio_variable()
 */
void stm32_gpio_clr_variable(stm32_Port_t* Port);

/** read current value of input pin
 * @param GPIOx  one of GPIO_BANK_A, GPIO_BANK_B, GPIO_BANK_C, GPIO_BANK_D, GPIO_BANK_E, GPIO_BANK_F
 * @param Pin    0..15
 */
bool stm32_gpio_get(stm32_GPIO_e GPIOx, u8_t Pin);

/** set output pin to logical one
 * @param GPIOx  one of GPIO_BANK_A, GPIO_BANK_B, GPIO_BANK_C, GPIO_BANK_D, GPIO_BANK_E, GPIO_BANK_F
 * @param Pin    0..15
 */
void stm32_gpio_clr(stm32_GPIO_e GPIOx, u8_t Pin);

/** set output pin to logical zero
 * @param GPIOx  one of GPIO_BANK_A, GPIO_BANK_B, GPIO_BANK_C, GPIO_BANK_D, GPIO_BANK_E, GPIO_BANK_F
 * @param Pin    0..15
 */
void stm32_gpio_set(stm32_GPIO_e GPIOx, u8_t Pin);

/** Calculate bit-banding address that corresponds to given bit in given 32-bit word
 *
 * @param   PeripheralWord   address of 32-bit word in which desired bit is located
 * @param   BitNumber        0..31
 * @return  address in peripheral bit-banding region that corresponds to given word and bit-number
 */
// volatile u32_t* stm32_calc_peripheral_BitBandAddress(void* PeripheralWord, u8_t BitNumber);
#define stm32_calc_peripheral_BitBandAddress(PeripheralWord, BitNumber) (u32_t*) (0x42000000 + 32 * (((u32_t) PeripheralWord)  - 0x40000000) + BitNumber * 4)


/** creates numeric 8-bit representing given GPIO-bank and pin number
 *
 * @param GPIOx  GPIO bank to use (-> ttc_gpio_types.h:ttc_GPIO_BANK)
 *               Note: GPIOx, Pin can be given as one argument by use of Pin_Pxn macro (GPIO_BANK_A, 7) = Pin_PA7
 * @param Pin    0..15
 * @return       8-bit value representing given GPIO-pin in a memory friendly format
 */
u8_t stm32_gpio_create_index8(ttc_GPIO_BANK GPIOx, u8_t Pin);

/** restores GPIO-bank and pin number from 8-bit value
 *
 * @param PhysicalIndex  8-bit value representing GPIO-bank + pin
 * @param GPIOx          loaded with GPIO bank (-> ttc_gpio_types.h:ttc_GPIO_BANK)
 * @param Pin            loaded with pin number
 */
void stm32_gpio_from_index8(u8_t PhysicalIndex, GPIO_TypeDef** GPIOx, u8_t* Pin);

/** enable/ disable a complete GPIO bank
 *
 * @param Bank     GPIO bank to enable/disable (GPIO_BANKA, GPIO_BANKB, ...)
 */
void stm32_gpio_bank_enable(stm32_GPIO_e GPIOx);
void stm32_gpio_bank_disable(stm32_GPIO_e GPIOx);

/** initialize all pins of 16-bit wide parallel port at once
 *
 * Note: Runtime measured on STM32F107 @72MHz as < 3us
 *
 * @param Bank     GPIO bank to use (GPIO_BANKA, GPIO_BANKB, ...)
 */
void stm32_gpio_init_u16(ttc_GPIO_BANK GPIOx, ttc_gpio_mode_e Type, ttc_gpio_speed_e Speed);

/** read 16 bit value from gpio bank at once (fast access)
 *
 * @param GPIOx     GPIO bank to use (GPIO_BANKA, GPIO_BANKB, ...)
 */
// u16_t stm32_gpio_get_u16(ttc_GPIO_BANK GPIOx);
#define stm32_gpio_get_u16(GPIOx)  ((GPIO_TypeDef *) GPIOx)->IDR

/** write 16 bit value into gpio bank at once (fast access)
 *
 * Note: runtime measured on stm32f107 @72MHz was 240ns
 *
 * @param GPIOx     GPIO bank to use (GPIO_BANKA, GPIO_BANKB, ...)
 */
// void stm32_gpio_set_u16(ttc_GPIO_BANK GPIOx, u16_t Value);
#define stm32_gpio_set_u16(GPIOx, Value)  ((GPIO_TypeDef *) GPIOx)->ODR = Value

/** initialize 8 bit wide parallel port
 *
 * Note: Runtime measured on STM32F107 @72MHz as < 2.2us
 *
 * @param Bank+FirstPin  Bank and Pin given as one argument by use of Pin_Pxn macro: E.g. (GPIOA, 7) = Pin_PA7
 * @param Bank           GPIO bank to use (GPIO_BANKA, GPIO_BANKB, ...)
 * @param FirstPin       0..8  lowest of 8 consecutive pins to use
 * @param Type           mode to use for all 8 pins  (-> ttc_gpio_types.h:ttc_gpio_mode_e)
 * @param Speed          speed to use for all 8 pins (-> ttc_gpio_types.h:ttc_gpio_speed_e)
 */
// defined as macros:
// void stm32_gpio_init_u8_pin4(Bank, FirstPin, Type, Speed);
#define stm32_gpio_init_u8_pin4(BANK, FIRST_PIN, TYPE, SPEED)  stm32_gpio_init_u8_pin(BANK, FIRST_PIN, TYPE, SPEED)
void stm32_gpio_init_u8_pin(ttc_GPIO_BANK GPIOx, u8_t FirstPin, ttc_gpio_mode_e Type, ttc_gpio_speed_e Speed);

/** read 8 bit value starting at certain pin from gpio bank
 *
 * @param GPIOx     GPIO bank to use (GPIO_BANKA, GPIO_BANKB, ...)
 * @param Pin       0..8  lowest of 8 consecutive pins to use
 * @return          bits Pin..Pin+7 from input data register of given gpio bank
 */
#define stm32_gpio_get_u8_pin2(GPIOx, Pin)   (  ( (0xff << (Pin) ) & ((GPIO_TypeDef *) GPIOx)->IDR ) >> (Pin)  ) & 0xff

// function required to break Bank+Pin into two parameters (required by gcc macros)
u8_t stm32_gpio_get_u8_pin(ttc_GPIO_BANK GPIOx, u8_t Pin);

/** write 8 bit value starting at certain pin into gpio bank
 *
 * Note: runtime measured on stm32f107 @72MHz was 890ns
 *
 * @param GPIOx     GPIO bank to use (GPIO_BANKA, GPIO_BANKB, ...)
 * @param Pin       0..8  lowest of 8 consecutive pins to use
 * @param Value     written into bits Pin..Pin+7 of output data register of given gpio bank
 */
#define stm32_gpio_set_u8_pin3(GPIOx, Pin, Value)  ((GPIO_TypeDef *) GPIOx)->ODR = ( (Value) << (Pin) ) | ( (0xffff ^ (0xff << (Pin))) & ((GPIO_TypeDef *) GPIOx)->IDR )

// function required to break Bank+Pin into two parameters (required by gcc macros)
void stm32_gpio_set_u8_pin(ttc_GPIO_BANK GPIOx, u8_t Pin, u16_t Value);

/** compiles 4 bit configuration setting suitable for MODEx+CNFx fields of  GPIOx_CRL_t/ GPIOx_CRH_t
 *
 * Note: This function is private and normally should not be called from outside
 *
 * @param Type           mode to use for all 8 pins  (-> ttc_gpio_types.h:ttc_gpio_mode_e)
 * @param Speed          speed to use for all 8 pins (-> ttc_gpio_types.h:ttc_gpio_speed_e)
 * @return               4 bit value to be written at position corresponding to desired pin into GPIOx->CRL/ GPIOx->CRH
 */
u8_t _stm32_gpio_compile_port_config(ttc_gpio_mode_e Type, ttc_gpio_speed_e Speed);

//} Prototypes

#endif
