/*{ stm32_usb::.c -------------------------------------------------------------
 
                      The ToolChain
                      
   Universal Synchronous Asynchronous Receiver and Transmitters (usbs)
   for ARM CortexM3 microcontrollers.
   Currently tested to comply to STM32F10x types.
   
   written by Gregor Rebel 2012
   
   Note: See ttc_usb.h for description of architecture independent usb implementation.
 
}*/

#include "stm32_usb.h"

// stores pointers to structures of initialized usbs/ UARTs
// Note: index is not real, but logical usb-index as defined by makefile!
//static stm32_usb_architecture_t* stm32_USBs[TTC_AMOUNT_USBS];

#ifdef EXTENSION_400_stm32_usb_fs_host_lib
/** @defgroup USBH_USR_MAIN_Private_Variables
* @{
*/
__ALIGN_BEGIN USB_OTG_CORE_HANDLE           USB_OTG_Core_dev __ALIGN_END ;
__ALIGN_BEGIN USBH_HOST                     USB_Host __ALIGN_END ;

#endif
//{ low-level driver interface (mandatory functions) ----------------------------

ttc_usb_errorcode_e stm32_usb_get_defaults(u8_t PhysicalIndex, ttc_usb_generic_t* USB_Generic) {


    return tusbe_OK;
}
ttc_usb_errorcode_e stm32_usb_get_features(u8_t PhysicalIndex, ttc_usb_generic_t* USB_Generic) {

        
    return tusbe_OK;
}
ttc_usb_errorcode_e stm32_usb_init(ttc_usb_generic_t* USB_Generic) {


#ifdef EXTENSION_400_stm32_usb_fs_device_lib  //ToDo add stm32_usb layer
    if (1) Set_System();
    if (1) Set_USBClock();
    if (1) stm32_usb_interrupt_config(USB_Generic->LogicalIndex);
    if (1) USB_Init();

#elif EXTENSION_400_stm32_usb_fs_host_lib


    /*!< At this stage the microcontroller clock setting is already configured,
    this is done through SystemInit() function which is called from startup
    file (startup_stm32fxxx_xx.s) before to branch to application main.
    To reconfigure the default setting of SystemInit() function, refer to
    system_stm32fxxx.c file
    */

    /* Init Host Library */
    USBH_Init(&USB_OTG_Core_dev,
  #ifdef USE_USB_OTG_FS
              USB_OTG_FS_CORE_ID,
  #else
              USB_OTG_HS_CORE_ID,
  #endif
              &USB_Host,
              &HID_cb,
              &USR_Callbacks);

//    while (1)
//    {
//      /* Host Task handler */
      USBH_Process(&USB_OTG_Core_dev , &USB_Host);//ToDo spawn new task for USB Host processing

#endif

    return tusbe_OK;
}

ttc_usb_errorcode_e stm32_usb_interrupt_config(u8_t PhysicalIndex){

    (void) PhysicalIndex;
    NVIC_InitTypeDef NVIC_InitStructure;

    NVIC_PriorityGroupConfig(NVIC_PriorityGroup_1);

#if defined(STM32L1XX_MD) || defined(STM32L1XX_HD) || defined(STM32L1XX_MD_PLUS)
    NVIC_InitStructure.NVIC_IRQChannel = USB_LP_IRQn;
    NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 1;
    NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
    NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
    NVIC_Init(&NVIC_InitStructure);
    return tusbe_OK;
#elif defined(STM32F10X_CL)
    /* Enable the USB Interrupts */
    NVIC_InitStructure.NVIC_IRQChannel = OTG_FS_IRQn;
    NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 1;
    NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
    NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
    NVIC_Init(&NVIC_InitStructure);
    return tusbe_OK;
#else
    NVIC_InitStructure.NVIC_IRQChannel = USB_LP_CAN1_RX0_IRQn;
    NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 1;
    NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
    NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
    NVIC_Init(&NVIC_InitStructure);
    return tusbe_OK;
#endif /* STM32L1XX_XD */

return tusbe_NotImplemented;
}


ttc_usb_errorcode_e stm32_usb_send_raw(ttc_usb_generic_t * USB_Generic, u8_t* Buffer, Base_t Amount){


#ifdef EXTENSION_400_stm32_usb_fs_device_lib
    //will use the implementation in hw_config.c to send data via usb
    //USB_To_USART_Send_Data(Buffer, (u8_t) Amount);
    UART_To_USB_Send_Data((u8_t *) Buffer, (u8_t) Amount);
#else
    return tusbe_NotImplemented;
#endif
    if(USB_Generic->TxFunction_UCB)
        USB_Generic->TxFunction_UCB((ttc_usb_errorcode_e *) Buffer, (u8_t) Amount);

    return tusbe_OK;
}
ttc_usb_errorcode_e stm32_usb_send_word_blocking(ttc_usb_generic_t* USB_Generic, const u16_t Word) {

    //SERIAL_To_USB_Send_Data(Buffer, (u8_t) Amount);

    return tusbe_OK;
}
ttc_usb_errorcode_e stm32_usb_read_byte_blocking(ttc_usb_generic_t* USB_Generic, u8_t* Byte) {


    return tusbe_OK;
}
ttc_usb_errorcode_e stm32_usb_read_word_blocking(ttc_usb_generic_t* USB_Generic, u16_t* Word) {



    return tusbe_OK;
}


//} Function definitions
//{ private functions (ideally) -------------------------------------------------

//ttc_usb_errorcode_e _stm32_usb_send_word_blocking(USB_t* MyUSB, const u16_t Word, Base_t TimeOut) {

//    return tusbe_OK;
//}
//inline void _stm32_usb_send_byte_isr(USB_t* MyUSB, const u16_t Word) {

//}
//ttc_usb_errorcode_e _stm32_usb_wait_for_RXNE(USB_t* MyUSB, Base_t TimeOut) {


//    return tusbe_OK;
//}
//ttc_usb_errorcode_e _stm32_usb_wait_for_TXE(USB_t* MyUSB, Base_t TimeOut) {



//    return tusbe_OK;
//}
void _stm32_usb_rx_isr(u8_t PhysicalIndex, void* Argument) {

}
void _stm32_usb_tx_isr(physical_index_t PhysicalIndex, void* Argument) {

}

//} private functions
