/*{ stm32_basic.h ***********************************************
 *
 * TheToolChain.com
 *
 * Written by Gregor Rebel 2010-2012
 *
 * Basic datatypes, enums and functions
 * 
}*/

#ifndef TTC_BASIC_H
#define TTC_BASIC_H

#include "compile_options.h" // contains all constant defines from makefile

#ifdef TARGET_ARCHsITECTURE_CM3
#include "cm3/cm3_basic.h"
#endif
#ifdef TARGET_ARCHITECTURE_EM357
#include "em357/em357_basic.h"
#endif
#ifdef TARGET_ARCHITECTURE_STM32
#include "stm32/stm32_basic.h"
#endif

#ifndef Base_t
  #error No architecture definition for Base_t!
#endif

#ifndef NULL
#define NULL 0
#endif

typedef enum ErrorCode_E {
    ec_None,                           // 0
    ec_Malloc,                         // 1
    ec_Debug,
    ec_NULL,
    ec_InvalidArgument,
    ec_InvalidConfiguration,
    ec_InvalidImplementation,          // an check-routine detected a low-level implementation that does not work as expected
    ec_ArrayIndexOutOfBound,           // array access with invalid index (-> ttc_memory.h/A() )
    ec_IllegalPointer,
    ec_NotImplemented,
    ec_OutOfResources,                 // requested amount of resources has exceeded predefined allowed amount
    ec_DeviceNotFound,
    ec_HardFault,                      // hard fault handler called: programm has stopped
    ec_OutOfMemory,                    // dynamic memory allocation failed
    ec_TimeOut,                        // TimeOut occured during waiting for a resource
    ec_UNKNOWN
} ErrorCode_e;

#if TARGET_DATA_WIDTH==32
  #define TARGET_MAX_UNSIGNED 0xffffffff
  #define TARGET_MAX_POSITIVE 0x7fffffff
  #define TARGET_MAX_NEGATIVE 0xffffffff
#endif
#if TARGET_DATA_WIDTH==16
  #define TARGET_MAX_UNSIGNED 0xffff
  #define TARGET_MAX_POSITIVE 0x7fff
  #define TARGET_MAX_NEGATIVE 0xffff
#endif
#if TARGET_DATA_WIDTH==8
  #define TARGET_MAX_UNSIGNED 0xff
  #define TARGET_MAX_POSITIVE 0x7f
  #define TARGET_MAX_NEGATIVE 0xff
#endif

/*
#if TARGET_DATA_WIDTH==32
  #define strnlen strnlen__32
#endif
#if TARGET_DATA_WIDTH==16
  #define strnlen strnlen__32
#endif
#ifndef strnlen
  #define strnlen strnlen__8
#endif
*/

//{ Function prototypes **************************************************

// using these macros is faster than calling Assert_Halt_EC() directly
#define Assert(Condition, ErrorCode)  if (! (Condition) ) Assert_Halt_EC(ErrorCode);
#define Assert_NULL(Pointer)          if (Pointer==NULL)  Assert_Halt_NULL();

/** Will block endlessly. Use it to check arguments and system states.
 *
 * @param ErrorCode  developers hint why condition has failed
 */
void Assert_Halt_EC(volatile ErrorCode_e ErrorCode);

/** Will block endlessly. Use it to check arguments and system states.
 *
 * Note: This function is only a helper where macro Assert() cannot be used!
 *
 * @param Condition !=0 <=> TRUE; FALSE otherwise
 * @return value of Condition
 */
bool Assert_On_False(BOOL Condition);

/** calls Assert_Halt_EC(ec_NULL); (less code-overhead than Assert_Halt_EC() )
 */
void Assert_Halt_NULL();


/** basic hardware initialization (required by most microcontrollers)
 */
void ttc_hardware_init();

/** Used for automatic code tests.
 *
 * Call from a place in your code that must be reached to successfully pass the test.
 * Place a breakpoint inside ttc_Testpoint() to check if your test passes.
 *
 */
void ttc_testpoint();

//}Function prototypes
#endif
