/*{ extensions.h *********************************************************
 *
 *               ToolChain_STM32
 *
 * This code manages activated source-code extensions
 * written 2010-2012 by Gregor Rebel
 *
 * Basic framework template for
 *   - Olimex STM32-P107/ STM32-H107/ STM32-LCD eval board
 *   - StdPeripheralsLibrary CMSIS V3.4.0 (tested)
 *   - GNU toolchain from CodeSourcery G++
 *   - FreeRTOS v7.0.1
 *   - FixPointLib 
 *   - uIP TCP/IP-Stack
 *   - Color LCD Panel 320x240
 *   - local/ remote debugging (JTAG-Adapter may be connected to different computer)
 *
 * Coding-style
 *  + gnu99
 *    It's year 2012, get a decent compiler if your's does not comply with this.
 *    arm-none-eabi-gcc-4.4.1 does support gnu99.
 *     
 *  + Text Folding
 *    This source makes use of text folding to improve hierarchical readability 
 *    It has been written with jEdit + plugin Configurable Fold Handler.
 *    Activate folding in your favourite text-editor for { and } 
 *    (fold-start and -end) for best visual experience.
 *
}*/
//{ check build environment **********************************************

#ifndef SOURCE_COMPILER
  #error SOURCE_COMPILER not defined -> activate.050_
#endif
#ifndef BOARD
  #error BOARD not defined -> activate.100_board_
#endif
#ifndef uCONTROLLER
  #error uCONTROLLER not defined -> activate.100_board_
#endif
#ifndef UC_DEVICE
  #error UC_DEVICE  not defined -> activate.100_board_
#endif
#ifndef UC_PACKAGE_PINS
  #error UC_PACKAGE_PINS not defined -> activate.100_board_
#endif
//#ifndef RT_SCHEDULER
//  #error RT_SCHEDULER not defined -> activate.300_scheduler_
//#endif
//#ifndef SCHEDULER_HEAP
//  #error SCHEDULER_HEAP not defined -> activate.300_scheduler_heap
//#endif
#ifdef EXTENSION_400_lcd_320x240_olimex
  #ifdef EXTENSION_100_board_olimex_lcd
    #define LCD_BOARD EXTENSION_100_board_olimex_lcd
  #endif
  #ifndef LCD_BOARD
    #error EXTENSION_400_lcd_320x240_olimex requires an LCD-board!
  #endif
#endif

//}
//{ Defines/ TypDefs *****************************************************

struct FirstTaskArguments {
    void (*MainTask)(void*); // function that will be called as main task
    void* MainArgument;
};

//} Defines
//{ Includes *************************************************************

/**
 * IO definitions
 *
 * define access restrictions to peripheral registers
 *
 * (Copied from core_cm3.h because its required to include stm32f10x.h which is required t include core_cm3.h!)
 */

// Basic set of helper functions
//? #include "stddef.h"
#include "ttc_basic.h"
#include "ttc_task.h"


#ifdef EXTENSION_500_ttc_debug_registers
#ifdef TARGET_ARCHITECTURE_STM32
  #include "stm32_registers.h"
#endif
#endif

#ifdef EXTENSION_500_ttc_watchdogs
#include "ttc_watchdog.h"
#endif

#ifdef EXTENSION_600_example_gfx_benchmark
  #include "example_gfx_benchmark.h"
#endif

#ifdef EXTENSION_600_example_gfx_printf
  #include "example_gfx_printf.h"
#endif

#ifdef EXTENSION_600_example_gfx_boxes
  #include "example_gfx_boxes.h"
#endif

#if defined(EXTENSION_600_example_spi_slave) || defined(EXTENSION_600_example_spi_master)
  #include "example_spi.h"
#endif

#if defined(EXTENSION_600_example_i2c_master) || defined(EXTENSION_600_example_i2c_slave)
  #include "example_i2c.h"
#endif

#ifdef EXTENSION_600_example_radio
  #include "example_radio.h"
#endif

#ifdef EXTENSION_600_example_radio_serial
  #include "example_radio_serial.h"
#endif
#ifdef EXTENSION_600_example_usb_vcp
  #include "example_usb_vcp.h"
#endif
#ifdef EXTENSION_600_example_host_hid
  #include "example_usb_host_hid.h"
#endif

// FreeRTOS
#ifdef EXTENSION_300_scheduler_free_rtos
  #include "FreeRTOS.h"
  #include "task.h"
  //? #include "stm32f10x_it.h"
  //? #include "stm32f10x_tim.h"
#endif


#ifdef EXTENSION_600_example_threading_queues
  #include "example_threading_queues.h"
#endif
#ifdef EXTENSION_600_example_threading_semaphores
  #include "example_threading_semaphores.h"
#endif
#ifdef EXTENSION_600_example_leds
  #include "example_leds.h"
#endif
//X #ifdef EXTENSION_400_lcd_320x240_olimex
//X   #include "example_lcd_320x240.h"
//X #endif
#ifdef EXTENSION_400_network_uip_ste101p
  #include "stm32_eth.h"
  #include "ethernet.h"
  #ifdef EXTENSION_http_server
      #include "httpd.h"
  #endif
#endif
#ifdef EXTENSION_600_network_lwip
#endif
#ifdef EXTENSION_600_example_usart
#include "example_usart.h"
#endif
#ifdef EXTENSION_600_example_channel
#include "example_channel.h"
#endif

#ifdef EXTENSION_650_regression_memory_pools
#include "regression_memory_pools.h"
#endif


//} Includes
//{ Global Variables *****************************************************



//} Global Variables
//{ Function prototypes **************************************************

/* Starts all activated extensions, spawns first task and then calls given main task
 * @param MainTask      function being called from first task
 * @param StackSize     stack size to use for first task
 * @param MainArgument  passed as argument to MainTask()
 */
void startExtensions(void(*MainTask)(void*), u16_t StackSize, void* MainArgument);

/* task which spawns all other tasks + runs main eventloop
 */
void firstTask(void *pvParameters);

//} Function prototypes

