#ifndef TTC_I2C_TYPES_H
#define TTC_I2C_TYPES_H

/*{ ttc_i2c_types.h ***********************************************
 *
 * Written by Gregor Rebel 2010-2012
 *
 * Basic defines, enums and structures.
 *
 * Include this file from 
 * architecture independent header files (ttc_XXX.h) and 
 * architecture depend      header files (e.g. stm32_XXX.h)
 * 
}*/

#include "ttc_basic.h"

//{ Defines/ TypeDefs ****************************************************

// TTC_I2Cn has to be defined as constant by makefile.100_board_*
#ifdef TTC_I2C5
  #ifndef TTC_I2C4
    #error TTC_I2C5 is defined, but not TTC_I2C4 - all lower TTC_I2Cn must be defined!
  #endif
  #ifndef TTC_I2C3
    #error TTC_I2C5 is defined, but not TTC_I2C3 - all lower TTC_I2Cn must be defined!
  #endif
  #ifndef TTC_I2C2
    #error TTC_I2C5 is defined, but not TTC_I2C2 - all lower TTC_I2Cn must be defined!
  #endif
  #ifndef TTC_I2C1
    #error TTC_I2C5 is defined, but not TTC_I2C1 - all lower TTC_I2Cn must be defined!
  #endif

  #define TTC_AMOUNT_I2CS 5
#else
  #ifdef TTC_I2C4
    #define TTC_AMOUNT_I2CS 4

    #ifndef TTC_I2C3
      #error TTC_I2C5 is defined, but not TTC_I2C3 - all lower TTC_I2Cn must be defined!
    #endif
    #ifndef TTC_I2C2
      #error TTC_I2C5 is defined, but not TTC_I2C2 - all lower TTC_I2Cn must be defined!
    #endif
    #ifndef TTC_I2C1
      #error TTC_I2C5 is defined, but not TTC_I2C1 - all lower TTC_I2Cn must be defined!
    #endif
  #else
    #ifdef TTC_I2C3
    
      #ifndef TTC_I2C2
        #error TTC_I2C5 is defined, but not TTC_I2C2 - all lower TTC_I2Cn must be defined!
      #endif
      #ifndef TTC_I2C1
        #error TTC_I2C5 is defined, but not TTC_I2C1 - all lower TTC_I2Cn must be defined!
      #endif
    
      #define TTC_AMOUNT_I2CS 3
    #else
      #ifdef TTC_I2C2

        #ifndef TTC_I2C1
          #error TTC_I2C5 is defined, but not TTC_I2C1 - all lower TTC_I2Cn must be defined!
        #endif

        #define TTC_AMOUNT_I2CS 2
      #else
        #ifdef TTC_I2C1
          #define TTC_AMOUNT_I2CS 1
        #else
          #define TTC_AMOUNT_I2CS 0
        #endif
      #endif
    #endif
  #endif
#endif

//} Defines
//{ Structures/ Enums 1 **************************************************

typedef enum {     // ttc_i2c_errorcode_e
  tie_OK,                                  // =0: no error
  tie_TimeOut,                             // timeout occured in called function
  tie_TimeOut_StartCondition,              // timeout occured while waiting for start-condition to take place
  tie_TimeOut_StopCondition,               // timeout occured while waiting for stop-condition to take place
  tie_TimeOut_Flag_Busy,                   // timeout occured while waiting for busy flag to reset
  tie_TimeOut_Master_ModeSelect,           // timeout occured while waiting for event master mode select
  tie_TimeOut_Master_TransmitterMode,      // timeout occured while waiting for event master transmitter mode select
  tie_TimeOut_Master_ByteTransmitted,      // timeout occured while waiting for event master bytes transmitted
  tie_TimeOut_Master_RegisterSend,         // timeout occured while sending register address to slave
  tie_TimeOut_Master_BufferSend,           // timeout occured while sending data buffer to slave
  tie_TimeOut_Master_ByteRead,             // timeout occured while waiting for incoming byte from slave
  tie_BusError,                            // bus error occured
  tie_Master_ArbitrationLost,              // master has lost bus arbitration
  tie_AcknowledgeFailure,                  // Acknowledgement failed
  tie_PEC_Error,                           // packet error correction failed
  tie_SMBusAlert,                          // SMBus alert occured
  tie_NotImplemented,                      // function has no implementation for current architecture
  tie_DeviceNotFound,                      // adressed I2C device not available in current uC
  tie_InvalidArgument,                     // general argument error
  tie_DeviceNotReady,                      // choosen device has not been initialized properly
  tie_AddressNotAcknowledgedBySlave,       // protocol error: The slave device did not acknowledge its address
  
  tie_UnknownError
} ttc_i2c_errorcode_e;
typedef enum {     // ttc_i2c_register_type_e
    tiras_MSB_First_8Bit,    // 1 byte  used to transmit a register adress (Most significant byte transferred first)
    tiras_MSB_First_16Bit,   // 2 bytes used to transmit a register adress (Most significant byte transferred first)
    tiras_MSB_First_24Bit,   // 3 bytes used to transmit a register adress (Most significant byte transferred first)
    tiras_MSB_First_32Bit,   // 4 bytes used to transmit a register adress (Most significant byte transferred first)
    tiras_LSB_First_8Bit,    // 1 byte  used to transmit a register adress (Least significant byte transferred first)
    tiras_LSB_First_16Bit,   // 2 bytes used to transmit a register adress (Least significant byte transferred first)
    tiras_LSB_First_24Bit,   // 3 bytes used to transmit a register adress (Least significant byte transferred first)
    tiras_LSB_First_32Bit    // 4 bytes used to transmit a register adress (Least significant byte transferred first)
} ttc_i2c_register_type_e;

typedef struct {   // ttc_i2c_event_t  triggered event interrupts
  unsigned StartBitSend    : 1;     // =1: start bit send (SB)
  unsigned Address         : 1;     // =1: address send/ matched (ADDR)
  unsigned Address10       : 1;     // =1: 10-but header sent (ADD10)
  unsigned StopReceived    : 1;     // =1: Stop received (STOPF)
  unsigned ByteTransfered  : 1;     // =1: Data byte transfer finished (BTF)
  unsigned TxE             : 1;     // =1: Transmit buffer empty (TxE)
  unsigned RxNE            : 1;     // =1: Receive buffer not empty (RxNE)
  unsigned Reserved        : 1;
} __attribute__((__packed__)) ttc_i2c_event_t;
typedef struct {   // ttc_i2c_error_t  triggered error interrupts
  unsigned Bus             : 1;     // =1: Bus Error (BERR)
  unsigned ArbitrationLoss : 1;     // =1: Arbitration Loss (Master) (ARLO)
  unsigned AcknowledgeFail : 1;     // =1: AcknowledgeFailuire (AF)
  unsigned Overrun         : 1;     // =1: OverRun/ UnderRun (OVR)
  unsigned Checksum        : 1;     // =1: Packet received with invalid Checksum (PECERR)
  unsigned Timeout         : 1;     // =1: Timeout/ Tlow error (TIMEOUT)
  unsigned SMBusAlert      : 1;     // =1: SMBus Alert (SMBALERT)
  unsigned Reserved        : 1;
} __attribute__((__packed__)) ttc_i2c_error_t;

typedef enum { // ttc_i2c_direction_e
    tid_ReadingFromSlave,
    tid_WritingToSlave
} ttc_i2c_direction_e;

// Reference
// RM008 - STM32 Reference Manual p. 754 ff
// -> http://www.st.com/internet/com/TECHNICAL_RESOURCES/TECHNICAL_LITERATURE/REFERENCE_MANUAL/CD00171190.pdf

typedef struct { // Bits
  unsigned Master           : 1;  // =1: enable master mode (overrides Slave = 1); =0: not master
  unsigned Slave            : 1;  // =1: enable slave  mode; =0: not slave (automatically activates master mode)
  unsigned ModeFast         : 1;  // =1: enable fast mode; =0: standard mode
  unsigned DMA_TX           : 1;  // =1: enable direct memory access for transmitting data; =0: no DMA
  unsigned DMA_RX           : 1;  // =1: enable direct memory access for receiving data; =0: no DMA
  unsigned SMBus            : 1;  // =1: enable SMBus mode; =0: normal I2C mode
  unsigned SMBus_Alert      : 1;  // =1: use extra pin SMB_Alert; =0: only use SCL+SDA
  unsigned SMBus_Host       : 1;  // =1: enable Host configuration for SMBus mode; =0: Device configuration
  unsigned SMBus_ARP        : 1;  // =1: enable Address Resolution Protocol for SMBus mode; =0: no ARP
  unsigned DutyCycle_16_9   : 1;  // =1: enable Tlow/Thigh = 16/9 in fast mode; = 0: Tlow/Thigh = 2/1
  unsigned Acknowledgement  : 1;  // =1: enable acknowledgement mode; =0: no acknowledgements
  unsigned Adress_10Bit     : 1;  // =1: enable 10 bit wide adresses; =0: adresses are 7 bit wide
  unsigned PacketErrorCheck : 1;  // =1: enable packet error checking; =0: no error checking
  unsigned Slave_NoStretch  : 1;  // =1: enable clock stretching in slave mode; =0: no clock stretching
  unsigned GeneralCalls     : 1;  // =1: enable ACK of general address 0x00; =0: general address is not acknowledged
  unsigned Interrupt_Events : 1;  // =1: enable interrupts for events: =0: no interrupt generation for events
  unsigned Interrupt_Errors : 1;  // =1: enable interrupts for errors: =0: no interrupt generation for errors
  unsigned Reserved1        : 1;  // pad to 16 bits
} __attribute__((__packed__)) tib_Bits_t;

typedef struct { // architecture independent configuration data of single I2C
  union  {
    u32_t All;
    tib_Bits_t Bits;
  } Flags;
  u16_t OwnAddress;                     // adress of this device
  u32_t ClockSpeed;                     // clock frequency to use (< 400000 on stm32f10x)
  u32_t ErrorCount;                     // increased on every error condition
  u8_t  Layout;                         // select pin layout to use (some uC allow pin remapping)
  u32_t TimeOut;                        // maximum allowed time for individual transactions (usecs)
  void*(*ISR_Events)(ttc_i2c_event_t);  // function pointer to interrupt service routine for events
  void*(*ISR_Errors)(ttc_i2c_error_t);  // function pointer to interrupt service routine for errors
} __attribute__((__packed__)) ttc_i2c_generic_t;

typedef union {  // ttc_i2c_Address_7Bit_u
    struct {
        unsigned ReadMode : 1;  // =1: master is reading from slave
        unsigned Address  : 7;  // 7 bit slave address
    } Fields;
    u8_t Byte;
} ttc_i2c_Address_7Bit_u;

//}

#endif // TTC_I2C_TYPES_H
