#ifndef ttc_mutex_types_H
#define ttc_mutex_types_H

/*{ ttc_mutex_types.h ***********************************************
 * 
 * Written by Gregor Rebel 2010-2012
 *
 * Architecture independent support for inter task communication & synchronization.
 * 
 * Currently supported architectures/ schedulers: CortexM3, FreeRTOS
 * 
}*/

//{ includes

#include "ttc_basic.h"

//? typedef void* ttc_QueueHandle;

typedef enum ttc_mutex_error_E {      // return type of most functions in ttc_mutex
    tme_OK,                           // no error
    tme_HigherPriorityTaskWoken,      // an _isr()-call has unblocked a higher priority task to unblock. Call ttc_task_yield() before exiting your ISR!

    tme_Error,
    // Error Conditions below
    tme_TimeOut,                      // an operation has timed out
    tme_NotImplemented,               // required function not implemented for current architecture
    tme_MutexAlreadyLocked,                  // tried to lock a mutex that is already locked
    tme_UNKNOWN
} ttc_mutex_error_e;

//}includes
//{ Default settings (change via makefile) ******************************


#ifndef TTC_MUTEX_TIMEOUT_USECS
  #define TTC_MUTEX_TIMEOUT_USECS 1000000 // activate timeout-assert for endless mutexes (TimeOut==-1) in usecs
  // #define TTC_MUTEX_TIMEOUT_USECS 0    // no forced timeout
#endif

// enable use of Assert() for mutexes
#ifndef TTC_ASSERT_MUTEXES
  #define TTC_ASSERT_MUTEXES 1
#endif

//}Default settings
//{ Types & Structures ***************************************************

#if TTC_ASSERT_MUTEXES == 1 // use Assert()s in Queue code (somewhat slower but alot easier to debug)
  #define Assert_MUTEXES(Condition, ErrorCode) Assert(Condition, ErrorCode)
#else  // us no Assert()s in USART code (somewhat smaller + faster, but crashes are hard to debug)
  #define Assert_MUTEXES(Condition, ErrorCode)
#endif

//}

#endif
