/*{ ttc_radio::.c ************************************************
 
                      The ToolChain
                      
   Device independent support for 
   
   
   written by Gregor Rebel 2012
   
 
}*/

#include "ttc_radio.h"

//{ Global variables *****************************************************

ttc_array_define(ttc_radio_generic_t*, RadioConfigs, TTC_AMOUNT_RADIOS);

// stores released memory blocks for reuse as transmit or receive blocks
ttc_QueueHandle_Pointers_t ttc_radio_queue_empty_blocks = NULL;   // stores empty memory blocks for reuse

// protects queues from simultaneous access
ttc_Mutex*            ttc_radio_mutex_empty_blocks = NULL;

// Amount of radio network memory blocks allocated so far
u8_t  ttc_radio_allocated_blocks = 0;
u32_t ttc_radio_used_blocks = 0;
u32_t ttc_radio_released_blocks = 0;

// Index of radio device to use for standard output
u8_t RadioIndex_StdOut = 0;

//}Global variables
//{ Function definitions *************************************************

          inline u8_t ttc_radio_get_max_index() {

    return TTC_AMOUNT_RADIOS; // -> ttc_radio_types.h
}
 ttc_radio_generic_t* ttc_radio_get_configuration(u8_t LogicalIndex) {
             Assert_Radio(LogicalIndex > 0,                          ec_InvalidArgument); // logical index starts at 1
             Assert_Radio(LogicalIndex <= ttc_radio_get_max_index(), ec_InvalidArgument); // outside range of activated radios
             ttc_radio_generic_t* Radio_Generic = A(RadioConfigs, LogicalIndex-1);

             if (!Radio_Generic) {
                 Radio_Generic = (ttc_radio_generic_t*) ttc_memory_alloc_zeroed( sizeof(ttc_radio_generic_t) );
                 A(RadioConfigs, LogicalIndex-1) = Radio_Generic;

                 ttc_radio_load_defaults(LogicalIndex);
             }
             return Radio_Generic;
         }
ttc_radio_errorcode_e ttc_radio_load_defaults(u8_t LogicalIndex) {
    if ( (LogicalIndex > TTC_AMOUNT_RADIOS) || (LogicalIndex < 1) )
        return ec_DeviceNotFound;

    ttc_radio_generic_t* RadioGeneric = ttc_radio_get_configuration(LogicalIndex);
    memset(RadioGeneric, 0, sizeof(ttc_radio_generic_t));
    ttc_radio_errorcode_e Error = tre_NotImplemented;

    switch (ttc_radio_get_driver_type(LogicalIndex)) {
#ifdef EXTENSION_400_radio_em357
    case trd_em357:
        radio_em357_get_defaults(LogicalIndex, RadioGeneric);
        Error = tre_OK;
        break;
#endif
#ifdef EXTENSION_400_radio_cc1101_spi
    case trd_cc1101_spi:
        radio_cc1101_get_defaults(LogicalIndex, RadioGeneric);
        Error = tre_OK;
        break;
#endif
#ifdef EXTENSION_400_radio_cc1120_spi
    case trd_cc1120_spi:
        radio_cc1120_get_defaults(LogicalIndex, RadioGeneric);
        Error = tre_OK;
        break;
#endif
    default: break;
    }

    if (Error == tre_OK) { // low-level driver available: fill in generic configuration defaults
        RadioGeneric->LogicalIndex = LogicalIndex;
        RadioGeneric->Driver = ttc_radio_get_driver_type(LogicalIndex);
        Assert_Radio(RadioGeneric->Driver != trd_None, ec_InvalidConfiguration);

        if (!RadioGeneric->Size_QueueRx)
            RadioGeneric->Size_QueueRx = 5;
        if (!RadioGeneric->Size_QueueTx)
            RadioGeneric->Size_QueueTx = 5;

        RadioGeneric->AMP_Initialized = FALSE;
    }

    return Error;
}
ttc_radio_errorcode_e ttc_radio_get_features(u8_t LogicalIndex, ttc_radio_generic_t* Radio_Generic) {
    if ( (LogicalIndex > TTC_AMOUNT_RADIOS) || (LogicalIndex < 1) )
        return ec_DeviceNotFound;
    Assert_Radio(Radio_Generic, ec_InvalidArgument);

    memset(Radio_Generic, 0, sizeof(Radio_Generic));
    ttc_radio_errorcode_e Error = tre_NotImplemented;

    switch (ttc_radio_get_driver_type(LogicalIndex)) {
#ifdef EXTENSION_400_radio_em357
    case trd_cc1101_spi:
        radio_em357_get_features(LogicalIndex, Radio_Generic);
        Error = tre_OK;
        break;
#endif
#ifdef EXTENSION_400_radio_cc1101_spi
    case trd_cc1101_spi:
        radio_cc1101_get_features(LogicalIndex, Radio_Generic);
        Error = tre_OK;
        break;
#endif
#ifdef EXTENSION_400_radio_cc1120_spi
    case trd_cc1120_spi:
        radio_cc1120_get_features(LogicalIndex, Radio_Generic);
        Error = tre_OK;
        break;
#endif
    default:
        break;
    }

    if (Error == tre_OK) { // low-level driver available: fill in generic maximum values
        Radio_Generic->LogicalIndex = LogicalIndex;
        Radio_Generic->Driver = ttc_radio_get_driver_type(LogicalIndex);
        Radio_Generic->Flags.Bit.DelayedTransmits = 0;
        Radio_Generic->Flags.Bit.RxDMA            = 0;
        Radio_Generic->Flags.Bit.TxDMA            = 0;
    }

    return Error;
}
ttc_radio_errorcode_e ttc_radio_get_max_channel(u8_t LogicalIndex, u8_t* MaxChannelTX, u8_t* MaxChannelRX) {

    ttc_radio_generic_t* Radio_Generic = ttc_radio_get_configuration(LogicalIndex);
    Assert_Radio(MaxChannelTX, ec_InvalidArgument);
    Assert_Radio(MaxChannelRX, ec_InvalidArgument);

    Assert_Radio(FALSE, ec_NotImplemented); //D Problem here with function calls no low level function avaliable for CC1101 and CC1120
    ttc_radio_errorcode_e Error = tre_NotImplemented;
#ifdef EXTENSION_400_radio_em357
    if (Radio_Generic->Driver == trd_em357)
        Error = radio_em357_get_max_channel(LogicalIndex, MaxChannelTX, MaxChannelRX); //ToDo: Wrong function call
#endif
#ifdef EXTENSION_400_radio_cc1101_spi
    if (Radio_Generic->Driver == trd_cc1101_spi)
        Error = radio_cc1101_get_max_channel(LogicalIndex, MaxChannelTX, MaxChannelRX); //ToDo: Wrong function call
#endif
#ifdef EXTENSION_400_radio_cc1120_spi
    if (Radio_Generic->Driver == trd_cc1120_spi){
        // Error = radio_cc1120_get_max_channel(LogicalIndex, MaxChannelTX, MaxChannelRX); //ToDo: Wrong function call
    }
#endif

  return Error;
}
ttc_radio_errorcode_e ttc_radio_get_max_level(u8_t LogicalIndex, u8_t* MaxLevelTX, u8_t* MaxLevelRX) {
    ttc_radio_generic_t* Radio_Generic = ttc_radio_get_configuration(LogicalIndex);
    Assert_Radio(MaxLevelTX, ec_InvalidArgument);
    Assert_Radio(MaxLevelRX, ec_InvalidArgument);

    Assert_Radio(FALSE,ec_NotImplemented); //D Problem here with function calls no low level function avaliable for CC1101 and CC1120
    ttc_radio_errorcode_e Error = tre_NotImplemented;
#ifdef EXTENSION_400_radio_em357
    if (Radio_Generic->Driver == trd_em357)
        Error = radio_em357_get_max_level(LogicalIndex, MaxLevelTX, MaxLevelRX); //ToDo: Wrong function call
#endif
#ifdef EXTENSION_400_radio_cc1101_spi
    if (Radio_Generic->Driver == trd_cc1101_spi)
        Error = radio_cc1101_get_max_level(LogicalIndex, MaxLevelTX, MaxLevelRX); //ToDo: Wrong function call
#endif
#ifdef EXTENSION_400_radio_cc1120_spi
    if (Radio_Generic->Driver == trd_cc1120_spi){
        //  Error = radio_cc1120_get_max_level(LogicalIndex, MaxLevelTX, MaxLevelRX);  //ToDo: Wrong function call
        // write function and use CC1120_MAX_TX_LEVEL (check with Datasheet if value is right!!!)
    }
#endif

  return Error;
}
ttc_radio_errorcode_e ttc_radio_init(u8_t LogicalIndex, u8_t DeviceAddress) {
    ttc_radio_generic_t* Radio_Generic = ttc_radio_get_configuration(LogicalIndex);
    Assert_Radio(Radio_Generic, ec_InvalidArgument);
    Assert_Radio(Radio_Generic->LogicalIndex > 0, ec_InvalidArgument);
    Assert_Radio(Radio_Generic->LogicalIndex <= ttc_radio_get_max_index(), ec_InvalidArgument);

#ifdef EXTENSION_500_ttc_watchdogs
    ttc_watchdog_init1(0,256);
    ttc_watchdog_reload1();
#endif
    ttc_radio_get_features(Radio_Generic->LogicalIndex, Radio_Generic);
    // check given configuration against features of configured radio
    ttc_radio_generic_t Radio_Features;
    ttc_radio_get_features(Radio_Generic->LogicalIndex, &Radio_Features);

    Assert_Radio(Radio_Generic->ChannelRx   <= Radio_Features.ChannelRx,   ec_InvalidConfiguration);
    Assert_Radio(Radio_Generic->ChannelTx   <= Radio_Features.ChannelTx,   ec_InvalidConfiguration);
    Assert_Radio(Radio_Generic->Max_Header  <= Radio_Features.Max_Header,  ec_InvalidConfiguration);
    Assert_Radio(Radio_Generic->Max_Footer  <= Radio_Features.Max_Footer,  ec_InvalidConfiguration);
    Assert_Radio(Radio_Generic->Max_Payload <= Radio_Features.Max_Payload, ec_InvalidConfiguration);
    Assert_Radio(Radio_Generic->LevelRx     <= Radio_Features.MaxLevelRx,  ec_InvalidConfiguration);
    Assert_Radio(Radio_Generic->LevelTx     <= Radio_Features.MaxLevelTx,  ec_InvalidConfiguration);
    Assert_Radio(Radio_Generic->Driver      == Radio_Features.Driver,      ec_InvalidConfiguration);

    // limit configured flags to supported features
    Radio_Generic->Flags.All &= Radio_Features.Flags.All;

    if (!Radio_Generic->Queue_Tx)
        Radio_Generic->Queue_Tx = ttc_queue_pointer_create(Radio_Generic->Size_QueueTx);

    if (!Radio_Generic->Queue_Rx)
        Radio_Generic->Queue_Rx = ttc_queue_pointer_create(Radio_Generic->Size_QueueRx);

    if (!Radio_Generic->MutexPacketReceived) {
        Radio_Generic->MutexPacketReceived = ttc_mutex_create();
        ttc_mutex_lock(Radio_Generic->MutexPacketReceived, 0);
    }
    Radio_Generic->TaskYield = ttc_radio_task_yield;
    Radio_Generic->DeviceAddress = DeviceAddress; //set the device address used for packet filtering
    ttc_radio_errorcode_e Error = tre_NotImplemented;
    ttc_radio_set_amplifier(LogicalIndex, tram_Init);

    Error = tre_NotImplemented;
#ifdef EXTENSION_400_radio_em357
    if (Radio_Generic->Driver == trd_em357_spi)
        Error = radio_em357_init(Radio_Generic);
#elif EXTENSION_400_radio_cc1101_spi
    if (Radio_Generic->Driver == trd_cc1101_spi)
        Error = radio_cc1101_init(Radio_Generic);
#elif EXTENSION_400_radio_cc1120_spi
    if (Radio_Generic->Driver == trd_cc1120_spi)
        Error = radio_cc1120_init(Radio_Generic);
#else
    Assert(0,ec_InvalidConfiguration); //no radio type selected
#endif

    ttc_radio_set_operating_mode(LogicalIndex,trm_Receive);
    if (Error == tre_OK) { // radio has been initialized successfully: initialize some high-level stuff
        if (!Radio_Generic->Flags.Bit.Task_RX_Started) {
            Radio_Generic->Flags.Bit.Task_RX_Started = 1;
            ttc_task_create( _ttc_radio_task_rx, "_ttc_radio_task_rx", 128, Radio_Generic, 1, NULL);
        }
        if (!Radio_Generic->Flags.Bit.Task_TX_Started) {
            Radio_Generic->Flags.Bit.Task_TX_Started = 1;
            ttc_task_create( _ttc_radio_task_tx, "_ttc_radio_task_tx", 256, Radio_Generic, 0, NULL);
        }
        ttc_radio_set_power_tx(Radio_Generic->LogicalIndex, Radio_Generic->LevelTx);
    }
#ifdef EXTENSION_500_ttc_watchdogs
    ttc_watchdog_reload1();
#endif
    return Error;
}
                 void ttc_radio_send_block(u8_t LogicalIndex, ttc_memory_block_t* Block) {
    ttc_radio_generic_t* Radio_Generic = ttc_radio_get_configuration(LogicalIndex);

    if (Radio_Generic) {
        Block->Hint = Radio_Generic->TargetAddress;
        while (ttc_queue_pointer_push_back(Radio_Generic->Queue_Tx, Block) != tqe_OK)
            ttc_radio_task_yield();
    }
}
ttc_radio_errorcode_e ttc_radio_send_raw(u8_t LogicalIndex, const u8_t* Buffer, Base_t Amount) {
    ttc_radio_generic_t* Radio_Generic = ttc_radio_get_configuration(LogicalIndex);

    while (Amount > 0) {
        ttc_memory_block_t* Block = ttc_radio_get_empty_block();
        if (!Block)
            return tre_TimeOut;

        u8_t Size;

        if (Amount > Radio_Generic->Max_Payload)
            Size = Radio_Generic->Max_Payload;
        else
            Size = Amount;

        ttc_memory_copy(Block->Buffer, (u8_t*) Buffer, Size);
        Block->Size = Size;
        ttc_radio_send_block(LogicalIndex, Block);

        Buffer += Size ;
        Amount -= Size ;
    }
    return tre_OK;
}
ttc_radio_errorcode_e ttc_radio_send_string(u8_t LogicalIndex, const char* String, Base_t MaxSize) {
    ttc_radio_generic_t* Radio_Generic = ttc_radio_get_configuration(LogicalIndex);

    Base_t Bytes2Copy = MaxSize;

    while (Bytes2Copy > 0) {
        ttc_memory_block_t* Block = ttc_radio_get_empty_block();
        if (!Block)
            return tre_UnknownError;

        Block->Size = ttc_string_copy(Block->Buffer, (u8_t*) String, Radio_Generic->Max_Payload);
        if (Block->Size > 0) {
            ttc_radio_send_block(LogicalIndex, Block);

            String += Block->Size;
            if (*String == 0) // reached end of String[]
                Bytes2Copy = 0;
            else              // String[] is bigger than memory block: will continue with next block
                Bytes2Copy -= Block->Size;
        }
        else {
            ttc_memory_block_release(Block);
            Bytes2Copy = 0;
        }
    }
    return tre_OK;
}
ttc_radio_errorcode_e ttc_radio_send_string_const(u8_t LogicalIndex, const char* String, Base_t MaxSize) {
    ttc_memory_block_t* Block = ttc_radio_get_empty_block();
    if (!Block)
        return tre_TimeOut;

    Block->Buffer = (u8_t*) (char*) String;
    Block->Size   = strnlen__16(String, MaxSize);

    ttc_radio_send_block(LogicalIndex, Block);
    return tre_OK;
}
                 void ttc_radio_register_rx_function(u8_t LogicalIndex, ttc_memory_block_t* (*RxFunction)(struct ttc_radio_generic_s* RadioGeneric, ttc_memory_block_t* Block, u8_t RSSI, u8_t TargetAddress)) {
    ttc_radio_generic_t* Radio_Generic = ttc_radio_get_configuration(LogicalIndex);

    Radio_Generic->RxFunction_UCB = RxFunction;
}
                 void ttc_radio_register_tx_function(u8_t LogicalIndex, void (*TxFunction)(ttc_radio_generic_t* RadioGeneric, u8_t Size, const u8_t* Buffer)) {
    ttc_radio_generic_t* Radio_Generic = ttc_radio_get_configuration(LogicalIndex);

    Radio_Generic->TxFunction_UCB = TxFunction;
}
                 void ttc_radio_set_amplifier(u8_t LogicalIndex, ttc_radio_amplifier_mode_e Mode) {
      ttc_radio_generic_t* Radio_Generic = ttc_radio_get_configuration(LogicalIndex);

#ifdef EXTENSION_400_radio_cc1190
      radio_cc1190_set_mode(Radio_Generic, Mode); // ToDo: check if indexed radio has this amplifier!
#else

      (void) Mode;
      (void) Radio_Generic;
#endif
  }
                 void ttc_radio_set_destination(u8_t LogicalIndex, u8_t Address) {
                     ttc_radio_generic_t* Radio_Generic = ttc_radio_get_configuration(LogicalIndex);
                     Radio_Generic->TargetAddress = Address;
                 }
ttc_radio_errorcode_e ttc_radio_set_operating_mode(u8_t LogicalIndex, ttc_radio_mode_e Mode) {
                     ttc_radio_generic_t* Radio_Generic = ttc_radio_get_configuration(LogicalIndex);
    Radio_Generic->Mode = Mode;

#ifdef EXTENSION_400_radio_em357
    if (Radio_Generic->Driver == trd_em357)
        return radio_em357_set_operating_mode(Radio_Generic);
#endif
#ifdef EXTENSION_400_radio_cc1101_spi
    if (Radio_Generic->Driver == trd_cc1101_spi)
        return radio_cc1101_set_operating_mode(Radio_Generic);
#endif
#ifdef EXTENSION_400_radio_cc1120_spi
    if (Radio_Generic->Driver == trd_cc1120_spi)
        return radio_cc1120_set_operating_mode(Radio_Generic);
#endif

    return tre_NotImplemented;
}
                 void ttc_radio_set_power_tx(u8_t LogicalIndex, s8_t Level) {
    ttc_radio_generic_t* Radio_Generic = ttc_radio_get_configuration(LogicalIndex);

#ifdef EXTENSION_400_radio_em357
    if (Radio_Generic->Driver == trd_em357)
        radio_em357_set_power_tx(Radio_Generic, Level);
#endif
#ifdef EXTENSION_400_radio_cc1101_spi
    if (Radio_Generic->Driver == trd_cc1101_spi)
        radio_cc1101_set_power_tx(Radio_Generic, Level);
#endif
#ifdef EXTENSION_400_radio_cc1120_spi
    if (Radio_Generic->Driver == trd_cc1120_spi)
        radio_cc1120_set_power_tx(Radio_Generic, Level);
#endif

}
                 void ttc_radio_set_channel_rx(u8_t LogicalIndex, u8_t ChannelIndex) {
    ttc_radio_generic_t* Radio_Generic = ttc_radio_get_configuration(LogicalIndex);
    Assert_Radio(ChannelIndex > 0, ec_InvalidArgument);                             // logical index starts at 1
    Assert_Radio(ChannelIndex <= Radio_Generic->MaxChannelRx, ec_InvalidArgument);  // invalid channel index

    ttc_radio_errorcode_e Error = tre_NotImplemented;
#ifdef EXTENSION_400_radio_em357
    if (Radio_Generic->Driver == trd_em357)
        Error = radio_em357_set_channel(Radio_Generic, ChannelIndex); // cc1101 does not provide independent rx/tx channels;
#endif
#ifdef EXTENSION_400_radio_cc1101_spi
    if (Radio_Generic->Driver == trd_cc1101_spi)
        Error = radio_cc1101_set_channel(Radio_Generic, ChannelIndex); // cc1101 does not provide independent rx/tx channels;
#endif
#ifdef EXTENSION_400_radio_cc1120_spi
    if (Radio_Generic->Driver == trd_cc1120_spi)
        Error = radio_cc1120_set_channel(Radio_Generic, ChannelIndex); // cc1101 does not provide independent rx/tx channels;
#endif

  return;
}
                 void ttc_radio_set_channel_tx(u8_t LogicalIndex, u8_t ChannelIndex) {
    ttc_radio_generic_t* Radio_Generic = ttc_radio_get_configuration(LogicalIndex);
    Assert_Radio(ChannelIndex > 0, ec_InvalidArgument); // logical index starts at 1
    Assert_Radio(ChannelIndex <= Radio_Generic->MaxChannelTx, ec_InvalidArgument);  // invalid channel index

    ttc_radio_errorcode_e Error = tre_NotImplemented;
#ifdef EXTENSION_400_radio_em357
    if (Radio_Generic->Driver == trd_em357)
        Error = radio_em357_set_channel(Radio_Generic, ChannelIndex); // cc1101 does not provide independent rx/tx channels;
#endif
#ifdef EXTENSION_400_radio_cc1101_spi
    if (Radio_Generic->Driver == trd_cc1101_spi)
        Error = radio_cc1101_set_channel(Radio_Generic, ChannelIndex); // cc1101 does not provide independent rx/tx channels;
#endif
#ifdef EXTENSION_400_radio_cc1120_spi
    if (Radio_Generic->Driver == trd_cc1120_spi)
        Error = radio_cc1120_set_channel(Radio_Generic, ChannelIndex); // cc1101 does not provide independent rx/tx channels;
#endif

  return;
}
ttc_radio_errorcode_e ttc_radio_stdout_set(u8_t LogicalIndex) {
    ttc_radio_get_configuration(LogicalIndex); // will check if LogicalIndex is valid

    RadioIndex_StdOut = LogicalIndex;

    return tre_OK;
}
                 void ttc_radio_stdout_send_block(ttc_memory_block_t* Block) {
    if (RadioIndex_StdOut) {
        ttc_radio_send_block(RadioIndex_StdOut, Block);
    }
}
                 void ttc_radio_stdout_send_string(const char* String, Base_t MaxSize) {
    if (RadioIndex_StdOut) {
        ttc_radio_send_string(RadioIndex_StdOut, String, MaxSize);
    }
}
                 void ttc_radio_flush_tx(u8_t LogicalIndex) {
    ttc_radio_generic_t* Radio_Generic = ttc_radio_get_configuration(LogicalIndex);

    if (Radio_Generic->Flags.Bit.DelayedTransmits) { // delayed transmits have been activated for this radio

        // wait for transmit queue to run empty
        while (ttc_queue_pointer_get_amount(Radio_Generic->Queue_Tx) > 0)
            ttc_radio_task_yield();
    }
}
   ttc_radio_driver_e ttc_radio_get_driver_type(u8_t LogicalIndex) {
       switch (LogicalIndex) {
#ifdef TTC_RADIO1
       case 1: return TTC_RADIO1; // shall be defined as one from ttc_radio_types.h/ttc_radio_driver_e
#endif
#ifdef TTC_RADIO2
       case 2: return TTC_RADIO2; // shall be defined as one from ttc_radio_types.h/ttc_radio_driver_e
#endif
#ifdef TTC_RADIO3
       case 3: return TTC_RADIO3; // shall be defined as one from ttc_radio_types.h/ttc_radio_driver_e
#endif
#ifdef TTC_RADIO4
       case 4: return TTC_RADIO4; // shall be defined as one from ttc_radio_types.h/ttc_radio_driver_e
#endif
#ifdef TTC_RADIO5
       case 5: return TTC_RADIO5; // shall be defined as one from ttc_radio_types.h/ttc_radio_driver_e
#endif
       default: return trd_None;
       }
 }
  ttc_memory_block_t* ttc_radio_get_empty_block() {

    if (ttc_radio_queue_empty_blocks == NULL) { // first call: create queue
        ttc_radio_queue_empty_blocks = ttc_queue_pointer_create(TTC_RADIO_MAX_MEMORY_BUFFERS+1);
        ttc_radio_mutex_empty_blocks = ttc_mutex_create();
    }
    ttc_memory_block_t* Block = NULL;
    u16_t TimeOut = 1000;

    while (Block == NULL) {
        Assert_Radio(ttc_mutex_lock(ttc_radio_mutex_empty_blocks, 1000000) == tqe_OK, ec_TimeOut); // could not get lock!
        ttc_queue_error_e  Error = ttc_queue_pointer_pull_front(ttc_radio_queue_empty_blocks, (void**) &Block);

        if (Error != tqe_OK) { // no released memory block available: alloc new block
            if (ttc_radio_allocated_blocks < TTC_RADIO_MAX_MEMORY_BUFFERS) {
                Block = ttc_memory_alloc_block(TTC_NETWORK_MAX_PAYLOAD_SIZE,
                                               0, // will be used for target address
                                               _ttc_radio_release_block
                                               );
                ttc_radio_allocated_blocks++;
            }
            else{
                Block = NULL;
                Assert_Radio(FALSE, ec_Debug);//D can not get another empty block
            }
        }
        else { // reuse released memory block
            // Buffer-pointer might have been changed
            Assert_Radio(Block != NULL, ec_IllegalPointer); //D
            Block->Buffer = (u8_t*) (  ( (u32_t) Block ) + sizeof(ttc_memory_block_t)  );
            Block->Size = 0;
        }
        if (1) {
            if (0) ttc_task_end_criticalsection();
            else   ttc_mutex_unlock(ttc_radio_mutex_empty_blocks);
        }
        if (Block == NULL) { // no empty block available: wait some time and try again
            if(1) Assert(TimeOut-- > 0, ec_UNKNOWN); // cannot get empty memory block!
            //not possibele to assert here because if new buffers to be send over the radio are generated faster than the radio is able to transmitt them,
            //this timeout will be decreased causing this function to assert.
            //Hence it is not possible to wait until the ttc_radio_queue_empty_blocks has new free buffers
            ttc_task_yield();
        }
    }
    ttc_radio_used_blocks++;

    return Block;
}
                 void ttc_radio_task_yield() {
   ttc_task_yield();
   ttc_mutex_wait(); //D
}
                 u8_t _ttc_radio_check_bytes_received(ttc_radio_generic_t* Radio_Generic) {
#ifdef EXTENSION_400_radio_em357
    if (Radio_Generic->Driver == trd_em357)
        return radio_em357_spi_register_single_read( (radio_em357_driver_t*) Radio_Generic->DriverCfg, rcr_RXBYTES);
#endif
#ifdef EXTENSION_400_radio_cc1101_spi
    if (Radio_Generic->Driver == trd_cc1101_spi)
        return radio_cc1101_spi_register_single_read( (radio_cc1101_driver_t*) Radio_Generic->DriverCfg, rcr_RXBYTES);
#endif
#ifdef EXTENSION_400_radio_cc1120_spi
    if (Radio_Generic->Driver == trd_cc1120_spi)
        return radio_cc1120_spi_register_read((radio_cc1120_driver_t*) Radio_Generic->DriverCfg, rcr_CC112X_FIFO_NUM_RXBYTES);
#endif
    return 0;
}
                 BOOL _ttc_radio_check_mode(ttc_radio_generic_t* Radio_Generic) {

    BOOL IllegalMode = FALSE;
#ifdef EXTENSION_400_radio_em357
    if (Radio_Generic->Driver == trd_em357)
        IllegalMode = radio_em357_check_mode(Radio_Generic);
#endif
#ifdef EXTENSION_400_radio_cc1101_spi
    if (Radio_Generic->Driver == trd_cc1101_spi)
        IllegalMode = radio_cc1101_check_mode(Radio_Generic);
#endif
#ifdef EXTENSION_400_radio_cc1120_spi
    if (Radio_Generic->Driver == trd_cc1120_spi)
        IllegalMode = radio_cc1120_check_mode(Radio_Generic);
#endif
    return IllegalMode;
}
                 void _ttc_radio_release_block(ttc_memory_block_t* Block) {
    Assert(Block != NULL, ec_NULL);

    if (1) {
        if (0) ttc_task_begin_criticalsection(_ttc_radio_release_block);
        else   ttc_mutex_lock(ttc_radio_mutex_empty_blocks, -1);
    }
    ttc_queue_error_e Error = tqe_UNKNOWN;

    Error = ttc_queue_pointer_push_back(ttc_radio_queue_empty_blocks, Block);
    Assert_Radio(Error == tre_OK, ec_UNKNOWN); // could not release memory block (maybe queue is too small?)

    if (1) {
        if (0) ttc_task_end_criticalsection();
        else   ttc_mutex_unlock(ttc_radio_mutex_empty_blocks);
    }
    ttc_radio_released_blocks++;
}
                 void _ttc_radio_task_rx(void* Argument) {
    ttc_radio_generic_t* Radio_Generic = (ttc_radio_generic_t*) Argument;
    ttc_memory_block_t* Block = NULL;
    ttc_radio_errorcode_e Error = tre_OK;

    while (1) {
#ifdef EXTENSION_500_ttc_watchdogs
        ttc_watchdog_reload1();
#endif
        if (1) { //D

            // wait for interrupt service routine to unlock rx-mutex
            if (ttc_mutex_lock(Radio_Generic->MutexPacketReceived, 200000) != tqe_OK) {
#if TTC_DEBUG_RADIO==1
                if (1) ttc_gpio_set(TTC_DEBUG_RADIO_PIN3); //D
#endif
                if (_ttc_radio_check_mode(Radio_Generic) ) { // did not receive anything for a while: check radio
                    Error = ttc_radio_set_operating_mode(Radio_Generic->LogicalIndex, Radio_Generic->Mode); // radio in unexpected mode: try to correct
                    Assert_Radio(Error == tre_OK, ec_TimeOut);// could not fix unexpected radio mode


                }
#if TTC_DEBUG_RADIO==1
                if (1) ttc_gpio_clr(TTC_DEBUG_RADIO_PIN3);
#endif
            }
            else {
#if TTC_DEBUG_RADIO==1
                ttc_gpio_set(TTC_DEBUG_RADIO_PIN1); //D
#endif
                do {
                    if (!Block) {
                        Block = ttc_radio_get_empty_block();
                        Assert_Radio(Block, ec_NULL);
                    }
                    Block = _ttc_radio_read_packet(Radio_Generic, Block);
                } while (_ttc_radio_check_bytes_received(Radio_Generic));
                if (1) ttc_task_check_stack();
#if TTC_DEBUG_RADIO==1
                ttc_gpio_clr(TTC_DEBUG_RADIO_PIN1);
#endif
            }
        } //D
        else{ //D
            ttc_task_msleep(10000); //D
        } //D
    }
                 }
                 void _ttc_radio_task_tx(void* Argument) {
    ttc_radio_generic_t* Radio_Generic = (ttc_radio_generic_t*) Argument;

    ttc_QueueHandle_Pointers_t Queue_Tx = Radio_Generic->Queue_Tx;
    ttc_memory_block_t*        Block;
    Base_t MaxPayloadSize = Radio_Generic->Max_Payload;
    Base_t BytesRemaining;
    u16_t  Bytes2Send;
    u8_t*  Reader;

    while (1) {
        while (ttc_queue_pointer_pull_front(Queue_Tx, (void**) &Block) == tqe_OK) {
            Reader = Block->Buffer;
            BytesRemaining = Block->Size;
#if TTC_DEBUG_RADIO==1
                if (1) ttc_gpio_set(TTC_DEBUG_RADIO_PIN2); //D
#endif
            while (BytesRemaining > 0) { // send out packets until complete block has been transmitted
                if (BytesRemaining > MaxPayloadSize)
                    Bytes2Send = MaxPayloadSize;
                else
                    Bytes2Send = BytesRemaining;

                if (Radio_Generic->TxFunction_UCB)
                    Radio_Generic->TxFunction_UCB(Radio_Generic, Bytes2Send, Reader);

                ttc_radio_errorcode_e Error = tre_OK;
                ttc_radio_set_amplifier(Radio_Generic->LogicalIndex, tram_TransmitHighGain);
                switch (Radio_Generic->Driver) { // transmit single packet

#ifdef EXTENSION_400_radio_em357
                case trd_em357: {
                    Error = radio_em357_packet_send(Radio_Generic, Block->Hint, Bytes2Send, Reader);
                    break;
                }
#endif
#ifdef EXTENSION_400_radio_cc1101_spi
                case trd_cc1101_spi: {
                    Error = radio_cc1101_packet_send(Radio_Generic, Block->Hint, Bytes2Send, Reader);
                    break;
                }
#endif
#ifdef EXTENSION_400_radio_cc1120_spi
                case trd_cc1120_spi: {
                    Error = radio_cc1120_packet_send(Radio_Generic, Block->Hint, Bytes2Send, Reader);
                    break;
                }
#endif

                default:
                    mSleep(200); // Simulate transmission
                }
                if (Error != tre_OK) { // handle error condition
                    Assert_Radio(FALSE, ec_UNKNOWN); // ToDo: proper generic error handling
                }

                BytesRemaining -= Bytes2Send;
                Reader += Bytes2Send;
            }
            ttc_radio_set_amplifier(Radio_Generic->LogicalIndex, tram_ReceiveHighGain);

            // send block back to its originator (mostly _ttc_radio_release_block())
            ttc_memory_block_use(Block);
            Block->UseCount = 1; //D
            ttc_memory_block_release(Block);

#if TTC_DEBUG_RADIO==1
            if (1) ttc_gpio_clr(TTC_DEBUG_RADIO_PIN2); //D
#endif
        }
        ttc_radio_task_yield();
        ttc_task_check_stack();
    }
}
  ttc_memory_block_t* _ttc_radio_read_packet(ttc_radio_generic_t* Radio_Generic, ttc_memory_block_t* Block) {
    Assert_Radio(Radio_Generic, ec_NULL);
    Assert_Radio(Block, ec_NULL);

    u8_t RSSI = 0;
    //ToDo: add to Radio_Generic->Metainformation.RSSI
#ifdef EXTENSION_400_radio_em357
    if (Radio_Generic->Driver == trd_em357) {
        Block->Size = radio_em357_packet_receive(Radio_Generic, Block->Buffer, Block->MaxSize, &(Block->Hint), &RSSI);
        Assert_Radio(Block->Size < TTC_NETWORK_MAX_PAYLOAD_SIZE, ec_InvalidArgument); // got invalid block size (maybe corrupted data?)
    }
#endif
#ifdef EXTENSION_400_radio_cc1101_spi
    if (Radio_Generic->Driver == trd_cc1101_spi) {
        Block->Size = radio_cc1101_packet_receive(Radio_Generic, Block->Buffer, Block->MaxSize, &(Block->Hint), &RSSI);
        Assert_Radio(Block->Size < TTC_NETWORK_MAX_PAYLOAD_SIZE, ec_InvalidArgument); // got invalid block size (maybe corrupted data?)
    }
#endif
#ifdef EXTENSION_400_radio_cc1120_spi
    if (Radio_Generic->Driver == trd_cc1120_spi) {
        Block->Size = radio_cc1120_packet_receive(Radio_Generic, Block->Buffer, Block->MaxSize, &(Block->Hint), &RSSI);
        Assert_Radio(Block->Size < TTC_NETWORK_MAX_PAYLOAD_SIZE, ec_InvalidArgument); // got invalid block size (maybe corrupted data?)
    }
#endif
    if (Radio_Generic->RxFunction_UCB) {
        Block = Radio_Generic->RxFunction_UCB(Radio_Generic, Block, RSSI, Block->Hint);
    }

    return Block;
}

//} Function definitions
