#ifndef TTC_RANDOM_H
#define TTC_RANDOM_H

/** stm32_random.h ***********************************************
 
 * Written by Gregor Rebel 2010-2012
 *
 * Architecture independent random number generation
 * 
*/
//{ Defines/ TypeDefs ****************************************************

// To be defined by architecture driver
// RAND_MAX
//     E.g.: #define RAND_MAX  (u32_t) -1

//} Defines

//{ Includes *************************************************************

#include "ttc_basic.h"
#include "ttc_spi_types.h"

#ifdef TARGET_ARCHITECTURE_STM32
  #include "stm32_random.h"
#endif

#ifndef RAND_MAX
#define RAND_MAX  65535
#endif

//} Includes

//{ Function prototypes **************************************************

/** returns random number 0..RAND_MAX
 * @return random number 0..RAND_MAX
 */
// u32_t ttc_rand(void);
#define ttc_rand() _ttc_rand_16()

/** initializes pseudo random number generator with given value analog to rand()
 * @param Seed  will be used as new starting point
 */
// void ttc_srand(u32_t Seed);
#define ttc_srand(Seed) _ttc_srand_16(Seed)

/** returns 16-bit random number 0..RAND_MAX
 *
 * Note: Do not directly call this function, but use macro ttc_rand() instead!
 *
 * @return random number 0..RAND_MAX
 */
u16_t _ttc_rand_16(void);

/** initializes 16-bit pseudo random number generator with given value analog to srand()
 *
 * Note: Do not directly call this function, but use macro ttc_srand() instead!
 *
 * @param Seed  will be used as new starting point
 */
void _ttc_srand_16(u32_t Seed);

//}Function prototypes
#endif
