/*{ Template_FreeRTOS::.c ************************************************

     Several different boxes run over the screen.
     Each box is controlled by one task.
     
     written 2012 by Ioanna Tsekoura, Gregor Rebel
     
}*/

#include "example_gfx_boxes.h"

//{ Function definitions *************************************************

ttc_gfx_generic_t* DisplayConfig = NULL;
u32_t ColorBg = GFX_COLOR24_WHITE;

// size of area in which to draw boxes (start at (0,0))
u16_t DrawAreaWidth  = 0;
u16_t DrawAreaHeight = 0;

void gfx_boxes_initPorts() {

    u8_t MaxDisplay = ttc_gfx_get_max_display_index();
    Assert(MaxDisplay > 0, ec_DeviceNotFound); // no displays defined by board makefile!
    DisplayConfig = ttc_gfx_get_configuration(1);
    Assert(DisplayConfig, ec_DeviceNotFound);

    // set color to use for first screen clear
    ttc_gfx_set_color_bg24(ColorBg);

    ttc_gfx_init(1);

    // allow text display only in last line of display
    DisplayConfig->TextMinY = DisplayConfig->Height - DisplayConfig->Font->CharHeight;

    DrawAreaWidth  = DisplayConfig->Width;
    DrawAreaHeight = DisplayConfig->TextMinY;

    // print text in last line
    ttc_gfx_text_cursor_set(0,0);
    ttc_gfx_print("Boxes/s:", -1);
}
void lcd_start_boxes() {

    xQueueHandle DrawQueue = xQueueCreate(10, sizeof(DrawData_t));
    Assert(NULL != DrawQueue, ec_Malloc);

    gfx_boxes_initPorts();
    ttc_srand(32);

    for (s16_t I = 1; I < 10; I++) {
        DrawData_t* Box = newRandomBox(DrawQueue);
        Assert(Box != NULL, ec_Malloc);
    }

    ttc_task_create(tDrawIt,                       // function to start as thread
                    "tDrawIt",                             // thread name (just for debugging)
                    1 * TTC_TASK_MINIMAL_STACK_SIZE,          // stack size
                    (void *) DrawQueue,                    // passed as argument to taskLEDs()
                    1,                                     // task priority (higher values mean more process time)
                    (void*) NULL                           // can return a handle to created task
                    );

}
DrawData_t* newRandomBox(xQueueHandle DrawingQueue) {
    ttc_gfx_set_color_fg24( ((ttc_rand() & 0xff) << 16) | ((ttc_rand() & 0xff) << 8) | (ttc_rand() & 0xff) );

    return newBox( DrawingQueue,
                   DisplayConfig->ColorFg,
                   0 + ttc_rand() % DrawAreaWidth,
                   0 + ttc_rand() % DrawAreaWidth,
                   3 + ttc_rand() % 20,
                   3 + ttc_rand() % 20,
                   (ttc_rand() % 20) - 10,
                   (ttc_rand() % 20) - 10
                 );
}
DrawData_t* newBox(xQueueHandle DrawingQueue, u16_t Color, s16_t X, s16_t Y, u8_t Width, u8_t Height, s8_t SpeedX, s8_t SpeedY) {
    drawTaskArgs_t* NewBox = (drawTaskArgs_t*) ttc_memory_alloc( sizeof(drawTaskArgs_t) );

    if (NewBox != NULL) {
        NewBox->DrawQueue = DrawingQueue;
        NewBox->Box2Draw.Type = ddt_MovingBox;
        NewBox->Box2Draw.Polygon.MovingBox.Color  = Color;
        NewBox->Box2Draw.Polygon.MovingBox.PosX   = X;
        NewBox->Box2Draw.Polygon.MovingBox.PosY   = Y;
        NewBox->Box2Draw.Polygon.MovingBox.Width  = Width;
        NewBox->Box2Draw.Polygon.MovingBox.Height = Height;
        NewBox->Box2Draw.Polygon.MovingBox.SpeedX = SpeedX;
        NewBox->Box2Draw.Polygon.MovingBox.SpeedY = SpeedY;

        if ( ttc_task_create(tMoveBox,                      // function to start as thread
                        (const char*) NULL,                    // thread name (just for debugging)
                        (size_t) 1 * TTC_TASK_MINIMAL_STACK_SIZE, // stack size
                        (void *) NewBox,                       // passed as argument to task-function
                        (unsigned portBASE_TYPE) 1,        // task priority (higher values mean more process time)
                        (xTaskHandle*) NULL                // can return a handle to created task
                        )
             )
            return &(NewBox->Box2Draw);
    }

    // Error: could not create task (perhaps out of memory)
    return NULL;
}
void tMoveBox(void *pvParameters) {

    drawTaskArgs_t* Arguments = (drawTaskArgs_t*) pvParameters;
    xQueueHandle DrawQueue = Arguments->DrawQueue;

    Assert(NULL != DrawQueue, ec_InvalidArgument);

    DrawData_t* DrawMovingBox = &(Arguments->Box2Draw);
    elb_MovingBox_t* MovingBox = &(DrawMovingBox->Polygon.MovingBox);

    DrawData_t DrawClearBox;
    DrawClearBox.Type = ddt_MovingBox;
    elb_MovingBox_t* ClearBox = &(DrawClearBox.Polygon.MovingBox);
    ClearBox->Color  = ColorBg;
    ClearBox->SpeedX = 0;
    ClearBox->SpeedY = 0;

    DrawData_t DrawClearLPolygon;
    DrawClearLPolygon.Type = ddt_L_Polygon;
    elb_L_Polygon_t* ClearLPolygon = &(DrawClearLPolygon.Polygon.L_Polygon);
    ClearLPolygon->Color  = ColorBg;

    DrawData_t DrawLPolygon;
    DrawLPolygon.Type = ddt_L_Polygon;
    elb_L_Polygon_t* LPolygon = &(DrawLPolygon.Polygon.L_Polygon);

    queueInsert(DrawQueue, DrawMovingBox); // Drawing of the Box(es) in the initial position(s).
    u16_t CountDown = 0;
    u8_t UseImplementation = 0;

    while (1) {

        s16_t PosX = MovingBox->PosX;
        s16_t PosY = MovingBox->PosY;
        s16_t NewPosX = PosX + MovingBox->SpeedX;
        s16_t NewPosY = PosY + MovingBox->SpeedY;

        s16_t Border_Right = DrawAreaWidth  - MovingBox->Width;
        s16_t Border_Lower = DrawAreaHeight - MovingBox->Height;

        if (NewPosX > Border_Right) {
            MovingBox->SpeedX = -MovingBox->SpeedX;
            s16_t Excess = NewPosX - Border_Right;
            NewPosX -= Excess;
        }
        else if (NewPosX < 0) {
            MovingBox->SpeedX = -MovingBox->SpeedX;
            NewPosX = -NewPosX;
        }
        if (NewPosY > Border_Lower) {
            MovingBox->SpeedY = -MovingBox->SpeedY;
            s16_t Excess = NewPosY - Border_Lower;
            NewPosY -= Excess;
        }
        else if (NewPosY < 0) {
            MovingBox->SpeedY = -MovingBox->SpeedY;
            NewPosY = -NewPosY;
        }

        if (CountDown == 0) {
            UseImplementation = 1 - UseImplementation; // switch implementation
            CountDown = 100; // switch every 3 seconds
            ClearBox->PosX = 0;
            ClearBox->PosY = 0;
            ClearBox->Width  = 5;
            ClearBox->Height = 5;
            if (UseImplementation)  ClearBox->Color  = GFX_COLOR24_BLUE;
            else                    ClearBox->Color  = GFX_COLOR24_OLIVE;
            queueInsert(DrawQueue, &DrawClearBox);
            ClearBox->Color  = ColorBg;
        }
        CountDown--;
        if (UseImplementation) { // 1st implementation, clear the previous box and redraw all of it
            ClearBox->PosX = PosX;
            ClearBox->PosY = PosY;
            ClearBox->Width  = MovingBox->Width;
            ClearBox->Height = MovingBox->Height;
            queueInsert(DrawQueue, &DrawClearBox);

            MovingBox->PosX = NewPosX;
            MovingBox->PosY = NewPosY;
            queueInsert(DrawQueue, DrawMovingBox); // draw box as indicator of implementation switch
        }
        else {                   // 2nd implementation: clear as few pixels as possible

            if ((MovingBox->PosX==NewPosX)&&(MovingBox->PosY==NewPosY)) {
                // clear nothing
                MovingBox->PosX=NewPosX;
                MovingBox->PosY=NewPosY;
                queueInsert(DrawQueue, DrawMovingBox);
            }
            else if ((MovingBox->PosX==NewPosX)&&(MovingBox->PosY<NewPosY)) {
                if ((MovingBox->PosY+MovingBox->Height)<NewPosY) {
                    ClearBox->PosX   = PosX;
                    ClearBox->PosY   = PosY;
                    ClearBox->Width  = MovingBox->Width;
                    ClearBox->Height = MovingBox->Height;
                    queueInsert(DrawQueue, &DrawClearBox);

                    MovingBox->PosX=NewPosX;
                    MovingBox->PosY=NewPosY;
                    queueInsert(DrawQueue, DrawMovingBox);
                }
                else {
                    ClearBox->PosX   = PosX;
                    ClearBox->PosY   = PosY;
                    ClearBox->Width  = MovingBox->Width;
                    ClearBox->Height = NewPosY - PosY;
                    queueInsert(DrawQueue, &DrawClearBox);

                    MovingBox->PosX=NewPosX;
                    MovingBox->PosY=NewPosY;
                    queueInsert(DrawQueue, DrawMovingBox);
                }
            }
            else if ((MovingBox->PosX==NewPosX)&&(MovingBox->PosY>NewPosY)) {
                if ((NewPosY+MovingBox->Height)<MovingBox->PosY) {
                    ClearBox->PosX   = PosX;
                    ClearBox->PosY   = PosY;
                    ClearBox->Width  = MovingBox->Width;
                    ClearBox->Height = MovingBox->Height;
                    queueInsert(DrawQueue, &DrawClearBox);

                    MovingBox->PosX=NewPosX;
                    MovingBox->PosY=NewPosY;
                    queueInsert(DrawQueue, DrawMovingBox);
                }
                else {
                    ClearBox->PosX   = PosX;
                    ClearBox->PosY   = NewPosY+MovingBox->Height;
                    ClearBox->Width  = MovingBox->Width;
                    ClearBox->Height = PosY-NewPosY;
                    queueInsert(DrawQueue, &DrawClearBox);

                    MovingBox->PosX=NewPosX;
                    MovingBox->PosY=NewPosY;
                    queueInsert(DrawQueue, DrawMovingBox);
                }
            }
            else if ((MovingBox->PosX<NewPosX)&&(MovingBox->PosY==NewPosY)) {
                if ((MovingBox->PosX+MovingBox->Width)<NewPosX) {
                    ClearBox->PosX   = PosX;
                    ClearBox->PosY   = PosY;
                    ClearBox->Width  = MovingBox->Width;
                    ClearBox->Height = MovingBox->Height;
                    queueInsert(DrawQueue, &DrawClearBox);

                    MovingBox->PosX=NewPosX;
                    MovingBox->PosY=NewPosY;
                    queueInsert(DrawQueue, DrawMovingBox);
                }
                else {
                    ClearBox->PosX   = PosX;
                    ClearBox->PosY   = PosY;
                    ClearBox->Width  = NewPosX-PosX;
                    ClearBox->Height = MovingBox->Height;
                    queueInsert(DrawQueue, &DrawClearBox);

                    MovingBox->PosX=NewPosX;
                    MovingBox->PosY=NewPosY;
                    queueInsert(DrawQueue, DrawMovingBox);
                }
            }
            else if ((MovingBox->PosX>NewPosX)&&(MovingBox->PosY==NewPosY)) {
                if ((NewPosX+MovingBox->Width)<MovingBox->PosX) {
                    ClearBox->PosX   = PosX;
                    ClearBox->PosY   = PosY;
                    ClearBox->Width  = MovingBox->Width;
                    ClearBox->Height = MovingBox->Height;
                    queueInsert(DrawQueue, &DrawClearBox);

                    MovingBox->PosX=NewPosX;
                    MovingBox->PosY=NewPosY;
                    queueInsert(DrawQueue, DrawMovingBox);
                }
                else {
                    ClearBox->PosX   = NewPosX+MovingBox->Width;
                    ClearBox->PosY   = PosY;
                    ClearBox->Width  = PosX-NewPosX;
                    ClearBox->Height = MovingBox->Height;
                    queueInsert(DrawQueue, &DrawClearBox);

                    MovingBox->PosX=NewPosX;
                    MovingBox->PosY=NewPosY;
                    queueInsert(DrawQueue, DrawMovingBox);
                }
            }

            else if ((MovingBox->PosX<NewPosX)&&(MovingBox->PosY<NewPosY)) { /*Left to Right. Top to Bottom.*/
                if ((NewPosX>MovingBox->PosX+MovingBox->Width)||(NewPosY>MovingBox->PosY+MovingBox->Height)) {
                    ClearBox->PosX   = PosX;
                    ClearBox->PosY   = PosY;
                    ClearBox->Width  = MovingBox->Width;
                    ClearBox->Height = MovingBox->Height;
                    queueInsert(DrawQueue, &DrawClearBox);

                    MovingBox->PosX=NewPosX;
                    MovingBox->PosY=NewPosY;
                    queueInsert(DrawQueue, DrawMovingBox);
                }
                else {
                    ClearLPolygon->ExternalX = PosX;
                    ClearLPolygon->ExternalY = PosY;
                    ClearLPolygon->Width     = MovingBox->Width;
                    ClearLPolygon->Height    = MovingBox->Height;
                    ClearLPolygon->InternalX = NewPosX;
                    ClearLPolygon->InternalY = NewPosY;
                    queueInsert(DrawQueue, &DrawClearLPolygon);

                    LPolygon->Color = MovingBox->Color;
                    LPolygon->ExternalX = NewPosX+MovingBox->Width;
                    LPolygon->ExternalY = NewPosY+MovingBox->Height;
                    LPolygon->Width     = MovingBox->Width;
                    LPolygon->Height    = MovingBox->Height;
                    LPolygon->InternalX = PosX+MovingBox->Width;
                    LPolygon->InternalY = PosY+MovingBox->Height;
                    queueInsert(DrawQueue, &DrawLPolygon);
                    MovingBox->PosX=NewPosX;
                    MovingBox->PosY=NewPosY;
                }
            }
            else if ((MovingBox->PosX<NewPosX)&&(MovingBox->PosY>NewPosY)) { /*Left to Right. Bottom to Top*/
                if ((NewPosX>MovingBox->PosX+MovingBox->Width)||((NewPosY+MovingBox->Height)<MovingBox->PosY)) {

                    ClearBox->PosX   = PosX;
                    ClearBox->PosY   = PosY;
                    ClearBox->Width  = MovingBox->Width;
                    ClearBox->Height = MovingBox->Height;
                    queueInsert(DrawQueue, &DrawClearBox);

                    MovingBox->PosX=NewPosX;
                    MovingBox->PosY=NewPosY;
                    queueInsert(DrawQueue, DrawMovingBox);
                }
                else {
                    ClearLPolygon->ExternalX = PosX;
                    ClearLPolygon->ExternalY = PosY+MovingBox->Height;
                    ClearLPolygon->Width     = MovingBox->Width;
                    ClearLPolygon->Height    = MovingBox->Height;
                    ClearLPolygon->InternalX = NewPosX;
                    ClearLPolygon->InternalY = NewPosY+MovingBox->Height;
                    queueInsert(DrawQueue, &DrawClearLPolygon);

                    LPolygon->Color = MovingBox->Color;
                    LPolygon->ExternalX = NewPosX+MovingBox->Width;
                    LPolygon->ExternalY = NewPosY;
                    LPolygon->Width     = MovingBox->Width;
                    LPolygon->Height    = MovingBox->Height;
                    LPolygon->InternalX = PosX+MovingBox->Width;
                    LPolygon->InternalY = PosY;
                    queueInsert(DrawQueue, &DrawLPolygon);
                    MovingBox->PosX=NewPosX;
                    MovingBox->PosY=NewPosY;
                }
            }
            else if ((MovingBox->PosX>NewPosX)&&(MovingBox->PosY<NewPosY)) { /*Right to Left. Top to Bottom*/
                if (((NewPosX+MovingBox->Width)<MovingBox->PosX)||(NewPosY>MovingBox->PosY+MovingBox->Height)) {
                    ClearBox->PosX   = PosX;
                    ClearBox->PosY   = PosY;
                    ClearBox->Width  = MovingBox->Width;
                    ClearBox->Height = MovingBox->Height;
                    queueInsert(DrawQueue, &DrawClearBox);

                    MovingBox->PosX=NewPosX;
                    MovingBox->PosY=NewPosY;
                    queueInsert(DrawQueue, DrawMovingBox);
                }
                else {
                    ClearLPolygon->ExternalX = PosX+MovingBox->Width;
                    ClearLPolygon->ExternalY = PosY;
                    ClearLPolygon->Width     = MovingBox->Width;
                    ClearLPolygon->Height    = MovingBox->Height;
                    ClearLPolygon->InternalX = NewPosX+MovingBox->Width;
                    ClearLPolygon->InternalY = NewPosY;
                    queueInsert(DrawQueue, &DrawClearLPolygon);

                    LPolygon->Color     = MovingBox->Color;
                    LPolygon->ExternalX = NewPosX;
                    LPolygon->ExternalY = NewPosY+MovingBox->Height;
                    LPolygon->Width     = MovingBox->Width;
                    LPolygon->Height    = MovingBox->Height;
                    LPolygon->InternalX = PosX;
                    LPolygon->InternalY = PosY+MovingBox->Height;
                    queueInsert(DrawQueue, &DrawLPolygon);
                    MovingBox->PosX=NewPosX;
                    MovingBox->PosY=NewPosY;
                }
            }
            else if ((MovingBox->PosX>NewPosX)&&(MovingBox->PosY>NewPosY)) {
                if (((NewPosX+MovingBox->Width)<MovingBox->PosX)||((NewPosY+MovingBox->Height)<MovingBox->PosY)) {
                    ClearBox->PosX   = PosX;
                    ClearBox->PosY   = PosY;
                    ClearBox->Width  = MovingBox->Width;
                    ClearBox->Height = MovingBox->Height;
                    queueInsert(DrawQueue, &DrawClearBox);

                    MovingBox->PosX=NewPosX;
                    MovingBox->PosY=NewPosY;
                    queueInsert(DrawQueue, DrawMovingBox);
                }
                else {
                    ClearLPolygon->ExternalX = PosX+MovingBox->Width;
                    ClearLPolygon->ExternalY = PosY+MovingBox->Height;
                    ClearLPolygon->Width     = MovingBox->Width;
                    ClearLPolygon->Height    = MovingBox->Height;
                    ClearLPolygon->InternalX = NewPosX+MovingBox->Width;
                    ClearLPolygon->InternalY = NewPosY+MovingBox->Height;
                    queueInsert(DrawQueue, &DrawClearLPolygon);

                    LPolygon->Color     = MovingBox->Color;
                    LPolygon->ExternalX = NewPosX;
                    LPolygon->ExternalY = NewPosY;
                    LPolygon->Width     = MovingBox->Width;
                    LPolygon->Height    = MovingBox->Height;
                    LPolygon->InternalX = PosX;
                    LPolygon->InternalY = PosY;
                    queueInsert(DrawQueue, &DrawLPolygon);
                    MovingBox->PosX=NewPosX;
                    MovingBox->PosY=NewPosY;
                }
            }
        }

        mSleep(30);
    }
}
void queueInsert(xQueueHandle DrawQueue, DrawData_t *Box) {

    while (xQueueSend(DrawQueue, Box, (portTickType) -1) != pdTRUE) 
      ttc_task_yield();

}
void tDrawIt(void *pvParameters) {
    xQueueHandle DrawQueue= (xQueueHandle) pvParameters;
    Assert(NULL != DrawQueue, ec_InvalidArgument);
    u16_t TimeSecs1 = ttc_task_get_tick_count() / 1000000;
    u16_t TimeSecs2 = TimeSecs1;
    u16_t ElapsedSecs = -1;
    Base_t PolygonCount = 0;

    static u8_t String[20];

    while (1) {
        DrawData_t Box2Draw;
        while ( xQueueReceive(DrawQueue, &Box2Draw, (portTickType) -1) ) {
            PolygonCount++;
            if (Box2Draw.Type == ddt_MovingBox) {
                ttc_gfx_set_color_fg16(Box2Draw.Polygon.MovingBox.Color);

                ttc_gfx_rect_fill(Box2Draw.Polygon.MovingBox.PosX,
                                  Box2Draw.Polygon.MovingBox.PosY,
                                  Box2Draw.Polygon.MovingBox.Width,
                                  Box2Draw.Polygon.MovingBox.Height
                                  );
            }
            else {
                ttc_gfx_set_color_fg16(Box2Draw.Polygon.L_Polygon.Color);

                LCD_DrawLpolygon(Box2Draw.Polygon.L_Polygon.ExternalX,
                                 Box2Draw.Polygon.L_Polygon.ExternalY,
                                 Box2Draw.Polygon.L_Polygon.Width,
                                 Box2Draw.Polygon.L_Polygon.Height,
                                 Box2Draw.Polygon.L_Polygon.InternalX,
                                 Box2Draw.Polygon.L_Polygon.InternalY
                                );
            }
            TimeSecs2 = ttc_task_get_tick_count() / 1000000;
            if (TimeSecs1 != TimeSecs2) {
                if (ElapsedSecs == -1) { // first second passed: reset statistic
                    PolygonCount = 0;
                    ElapsedSecs  = 0;
                }
                else {
                    if (TimeSecs2 > TimeSecs1)
                        ElapsedSecs += TimeSecs2 - TimeSecs1;
                    else
                        ElapsedSecs += TimeSecs2; // ToDo: use correct calculation for tick-counter overflow

                    TimeSecs1 = TimeSecs2;
                    ttc_gfx_text_cursor_set(9, 0);
                    ttc_gfx_set_color_fg16(0);
                    if (ElapsedSecs) {
                        ttc_string_snprintf(String, 20, "%i", PolygonCount / ElapsedSecs);
                        ttc_gfx_print_solid( (char*) String, 20);
                    }
                }
            }
        }
        ttc_task_yield();
    }
}
void LCD_DrawLpolygon(s16_t ExternalX, s16_t ExternalY, s16_t Width, s16_t Height, s16_t InternalX, s16_t InternalY)
{
//    s16_t i;

    if ((ExternalX<InternalX)&&(ExternalY<InternalY)) {
        ttc_gfx_rect_fill(ExternalX, ExternalY, InternalX-ExternalX, Height);
//        s16_t k=ExternalY;
//        for(i=0; i<Height; i++, k++) {
//            u32 addr = k*0x100 + ExternalX;
//            LCD_FillMem(addr, Color, (InternalX-ExternalX));
//        }
        ttc_gfx_rect_fill(InternalX, ExternalY, ExternalX + Width - InternalX, InternalY-ExternalY);
//        k=ExternalY;
//        for(i=0; i<(InternalY-ExternalY); i++, k++) {
//            u32 addr = k*0x100 + InternalX;
//            LCD_FillMem(addr, Color, (ExternalX+Width-InternalX));
//        }
    }
    else if ((ExternalX<InternalX)&&(ExternalY>InternalY)) {
        ttc_gfx_rect_fill(ExternalX, ExternalY - Height, InternalX-ExternalX, Height);
//        s16_t k=ExternalY-Height;
//        for(i=0; i<Height; i++, k++) {
//            u32 addr = k*0x100 + ExternalX;
//            LCD_FillMem(addr, Color, (InternalX-ExternalX));
//        }
        ttc_gfx_rect_fill(InternalX, InternalY, ExternalX + Width - InternalX, ExternalY - InternalY);
//        k=InternalY;
//        for(i=0; i<(ExternalY-InternalY); i++, k++) {
//            u32 addr = k*0x100 + InternalX;
//            LCD_FillMem(addr, Color, (ExternalX+Width-InternalX));
//        }
    }
    else if ((ExternalX>InternalX)&&(ExternalY<InternalY)) {
        ttc_gfx_rect_fill(ExternalX - Width, ExternalY, Width, InternalY-ExternalY);
//        s16_t k=ExternalY;
//        for(i=0; i<(InternalY-ExternalY); i++, k++) {
//            u32 addr = k*0x100 + ExternalX-Width;
//            LCD_FillMem(addr, Color, Width);
//        }
        ttc_gfx_rect_fill(InternalX, InternalY, ExternalX-InternalX, ExternalY + Height - InternalY);
//        k=InternalY;
//        for(i=0; i<(ExternalY+Height-InternalY); i++, k++) {
//            u32 addr = k*0x100 + InternalX;
//            LCD_FillMem(addr, Color, (ExternalX-InternalX));
//        }
    }
    else {
        ttc_gfx_rect_fill(ExternalX - Width, InternalY, Width, ExternalY-InternalY);
//        s16_t k=InternalY;
//        for(i=0; i<(ExternalY-InternalY); i++, k++) {
//            u32 addr = k*0x100 + ExternalX-Width;
//            LCD_FillMem(addr, Color, Width);
//        }
        ttc_gfx_rect_fill(InternalX, ExternalY - Height, ExternalX-InternalX, InternalY + Height - ExternalY);
//        k=ExternalY-Height;
//        for(i=0; i<(InternalY+Height-ExternalY); i++, k++) {
//            u32 addr = k*0x100 + InternalX;
//            LCD_FillMem(addr, Color, (ExternalX-InternalX));
//        }
    }
}

//} Function definitions
