/*{ example_radio.c ******************************************************
 
  Basic test of radio transceivers without any protocol.

  Initializes first radio and periodically transmits data.
  Received data is echoed via TTC_USART1.

}*/

#include "example_radio.h"

//{ Function definitions *************************************************

// configuration data of single radio
ttc_radio_generic_t* MyRadio1;
ttc_radio_generic_t* MyRadio2;

// randomly chosen ID of this node
u8_t MyID = 0;
u8_t TxLedState = 0;
u8_t RxLedState = 0;

#define LED_TX TTC_LED1
#define LED_RX TTC_LED2

// disable individual radios
// #undef TTC_RADIO1
// #undef TTC_RADIO2

void example_radio_start() {
    example_radio_init();
#ifdef TTC_RADIO1
    ttc_task_create( taskRadio,                    // function to start as thread
                     "taskRadio1",                 // thread name (just for debugging)
                     TTC_TASK_MINIMAL_STACK_SIZE,  // stack size
                     (void *) 1,                   // passed as argument to taskRadio()
                     1,                            // task priority (higher values mean more process time)
                     (void *) NULL                 // can return a handle to created task
                   );
#endif
    if (1) ttc_task_msleep(500);
#ifdef TTC_RADIO2x
    ttc_task_create( taskRadio,                    // function to start as thread
                     "taskRadio2",                 // thread name (just for debugging)
                     TTC_TASK_MINIMAL_STACK_SIZE,  // stack size
                     (void *) 2,                   // passed as argument to taskRadio()
                     1,                            // task priority (higher values mean more process time)
                     (void *) NULL                 // can return a handle to created task
                   );
#endif
}
void example_radio_init() {
    ttc_gpio_init(LED_TX, tgm_output_push_pull);
    ttc_gpio_init(LED_RX, tgm_output_push_pull);
#ifdef TTC_RADIO1
    ttc_radio_init(1,0xB5);
    ttc_radio_register_tx_function(1, txFunction_R1);
    ttc_radio_register_rx_function(1, rxFunction_R1);
#endif
#ifdef TTC_RADIO2
    ttc_radio_init(2,0x00);
    ttc_radio_register_tx_function(2, txFunction_R2);
    ttc_radio_register_rx_function(2, rxFunction_R2);
#endif

#if TTC_DEBUG_RADIO==1
    ttc_gpio_init(TTC_DEBUG_RADIO_PIN1, tgm_output_push_pull);
    ttc_gpio_init(TTC_DEBUG_RADIO_PIN2, tgm_output_push_pull);
    ttc_gpio_init(TTC_DEBUG_RADIO_PIN3, tgm_output_push_pull);
#endif
    // initialize random number generator
    ttc_srand(ttc_task_get_tick_count());

    // create random id
    MyID = (u8_t) ttc_rand() & 0xff;

    // initialize first USART with default settings
    ttc_usart_generic_t * usart_generic = ttc_usart_get_configuration(1);
    usart_generic->Flags.Bits.DelayedTransmits = 1;
    ttc_usart_init(1);

    // set usart as stdout interface
    ttc_string_register_stdout(ttc_usart_stdout_send_block, ttc_usart_stdout_send_string);


#if TTC_DEBUG_RADIO==1
    ttc_string_printf("STM32 Example Radio");
#endif
}
void taskRadio(void *pvParameters) {

    u8_t RadioIndex = 0xff & (u32_t) pvParameters;
#define TEMP_SIZE 100
    u8_t Temp[TEMP_SIZE];
    u16_t TxCount = 0;
    ttc_radio_set_destination(RadioIndex, 255);

    while (1) {
        snprintf(Temp, TEMP_SIZE, "Hello World this is #%i.%i TX#%i\n", MyID, RadioIndex, TxCount++);
        ttc_radio_errorcode_e Error;
        if ( (Error = ttc_radio_send_string(RadioIndex, (char*) Temp, TEMP_SIZE)) != tre_OK)
            ttc_string_printf("ERROR - Cannot send text #%i via Radio #%i (Status=%i)\n", TxCount-1, RadioIndex, Error);
        mSleep((ttc_rand() &0xFF)+ 100);
    }
}
u8_t uart_tx_bypass_buffer[TTC_NETWORK_MAX_PAYLOAD_SIZE+20];
void txFunction_R1(ttc_radio_generic_t* RadioCfg, u8_t Size, const u8_t* Buffer) {
    Assert(RadioCfg!=NULL, ec_NULL);
    (void) RadioCfg;
    TxLedState = 1 - TxLedState;
    ttc_gpio_setn(LED_TX, TxLedState);
    static u8_t* Temp = NULL;
    static u8_t* Additional = NULL;
    if(Size>TTC_NETWORK_MAX_PAYLOAD_SIZE){
        Assert(NULL,ec_ArrayIndexOutOfBound);
    }
    if (0) { //D
        if (Temp == NULL) {
            Temp = ttc_memory_alloc(TTC_NETWORK_MAX_PAYLOAD_SIZE+10);
            Assert(Temp!=NULL, ec_Malloc);
            Additional = snprintf(Temp, TTC_NETWORK_MAX_PAYLOAD_SIZE, "TX->%i: ", RadioCfg->TargetAddress);
        }

        ttc_memory_copy(Additional, Buffer, Size);
        Additional[Size] = 0;
        ttc_string_print(Temp, TTC_NETWORK_MAX_PAYLOAD_SIZE+10);
    }//D
}
void txFunction_R2(ttc_radio_generic_t* RadioCfg, u8_t Size, const u8_t* Buffer) {
    (void) RadioCfg;
    TxLedState = 1 - TxLedState;
    ttc_gpio_setn(LED_TX, TxLedState);
    static u8_t* Temp = NULL;
    static u8_t* Additional = NULL;
    if (Temp == NULL) {
        Temp = ttc_memory_alloc(TTC_NETWORK_MAX_PAYLOAD_SIZE+10);
        Additional = snprintf(Temp, TTC_NETWORK_MAX_PAYLOAD_SIZE, "TX->%i: ", RadioCfg->TargetAddress);
    }

    ttc_memory_copy(Additional, Buffer, Size);
    Additional[Size] = 0;
    print(Temp, TTC_NETWORK_MAX_PAYLOAD_SIZE+10);
}
ttc_memory_block_t* rxFunction_R1(ttc_radio_generic_t* RadioCfg, ttc_memory_block_t* Block, u8_t RSSI, u8_t TargetAddress) {
    (void) RadioCfg;
    (void) Block;

    RxLedState = 1 - RxLedState;
    ttc_gpio_setn(LED_RX, RxLedState);

    static u8_t* Temp = NULL;
    static u8_t* Additional = NULL;
    static u16_t RxCount = 0;
    if (Temp == NULL) {
        Temp = ttc_memory_alloc(TTC_NETWORK_MAX_PAYLOAD_SIZE+10);
        Additional = snprintf(Temp, TTC_NETWORK_MAX_PAYLOAD_SIZE, "RX #%i.1<- #", MyID);
    }
    u8_t* Writer = snprintf(Additional, TTC_NETWORK_MAX_PAYLOAD_SIZE - (Additional - Temp), "%i: ", RxCount++);
    u8_t Bytes2Copy = Block->Size;
    u8_t BufferSize = TTC_NETWORK_MAX_PAYLOAD_SIZE - (Writer - Temp);
    while (Bytes2Copy > 0) {
        if (Bytes2Copy > BufferSize){
            ttc_memory_copy(Writer, Block->Buffer, BufferSize-1);
            Writer[BufferSize-1] = 0;
            Bytes2Copy -= BufferSize;
        }
        else {
            ttc_memory_copy(Writer, Block->Buffer, Bytes2Copy);
            Writer[Bytes2Copy] = 0;
            Bytes2Copy = 0;
        }
        ttc_string_print(Temp, TTC_NETWORK_MAX_PAYLOAD_SIZE+10);
    }

    return Block; // let radio reuse this block immediately
}
ttc_memory_block_t* rxFunction_R2(ttc_radio_generic_t* RadioCfg, ttc_memory_block_t* Block, u8_t RSSI, u8_t TargetAddress) {
    (void) RadioCfg;
    (void) Block;

    RxLedState = 1 - RxLedState;
    ttc_gpio_setn(LED_RX, RxLedState);

    static u8_t* Temp = NULL;
    static u8_t* Additional = NULL;
    static u16_t RxCount = 0;
    if (Temp == NULL) {
        Temp = ttc_memory_alloc(TTC_NETWORK_MAX_PAYLOAD_SIZE+10);
        Additional = snprintf(Temp, TTC_NETWORK_MAX_PAYLOAD_SIZE, "RX #%i.2<- #", MyID);
    }
    u8_t* Writer = snprintf(Additional, TTC_NETWORK_MAX_PAYLOAD_SIZE - (Additional - Temp), "%i: ", RxCount++);
    u8_t Bytes2Copy = Block->Size;
    u8_t BufferSize = TTC_NETWORK_MAX_PAYLOAD_SIZE - (Writer - Temp);
    while (Bytes2Copy > 0) {
        if (Bytes2Copy > BufferSize){
            ttc_memory_copy(Writer, Block->Buffer, BufferSize-1);
            Writer[BufferSize-1] = 0;
            Bytes2Copy -= BufferSize;
        }
        else {
            ttc_memory_copy(Writer, Block->Buffer, Bytes2Copy);
            Writer[Bytes2Copy] = 0;
            Bytes2Copy = 0;
        }
        ttc_string_print(Temp, TTC_NETWORK_MAX_PAYLOAD_SIZE+10);
    }
    return Block; // let radio reuse this block immediately
}

//} Function definitions

