#!/bin/bash

#
#  Install script for Fixpoint Mathematical Library 
#  URL: http://code.google.com/p/fixpointlib/
#
#  Initial Script written by Gregor Rebel 2010-2012.
# 
#  Published under GNU Lesser General Public License
#  Check LEGAL.txt for details.
#
# Tested successfully on:
# openSuSE 12.1 x86
# XUbuntu 11.10 x64

source scripts/installFuncs.sh

if [ "$USER" == "root" ]; then
  echo "$0: ERROR this script should not been run as user root!"
  exit 10
fi

INSTALLPATH="000_Content"
dir "$INSTALLPATH"
cd "$INSTALLPATH"

#{ IMPLEMENT YOUR INSTALL SCRIPT BELOW -----------------------------------------------

if [ ! -e OK.Docs ];    then    #{ download documentation

  Get_Fails=""
  
  # download doc-file + move to Documentation/ + create symbolic link into Documentation/ right here 
  # File="STM32F10x_Standard_Peripherals_Library.html"
  # getFile http://www.mikrocontroller.net/articles/ STM32F10x_Standard_Peripherals_Library $File
  # addDocumentationFile $File STM
  
  if [ "$Get_Fails" == "" ]; then #{ check get() inside scripts/installFuncs.sh 
    touch OK.Docs
  else
    echo "$0 - ERROR: missing files: $Get_Fails"
  fi #}
fi #}
if [ ! -e OK.Install ]; then    #{ download + install software
  if [ ! -e OK.Packages ]; then #{ install packages
    # installPackage libgmp3-dev
    
    touch OK.Packages
  fi #}
  if [ -e OK.Packages ];   then #{ install software
    echo "installing in $INSTALLPATH ..."
    InstallDir=`pwd`

    Version=`cat ../../Version`
    Archive=ToolChain_STM32.Contents_${Version}.tar.bz2
    get http://thetoolchain.com/mirror/ $Archive $Archive
    # unZIP "UNKNOWN" stm32_i2c_cpal.zip
    
    if [ -e "$Archive" ]; then
      echo "`pwd`> testing archive integrity: $Archive .."
      ArchiveCorrupt=`bzip2 -t $Archive 2>&1`
      if [ "$ArchiveCorrupt" == "" ]; then
        ArchDir=`pwd`
        ContentDir="$HOME/Source/ToolChain_STM32.Contents/"
        dir $ContentDir
        cd $ContentDir
        echo "extracting contents archive $ArchDir/$Archive -> $ContentDir ..."
        tar xjvf $ArchDir/$Archive && \
        cd $ArchDir
        echo "" >OK.Install
      else
        echo "Archive is corrupt: `pwd`/$Archive"
        mv $Archive ${Archive}.defekt
      fi
    fi
  else
    echo "$0 - ERROR: Packages were not installed correctly!"
  fi #}OK.Packages
fi #} [ ! -e OK.Install ]; then
if [   -e OK.Install ]; then    #{ create makefiles and activate scripts
  Name="${INSTALLPATH}"
  
  cd "$OldPWD"
  echo "Installed successfully: $INSTALLPATH"
#}
else                            #{ Install failed
  echo "failed to install $INSTALLPATH"
fi #}

cd ..
#} IMPLEMENT YOUR INSTALL SCRIPT ABOVE -----------------------------------------------

exit 0
