#!/bin/bash

#
#  Install script for Fixpoint Mathematical Library 
#  URL: http://code.google.com/p/fixpointlib/
#
#  Script written by Gregor Rebel 2010-2011.
# 
#  Feel free do distribute and change to your own needs!
#
# Tested successfully on:
# Ubuntu 10.04 Netbook Edition x86
# KUbuntu 11.04 x86
# openSuSE 11.4 x86
# XUbuntu 11.10 x64

source scripts/installFuncs.sh
                          
if [ "$USER" == "root" ]; then
  echo "$0: ERROR this script should not been run as user root!"
  exit 10
fi

INSTALLPATH="100_board_olimex_p107"
dir "$INSTALLPATH"
cd "$INSTALLPATH"

#{ IMPLEMENT YOUR INSTALL SCRIPT BELOW -----------------------------------------------
  Dir=`pwd`
  if [ ! -e OK.Docs ]; then #{
  
    Get_Fails=""
    dir ../../Documentation/Boards
    cd ../../Documentation/Boards
    getFile http://olimex.com/dev/pdf/ARM/ST/ STM32-P107.pdf Board_STM32-P107.pdf
    getFile http://pdf1.alldatasheet.com/datasheet-pdf/view/162452/STMICROELECTRONICS/STE101P/+0W3W7WVPSLpETSpPKIbaZZTOfqgjUppKk+/ datasheet.pdf Board_STM32-P107_EthernetPhy_STE101P.pdf
  
    cd "$Dir"
    if [ "$Get_Fails" == "" ]; then
      touch OK.Docs
    else
      echo "$0 - ERROR: missing files: $Get_Fails"
    fi
  fi #}

  Name="${INSTALLPATH}" #{
  createExtensionMakefileHead ${Name} #{
  File="../extensions/makefile.${Name}" 
  cat <<END_OF_MAKEFILE >>$File 

# activate certain parts of source-code
COMPILE_OPTS += -DEXTENSION_${Name}=1
COMPILE_OPTS += -DBOARD=$Name
BOARD=$Name

# define microcontroller used on board
STM32F10X_CL  =1
COMPILE_OPTS += -DUC_DEVICE=STM32F107VCT6
COMPILE_OPTS += -DUC_PACKAGE_PINS=100

# Note: Make sure to set correct sizes for RAM and Flash
#
# updateProjectFiles.pl will automatically create memory regions in configs/memory.ld 
# for every define of this format (xxx = name of region, n = numeric value)
# COMPILE_OPTS += -DTTC_MEMORY_xxx_SIZE_K=n
# COMPILE_OPTS += -DTTC_MEMORY_xxx_START=n
# COMPILE_OPTS += -DTTC_MEMORY_xxx_ACCESS=rxwa
#
COMPILE_OPTS += -DTTC_MEMORY_RAM_SIZE_K=64
COMPILE_OPTS += -DTTC_MEMORY_RAM_START=0x20000000
COMPILE_OPTS += -DTTC_MEMORY_RAM_ACCESS=rxw
COMPILE_OPTS += -DTTC_MEMORY_ROM_SIZE_K=256
COMPILE_OPTS += -DTTC_MEMORY_ROM_START=0x08000000
COMPILE_OPTS += -DTTC_MEMORY_ROM_ACCESS=rx

# Define port pins
# reference RevA: http://olimex.com/dev/pdf/ARM/ST/STM32-P107-Rev.A-schematic.pdf
# reference RevB: http://olimex.com/dev/ARM/ST/STM32-P107/STM32-P107-REV-B-SCH.pdf

COMPILE_OPTS += -DTTC_LED1=PIN_PC6
COMPILE_OPTS += -DTTC_LED2=PIN_PC7

COMPILE_OPTS += -DTTC_SWITCH1=PIN_PC13                # press button switch 1 (named Tamper)
COMPILE_OPTS += -DTTC_SWITCH1_TYPE=tgm_input_floating # -> ttc_gpio_types.h
COMPILE_OPTS += -DTTC_SWITCH1_LOWACTIVE=1
COMPILE_OPTS += -DTTC_SWITCH2=PIN_PA0                 # press button switch 2 (named WKUP)
COMPILE_OPTS += -DTTC_SWITCH2_TYPE=tgm_input_floating # -> ttc_gpio_types.h
COMPILE_OPTS += -DTTC_SWITCH2_LOWACTIVE=0

BoardRevA=1
#BoardRevB=1

ifdef BoardRevA  #{ USART pin configuration for Board rev.A
  COMPILE_OPTS += -DTTC_USART1=INDEX_USART3       # USART connected to RS232 connector 9-pin female
  COMPILE_OPTS += -DTTC_USART1_TX=PIN_PD8   # RS232 connector 9-pin female TX-pin
  COMPILE_OPTS += -DTTC_USART1_RX=PIN_PD9   # RS232 connector 9-pin female RX-pin
  COMPILE_OPTS += -DTTC_USART1_RTS=PIN_PD12 # RS232 connector 9-pin female RTS-pin
  COMPILE_OPTS += -DTTC_USART1_CTS=PIN_PD11 # RS232 connector 9-pin female CTS-pin
  
  COMPILE_OPTS += -DTTC_USART2=INDEX_USART2       # USART connected to 10-pin male header connector
  COMPILE_OPTS += -DTTC_USART2_TX=PIN_PD5   # RS232 on 10-pin male header connector TX-Pin
  COMPILE_OPTS += -DTTC_USART2_RX=PIN_PD6   # RS232 on 10-pin male header connector RX-Pin
  
#{ define pin configuration of GPIOs on the UEXT Conncector
  COMPILE_OPTS += -DUEXT1_USART_TX=PIN_PD5 # USART2
  COMPILE_OPTS += -DUEXT1_USART_RX=PIN_PD6 # USART2
  COMPILE_OPTS += -DUEXT1_I2C_SCL=PIN_PB6
  COMPILE_OPTS += -DUEXT1_I2C_SDA=PIN_PB7
  COMPILE_OPTS += -DUEXT1_SPI_MISO=PIN_PC11
  COMPILE_OPTS += -DUEXT1_SPI_MOSI=PIN_PC12
  COMPILE_OPTS += -DUEXT1_SPI_NSCK=PIN_PC10
  COMPILE_OPTS += -DUEXT1_SPI_NSS=PIN_PB15
  
  COMPILE_OPTS += -DUEXT1_SPI_INDEX=3
  COMPILE_OPTS += -DUEXT1_USART_INDEX=2
  COMPILE_OPTS += -DUEXT1_I2C_INDEX=1
#}
endif #}
ifdef BoardRevB  #{ USART pin configuration for Board rev.B
  COMPILE_OPTS += -DTTC_USART1=INDEX_USART2       # USART connected to RS232 connector 9-pin female
  COMPILE_OPTS += -DTTC_USART1_TX=PIN_PD8   # RS232 connector 9-pin female TX-pin
  COMPILE_OPTS += -DTTC_USART1_RX=PIN_PD9   # RS232 connector 9-pin female RX-pin
  COMPILE_OPTS += -DTTC_USART1_RTS=PIN_PD12 # RS232 connector 9-pin female RTS-pin
  COMPILE_OPTS += -DTTC_USART1_CTS=PIN_PD11 # RS232 connector 9-pin female CTS-pin
  
  COMPILE_OPTS += -DTTC_USART2=INDEX_USART3       # USART connected to 10-pin male header connector
  COMPILE_OPTS += -DTTC_USART2_TX=PIN_PD4   # RS232 on 10-pin male header connector TX-Pin
  COMPILE_OPTS += -DTTC_USART2_RX=PIN_PD5   # RS232 on 10-pin male header connector RX-Pin

#{ define pin configuration of GPIOs on the UEXT Conncector
  COMPILE_OPTS += -DUEXT1_USART_TX=PIN_PD8 # USART3
  COMPILE_OPTS += -DUEXT1_USART_RX=PIN_PD9 # USART3
  COMPILE_OPTS += -DUEXT1_I2C_SCL=PIN_PB8
  COMPILE_OPTS += -DUEXT1_I2C_SDA=PIN_PB9
  COMPILE_OPTS += -DUEXT1_SPI_MISO=PIN_PC11
  COMPILE_OPTS += -DUEXT1_SPI_MOSI=PIN_PC12
  COMPILE_OPTS += -DUEXT1_SPI_SCK=PIN_PC10
  COMPILE_OPTS += -DUEXT1_SPI_NSS=PIN_PB15
  
  COMPILE_OPTS += -DUEXT1_SPI_INDEX=3
  COMPILE_OPTS += -DUEXT1_USART_INDEX=3
  COMPILE_OPTS += -DUEXT1_I2C_INDEX=1
#}
endif #}

#{ parallel ports (8 or 16 consecutive pins in same bank)
#
# Every 8 available, consecutive bits in same GPIO bank can form an 8-bit wide port.
# A 16-bit wide port requires 16 available, consecutive bits (obviously).
# A pin is available if it not driven externally or used for something else.
# Check circuit of your board for details.

# 8-bit wide parallel port #1 starting at pin 0 of bank E
COMPILE_OPTS += -DTTC_PARALLEL08_1_BANK=GPIO_BANK_E
COMPILE_OPTS += -DTTC_PARALLEL08_1_FIRSTPIN=0

# 8-bit wide parallel port #2 starting at pin 8 of bank D (overlaps with USART2, USART3)
COMPILE_OPTS += -DTTC_PARALLEL08_2_BANK=GPIO_BANK_D
COMPILE_OPTS += -DTTC_PARALLEL08_2_FIRSTPIN=8


#}parallel ports
#{ SPI1
COMPILE_OPTS += -DTTC_SPI1=SPI3          # SPI connected to RS232 10-pin male header UEXT and SDCARD slot
COMPILE_OPTS += -DTTC_SPI1_MOSI=PIN_PC12 # UEXT pin 8
COMPILE_OPTS += -DTTC_SPI1_MISO=PIN_PC11 # UEXT pin 7
COMPILE_OPTS += -DTTC_SPI1_SCK=PIN_PC10  # UEXT pin 9
COMPILE_OPTS += -DTTC_SPI1_NSS=PIN_PB1   # UEXT pin 10 (SoftwareNSS)
#}SPI1
#{ I2C1 - define inter-integrated circuit interface #1
  COMPILE_OPTS += -DTTC_I2C1=I2C1           # internal device being used for this port
  COMPILE_OPTS += -DTTC_I2C1_SDA=PIN_PB9    # UEXT-6
  COMPILE_OPTS += -DTTC_I2C1_SCL=PIN_PB8    # UEXT-5
  COMPILE_OPTS += -DTTC_I2C1_SMBAL=PIN_PB5  # 
#}I2C1

END_OF_MAKEFILE
  createExtensionMakefileTail ${Name} #}
  File="../extensions/activate.${Name}.sh" #{
  createActivateScriptHead $File $0 "Protoboard P107 from Olimex -> http://olimex.com/dev/stm32-p107.html"
  cat <<END_OF_ACTIVATE >>$File #{

  rm 2>/dev/null \$Dir_ExtensionsActive/makefile.100_board_*
  
  # create links into extensions.active/
  createLink \$Dir_Extensions/makefile.$Name \$Dir_ExtensionsActive/makefile.$Name '' QUIET
  
  activate.200_cpu_stm32f10x.sh QUIET "\$0"

END_OF_ACTIVATE
#}
  createActivateScriptTail $File $Name
  #}

#}
  echo "Installed successfully: $INSTALLPATH"

cd ..
rmdir 2>/dev/null $INSTALLPATH
#} IMPLEMENT YOUR INSTALL SCRIPT ABOVE -----------------------------------------------

exit 0
