#!/bin/bash

#
#  Install script for Fixpoint Mathematical Library 
#  URL: http://code.google.com/p/fixpointlib/
#
#  Script written by Gregor Rebel 2010-2011.
# 
#  Feel free do distribute and change to your own needs!
#
# Tested successfully on:
# Ubuntu 10.04 Netbook Edition x86
# KUbuntu 11.04 x86
# openSuSE 11.4 x86
# XUbuntu 11.10 x64

source scripts/installFuncs.sh

if [ "$USER" == "root" ]; then
  echo "$0: ERROR this script should not been run as user root!"
  exit 10
fi

INSTALLPATH="100_board_olimex_h107"
dir "$INSTALLPATH"
cd "$INSTALLPATH"

#{ IMPLEMENT YOUR INSTALL SCRIPT BELOW -----------------------------------------------
  Dir=`pwd`
  if [ ! -e OK.Docs ]; then #{
    Get_Fails=""
    dir ../../Documentation/Boards

    getFile http://olimex.com/dev/pdf/ARM/ST/ STM32-H107.pdf Board_STM32-H107.pdf ../../Documentation/Boards/

    if [ "$Get_Fails" == "" ]; then
      touch OK.Docs
    else
      echo "$0 - ERROR: missing files: $Get_Fails"
    fi
  fi #}
  echo "installing in $INSTALLPATH ..."

  Name="${INSTALLPATH}" #{
  createExtensionMakefileHead ${Name} #{
  File="../extensions/makefile.${Name}" 
  cat <<END_OF_MAKEFILE >>$File 

  # activate certain parts of source-code
  COMPILE_OPTS += -DEXTENSION_${Name}=1
  COMPILE_OPTS += -DBOARD=$Name
  BOARD=$Name

  # define microcontroller used on board
  STM32F10X_CL  =1
  COMPILE_OPTS += -DUC_DEVICE=STM32F107VCT6
  COMPILE_OPTS += -DUC_PACKAGE_PINS=100

  # Note: Make sure to set correct sizes for RAM and Flash
  #
  # updateProjectFiles.pl will automatically create memory regions in configs/memory.ld 
  # for every define of this format (xxx = name of region, n = numeric value)
  # COMPILE_OPTS += -DTTC_MEMORY_xxx_SIZE_K=n
  # COMPILE_OPTS += -DTTC_MEMORY_xxx_START=n
  # COMPILE_OPTS += -DTTC_MEMORY_xxx_ACCESS=rxwa
  #
  COMPILE_OPTS += -DTTC_MEMORY_RAM_SIZE_K=64
  COMPILE_OPTS += -DTTC_MEMORY_RAM_START=0x20000000
  COMPILE_OPTS += -DTTC_MEMORY_RAM_ACCESS=rxw
  COMPILE_OPTS += -DTTC_MEMORY_ROM_SIZE_K=256
  COMPILE_OPTS += -DTTC_MEMORY_ROM_START=0x08000000
  COMPILE_OPTS += -DTTC_MEMORY_ROM_ACCESS=rx

  #{ single port pins
  # reference: http://olimex.com/dev/pdf/ARM/ST/STM32-H107-schematic.pdf

  COMPILE_OPTS += -DTTC_LED1=PIN_PC6
  COMPILE_OPTS += -DTTC_LED2=PIN_PC7
  
  COMPILE_OPTS += -DTTC_SWITCH1=PIN_PA0    # press button switch 1 (named WKUP)
  COMPILE_OPTS += -DTTC_SWITCH1_TYPE=tgm_input_floating # -> ttc_gpio_types.h
  COMPILE_OPTS += -DTTC_SWITCH1_LOWACTIVE=0

  #}port pins
  #{ parallel ports (8 or 16 consecutive pins in same bank)
  #
  # Every 8 available, consecutive bits in same GPIO bank can form an 8-bit wide port.
  # A 16-bit wide port requires 16 available, consecutive bits (obviously).
  # A pin is available if it not driven externally or used for something else.
  # Check circuit of your board for details.
  
  # 16-bit wide parallel port #1 using whole bank E (16-bit always starts a pin 0)
  COMPILE_OPTS += -DTTC_PARALLEL16_1_BANK=GPIO_BANK_E
  
  # 8-bit wide parallel port #1 starting at pin 0 of bank E
  COMPILE_OPTS += -DTTC_PARALLEL08_1_BANK=GPIO_BANK_E
  COMPILE_OPTS += -DTTC_PARALLEL08_1_FIRSTPIN=0

  # 8-bit wide parallel port #2 starting at pin 8 of bank E
  COMPILE_OPTS += -DTTC_PARALLEL08_2_BANK=GPIO_BANK_E
  COMPILE_OPTS += -DTTC_PARALLEL08_2_FIRSTPIN=8
  
  # 8-bit wide parallel port #3 starting at pin 0 of bank A
  COMPILE_OPTS += -DTTC_PARALLEL08_3_BANK=GPIO_BANK_A
  COMPILE_OPTS += -DTTC_PARALLEL08_3_FIRSTPIN=0
  
  # 8-bit wide parallel port #4 starting at pin 5 of bank B
  COMPILE_OPTS += -DTTC_PARALLEL08_4_BANK=GPIO_BANK_B
  COMPILE_OPTS += -DTTC_PARALLEL08_4_FIRSTPIN=5
  
  # 8-bit wide parallel port #5 starting at pin 8 of bank D
  COMPILE_OPTS += -DTTC_PARALLEL08_5_BANK=GPIO_BANK_D
  COMPILE_OPTS += -DTTC_PARALLEL08_5_FIRSTPIN=8
  
  
  #}parallel ports
  #{ USART1
  # Note: Default layout of USART1 is occupied by USB OTG
  COMPILE_OPTS += -DTTC_USART1=INDEX_USART1
  COMPILE_OPTS += -DTTC_USART1_TX=PIN_PB6
  COMPILE_OPTS += -DTTC_USART1_RX=PIN_PB7
  #}USART1
  #{ USART2
  COMPILE_OPTS += -DTTC_USART2=INDEX_USART2
  COMPILE_OPTS += -DTTC_USART2_TX=PIN_PA2
  COMPILE_OPTS += -DTTC_USART2_RX=PIN_PA3
  COMPILE_OPTS += -DTTC_USART2_CK=PIN_PA4
  COMPILE_OPTS += -DTTC_USART2_RTS=PIN_PA1
  # COMPILE_OPTS += -DTTC_USART2_CTS=PIN_PB13  # Note: pin used for WKUP-button
  #}USART2
  #{ USART3
  COMPILE_OPTS += -DTTC_USART3=INDEX_USART3
  COMPILE_OPTS += -DTTC_USART3_TX=PIN_PB10
  COMPILE_OPTS += -DTTC_USART3_RX=PIN_PB11
  COMPILE_OPTS += -DTTC_USART3_CK=PIN_PB12
  COMPILE_OPTS += -DTTC_USART3_CTS=PIN_PB13
  COMPILE_OPTS += -DTTC_USART3_RTS=PIN_PB14
  #}USART3
  #{ USART4
  COMPILE_OPTS += -DTTC_USART4=INDEX_USART4
  COMPILE_OPTS += -DTTC_USART4_TX=PIN_PC10
  COMPILE_OPTS += -DTTC_USART4_RX=PIN_PC11
  #}USART4
  #{ USART5
  COMPILE_OPTS += -DTTC_USART5=INDEX_USART5
  COMPILE_OPTS += -DTTC_USART5_TX=PIN_PC12
  COMPILE_OPTS += -DTTC_USART5_RX=PIN_PD2
  #}USART5
  #{ SPI1
  COMPILE_OPTS += -DTTC_SPI1=SPI1          # 
  COMPILE_OPTS += -DTTC_SPI1_MOSI=PIN_PA7  # EXT2-25
  COMPILE_OPTS += -DTTC_SPI1_MISO=PIN_PA6  # EXT2-26 
  COMPILE_OPTS += -DTTC_SPI1_SCK=PIN_PA5   # EXT2-27
  COMPILE_OPTS += -DTTC_SPI1_NSS=PIN_PA4   # EXT2-28
  #}SPI1
  #{ SPI2
  COMPILE_OPTS += -DTTC_SPI2=SPI2          # 
  COMPILE_OPTS += -DTTC_SPI2_MOSI=PIN_PB15 # EXT1-27
  COMPILE_OPTS += -DTTC_SPI2_MISO=PIN_PB14 # EXT1-26
  COMPILE_OPTS += -DTTC_SPI2_SCK=PIN_PB13  # EXT1-25
  COMPILE_OPTS += -DTTC_SPI2_NSS=PIN_PB12  # EXT1-24
  #}SPI2
  #{ I2C1 - define inter-integrated circuit interface #1
    COMPILE_OPTS += -DTTC_I2C1=I2C1           # internal device being used for this port
    COMPILE_OPTS += -DTTC_I2C1_SDA=PIN_PB7    # EXT1-19
    COMPILE_OPTS += -DTTC_I2C1_SCL=PIN_PB6    # EXT1-18
    COMPILE_OPTS += -DTTC_I2C1_SMBAL=PIN_PB5  # EXT1-17
  #}I2C1
  #{ I2C2 - define inter-integrated circuit interface #2
    COMPILE_OPTS += -DTTC_I2C2=I2C2           # internal device being used for this port
    COMPILE_OPTS += -DTTC_I2C2_SDA=PIN_PB11   # EXT1-23
    COMPILE_OPTS += -DTTC_I2C2_SCL=PIN_PB10   # EXT1-22
    COMPILE_OPTS += -DTTC_I2C2_SMBAL=PIN_PB12 # EXT1-24
  #}I2C2


END_OF_MAKEFILE
  createExtensionMakefileTail ${Name} #}
  File="../extensions/activate.${Name}.sh" #{
  createActivateScriptHead $File $0 "Protoboard H107 from Olimex -> http://olimex.com/dev/stm32-h107.html"
  cat <<END_OF_ACTIVATE >>$File #{

  rm 2>/dev/null \$Dir_ExtensionsActive/makefile.100_board_*
  
  # create links into extensions.active/
  createLink \$Dir_Extensions/makefile.$Name \$Dir_ExtensionsActive/makefile.$Name '' QUIET
  
  activate.200_cpu_stm32f10x.sh QUIET "\$0"

END_OF_ACTIVATE
#}
  createActivateScriptTail $File $Name
  #}

#}
  echo "Installed successfully: $INSTALLPATH"

cd ..
rmdir 2>/dev/null $INSTALLPATH
#} IMPLEMENT YOUR INSTALL SCRIPT ABOVE -----------------------------------------------

exit 0
