#!/bin/bash

#
#  Install script for protoboard Mini-STM32 v3.0 with 2,8" Color Touch Display
#  URL: http://code.google.com/p/fixpointlib/
#
#  Script written by Gregor Rebel 2010-2011.
# 
#  Feel free do distribute and change to your own needs!
#
# Tested successfully on:
# Ubuntu 10.04 Netbook Edition x86
# KUbuntu 11.04 x86
# openSuSE 11.4 x86
# XUbuntu 11.10 x64

source scripts/installFuncs.sh

if [ "$USER" == "root" ]; then
  echo "$0: ERROR this script should not been run as user root!"
  exit 10
fi

INSTALLPATH="100_board_mini_stm32"
dir "$INSTALLPATH"
cd "$INSTALLPATH"

#{ IMPLEMENT YOUR INSTALL SCRIPT BELOW -----------------------------------------------
  Dir=`pwd`
  dir ../../Documentation/Boards
  cd ../../Documentation/Boards
  
  dir Mini-STM32
  cd Mini-STM32
  DocDir=`pwd`
  if [ ! -e $Dir/OK.Docs ]; then #{
    Get_Fails=""
    getFile http://www.cutedigi.com/pub/ARMCORTEX/ STM32-FD-HX.pdf Schematics.pdf
    getFile http://www.rcos.eu/targets/stm32tfth-backend/Datasheets/ Mini%20STM32%20Entwicklungsboard%20Anleitung.pdf Mini-STM32_Entwicklungsboard_Anleitung.pdf
    if [ "$Get_Fails" == "" ]; then
      touch $Dir/OK.Docs
    else
      echo "$0 - ERROR: missing files: $Get_Fails"
    fi
  fi #}
  cd "$Dir"
  if [ ! -e OK.Install ]; then #{
    getFile "http://hlb-labor.de/cortexm3/ProtoBoards/MiniSTM32/" "FreeRTOS_on_MINI-STM32.tar.bz2"
    getFile "http://hlb-labor.de/cortexm3/ProtoBoards/MiniSTM32/" "MINI-STM32.tar.bz2"
    untgj FreeRTOS_on_MINI-STM32 FreeRTOS_on_MINI-STM32.tar.bz2
    untgj MINI-STM32 MINI-STM32.tar.bz2
  
    if [ -d FreeRTOS_on_MINI-STM32/ ]; then #{
      chmod -R +rw FreeRTOS_on_MINI-STM32
      if [ -d MINI-STM32/ ]; then
        chmod -R +rw MINI-STM32
        cd MINI-STM32/
        for File in "MINI-STM32 Schematic" "TFT Driver Information" "TouchScreen Information" "usermanual"; do
          echo "copying docmentation '$File'"
          if [ -d "$File" ]; then
            cp -R "$File" $DocDir/
            #echo "cp -Rv \"$File\" $DocDir/"
          fi
        done
        cd ..
        touch OK.Install
      fi
    fi #}
  fi #}
  if [ -e OK.Install ]; then #{
    for BoardRev in 20 30 40; do
      Name="${INSTALLPATH}_rev${BoardRev}"
      addLine ../scripts/createLinks.sh "rm 2>/dev/null $INSTALLPATH; ln -sv \$Source/ToolChain_STM32/ToolChain/$INSTALLPATH/FreeRTOS_on_MINI-STM32/Source/Mini_STM32/  $INSTALLPATH"
      createExtensionMakefileHead ${Name} #{
      File="../extensions/makefile.${Name}"
      cat <<END_OF_MAKEFILE >>$File 

# activate certain parts of source-code
COMPILE_OPTS += -DEXTENSION_${Name}=1
COMPILE_OPTS += -DBOARD=$Name
COMPILE_OPTS += -DBOARD_REVISION=${BoardRev}
BOARD=$Name
BOARD_REVISION=${BoardRev}

# define microcontroller used on board
STM32F10X_MD  =1
COMPILE_OPTS += -DUC_DEVICE=STM32F103RBT6
COMPILE_OPTS += -DUC_PACKAGE_PINS=64

INCLUDE_DIRS += -I additionals/$Name/

# Note: Make sure to set correct sizes for RAM and Flash
#
# updateProjectFiles.pl will automatically create memory regions in configs/memory.ld 
# for every define of this format (xxx = name of region, n = numeric value)
# COMPILE_OPTS += -DTTC_MEMORY_xxx_SIZE_K=n
# COMPILE_OPTS += -DTTC_MEMORY_xxx_START=n
# COMPILE_OPTS += -DTTC_MEMORY_xxx_ACCESS=rxwa
#
COMPILE_OPTS += -DTTC_MEMORY_RAM_SIZE_K=20
COMPILE_OPTS += -DTTC_MEMORY_RAM_START=0x20000000
COMPILE_OPTS += -DTTC_MEMORY_RAM_ACCESS=rxw
COMPILE_OPTS += -DTTC_MEMORY_ROM_SIZE_K=128
COMPILE_OPTS += -DTTC_MEMORY_ROM_START=0x08000000
COMPILE_OPTS += -DTTC_MEMORY_ROM_ACCESS=rx

vpath %.c additionals/$Name/

#{ LCD-Display 320x240x24

COMPILE_OPTS += -DTTC_GFX1_WIDTH=240
COMPILE_OPTS += -DTTC_GFX1_HEIGHT=320
COMPILE_OPTS += -DTTC_GFX1_DEPTH=24
COMPILE_OPTS += -DTTC_GFX1_DRIVER=tgd_ILI9320    # one from ttc_gfx_types.h:ttc_gfx_driver_e

#} LCD-Display
#{ single port pins
# reference: http://hlb-labor.de/cortexm3/ProtoBoards/MiniSTM32/MiniSTM32-Schematics.pdf

COMPILE_OPTS += -DTTC_LED1=PIN_PA2       # status LED 1
COMPILE_OPTS += -DTTC_LED2=PIN_PA3       # status LED 2
COMPILE_OPTS += -DTTC_SWITCH1=PIN_PA0    # press button switch 1
COMPILE_OPTS += -DTTC_SWITCH1_TYPE=tgm_input_floating # -> ttc_gpio_types.h
COMPILE_OPTS += -DTTC_SWITCH2=PIN_PA1    # press button switch 2
COMPILE_OPTS += -DTTC_SWITCH2_TYPE=tgm_input_floating # -> ttc_gpio_types.h
COMPILE_OPTS += -DPB_VR1=PIN_PB0        # potentiometer 3V3 -|R33|- GND

#}single port pins
#{ parallel ports (8 or 16 consecutive pins in same bank)
#
# Every 8 available, consecutive bits in same GPIO bank can form an 8-bit wide port.
# A 16-bit wide port requires 16 available, consecutive bits (obviously).
# A pin is available if it not driven externally or used for something else.
# Check circuit of your board for details.

# 8-bit wide parallel port #1 providing DB00..DB07
COMPILE_OPTS += -DTTC_PARALLEL08_1_BANK=GPIO_BANK_B
COMPILE_OPTS += -DTTC_PARALLEL08_1_FIRSTPIN=8

# 8-bit wide parallel port #2 providing DB08..DB15
COMPILE_OPTS += -DTTC_PARALLEL08_2_BANK=GPIO_BANK_C
COMPILE_OPTS += -DTTC_PARALLEL08_2_FIRSTPIN=0

# 8-bit wide parallel port #3 for external use (overlaps some leds and switches)
COMPILE_OPTS += -DTTC_PARALLEL08_3_BANK=GPIO_BANK_A
COMPILE_OPTS += -DTTC_PARALLEL08_3_FIRSTPIN=0

#}parallel ports
#{ USART1  - define universal synchronous asynchronous receiver transmitter #1
  COMPILE_OPTS += -DTTC_USART1=INDEX_USART1       # Connected to PL2303HX Serial->USB converter
  COMPILE_OPTS += -DTTC_USART1_TX=PIN_PA9   # UEXT1-3, UEXT40-3
  COMPILE_OPTS += -DTTC_USART1_RX=PIN_PA10  # UEXT1-4, UEXT40-4
  COMPILE_OPTS += -DTTC_USART1_CK=PIN_PA8   # EXT-40
#}USART1
#{ USART2  - define universal synchronous asynchronous receiver transmitter #2
  COMPILE_OPTS += -DTTC_USART2=INDEX_USART3       # Connected to male header
  COMPILE_OPTS += -DTTC_USART2_TX=PIN_PB10  # XS5-27
  COMPILE_OPTS += -DTTC_USART2_RX=PIN_PB11  # XS5-22
  COMPILE_OPTS += -DTTC_USART2_CK=PIN_PB12  # XS5-34
  COMPILE_OPTS += -DTTC_USART2_CTS=PIN_PB13 # XS5-33
  COMPILE_OPTS += -DTTC_USART2_RTS=PIN_PB14 # XS5-32
#}USART2
#{ SPI1    - define serial peripheral interface #1
  COMPILE_OPTS += -DTTC_SPI1=SPI1          # Connected to male header
  COMPILE_OPTS += -DTTC_SPI1_MOSI=PIN_PB15 # XS5-35
  COMPILE_OPTS += -DTTC_SPI1_MISO=PIN_PB14 # XS5-32
  COMPILE_OPTS += -DTTC_SPI1_SCK=PIN_PB13  # XS5-33
  COMPILE_OPTS += -DTTC_SPI1_NSS=PIN_PB12  # XS5-34
#}SPI1
#{ SPI2    - define serial peripheral interface #2
  COMPILE_OPTS += -DTTC_SPI2=SPI1          # Connected to TFT
  COMPILE_OPTS += -DTTC_SPI2_MOSI=PIN_PA7  # XS2-6
  COMPILE_OPTS += -DTTC_SPI2_MISO=PIN_PA6  # XS2-8
  COMPILE_OPTS += -DTTC_SPI2_SCK=PIN_PA5   # XS2-2
  COMPILE_OPTS += -DTTC_SPI2_NSS=PIN_PA4   # XS2-4
#}SPI2
#{ I2C1    - define inter-integrated circuit interface #1
  COMPILE_OPTS += -DTTC_I2C1=I2C1           # internal device being used for this port
  COMPILE_OPTS += -DTTC_I2C1_SDA=PIN_PB9    # XS5-20
  COMPILE_OPTS += -DTTC_I2C1_SCL=PIN_PB8    # XS5-30
#}I2C1
#{ I2C2    - define inter-integrated circuit interface #2
  COMPILE_OPTS += -DTTC_I2C2=I2C2           # internal device being used for this port
  COMPILE_OPTS += -DTTC_I2C2_SDA=PIN_PB11   # XS5-22
  COMPILE_OPTS += -DTTC_I2C2_SCL=PIN_PB10   # XS5-27
  COMPILE_OPTS += -DTTC_I2C2_SMBAL=PIN_PB12 # XS5-34
#}I2C2

END_OF_MAKEFILE
    createExtensionMakefileTail ${Name} #}
      File="../extensions/activate.${Name}.sh" #{
      createActivateScriptHead $File $0 "Protoboard Mini-STM32 with 2,8\" 320x240 Color Touch LCD and STM32F103RBT6 (Hardware revision ${BoardRev})"
      cat <<END_OF_ACTIVATE >>$File #{

  rm 2>/dev/null \$Dir_ExtensionsActive/makefile.100_board_*
  
  # create links into extensions.active/
  createLink \$Dir_Extensions/makefile.$Name \$Dir_ExtensionsActive/makefile.$Name '' QUIET
  
  activate.200_cpu_stm32f10x.sh QUIET "\$0"
  activate.400_lcd_320x240_ili9320.sh QUIET "\$0"
  activate.400_input_ads7843.sh QUIET "\$0"

END_OF_ACTIVATE
#}
      createActivateScriptTail $File $Name
      #}

    done
    echo "Installed successfully: $INSTALLPATH"
  else
    echo "ERROR occured during install: $INSTALLPATH"
    exit 10
  fi #}
  cd ..
  rmdir 2>/dev/null $INSTALLPATH  # removes only empty install dir
#} IMPLEMENT YOUR INSTALL SCRIPT ABOVE -----------------------------------------------

exit 0
