#!/bin/bash

#
#  Install script for Microcontroller STM32 F1xx 
#  with ARM Cortex M3 core + Std Peripherals Library for STM32F1xx.
#
#  Script written by Gregor Rebel 2012
# 
#  Feel free do distribute and change to your own needs!
#
# Tested successfully on:
# openSuSE 12.1 x86
# XUbuntu 11.10 x64

source scripts/installFuncs.sh

if [ "$USER" == "root" ]; then
  echo "$0: ERROR this script should not been run as user root!"
  exit 10
fi

INSTALLPATH="190_cpu_cortexm3"
dir "$INSTALLPATH"
cd "$INSTALLPATH"

#{ IMPLEMENT YOUR INSTALL SCRIPT BELOW -----------------------------------------------
  
  if [ ! -e OK.Documentation ]; then #{
    InstallDir=`pwd`
    dir ../../Documentation/uC
    dir ../../Documentation/uC
    cd ../../Documentation/uC
    
    dir ARM7
    cd ARM7
    getFile http://www.eecs.umich.edu/~prabal/teaching/eecs373-f11/readings/                            ARMv7-M_ARM.pdf ARMv7-M_Architecture_Reference_Manual.pdf                                   || ERROR="1"
    cd ..

    dir CortexM3
    cd CortexM3
    getFile http://www.st.com/internet/com/TECHNICAL_RESOURCES/TECHNICAL_LITERATURE/PROGRAMMING_MANUAL/ CD00228163.pdf Cortex-M3_programming_manual.pdf                                             || ERROR="1"
    getFile http://www.arm.com/files/pdf/                                                        IntroToCortex-M3.pdf An_Introduction_to_the_ARM_Cortex-M3_Processor.pdf
    cd ..
    dir STM32F1xx #{
    cd STM32F1xx
    DocDir=`pwd`
    
    Error=""
    getFile http://www.st.com/internet/com/TECHNICAL_RESOURCES/TECHNICAL_LITERATURE/REFERENCE_MANUAL/   CD00240193.pdf STM32_RM0038_Advanced_ARM-based_32bit_MCUs.pdf                               || ERROR="1"
    getFile http://www.st.com/internet/com/TECHNICAL_RESOURCES/TECHNICAL_LITERATURE/USER_MANUAL/        CD00298482.pdf Advanced_developers_guide_for-STM32F103xx-STM32F100xx.pdf                    || ERROR="1"
    getFile http://www.st.com/internet/com/TECHNICAL_RESOURCES/TECHNICAL_LITERATURE/REFERENCE_MANUAL/   CD00246267.pdf STM32_RM0041_ReferenceManual_STM32F100xx.pdf                                 || ERROR="1"
    getFile http://www.st.com/internet/com/TECHNICAL_RESOURCES/TECHNICAL_LITERATURE/REFERENCE_MANUAL/   CD00171190.pdf STM32_RM0008_STM32F101xx_STM32F102xx_STM32F103xx_STM32F105xx_STM32F107xx.pdf || ERROR="1"
    getFile http://www.st.com/internet/com/TECHNICAL_RESOURCES/TECHNICAL_LITERATURE/APPLICATION_NOTE/   CD00209826.pdf STM32_AN2824_STM32F10xxx_I2C_optimized_examples.pdf                          || ERROR="1"
    getFile http://www.st.com/internet/com/TECHNICAL_RESOURCES/TECHNICAL_LITERATURE/DATASHEET/          CD00220364.pdf STM32_STM32F105xx_STM32F107xx_Connectivity_line.pdf                          || ERROR="1"
    getFile http://www.st.com/internet/com/TECHNICAL_RESOURCES/TECHNICAL_LITERATURE/USER_MANUAL/        CD00251227.pdf UM0819_EthernetPhy_ST802RT1.pdf                                              || ERROR="1"
    getFile http://www.st.com/internet/com/TECHNICAL_RESOURCES/TECHNICAL_LITERATURE/APPLICATION_NOTE/   CD00209826.pdf STM32F10xxx_I2C_optimized_examples.pdf                                       || ERROR="1"

    cd .. #}
    
    cd "$InstallDir"
#}

    if [ "$Error" == "" ]; then
      echo "Documentation complete: $INSTALLPATH."
      touch OK.Documentation
    fi
  fi #}
  if [ ! -e OK.Install ];       then #{
    if [ ! -e OK.Packages ]; then #{
      touch OK.Packages
    fi #}
    if [ -e OK.Packages ]; then #{
      Error=""
      
    
      if [ "$Error" == "" ]; then
        touch OK.Install
      else
        echo "$0 - ERROR: $ERROR!"
        exit 10
      fi
    fi #}
  fi #}
  if [   -e OK.Install ];       then #{
    Name="${INSTALLPATH}"
    
    createExtensionMakefileHead ${Name} #{         create makefile for cpu
    File="../extensions/makefile.${Name}" 
    cat <<END_OF_MAKEFILE >>$File 

COMPILE_OPTS += -DEXTENSION_${Name}=1

ARM_CPU     =arm7tdmi
TARGET      =-mcpu=cortex-m3 -mthumb
TARGET_ARCHITECTURE_CM3=1
COMPILE_OPTS += -DTARGET_ARCHITECTURE_CM3
COMPILE_OPTS += -DTARGET_DATA_WIDTH=32

# architecture specific support code
INCLUDE_DIRS += -Icommon/cm3
vpath %.c common/cm3

# Basic Cortex M3 support library
INCLUDE_DIRS += -Iadditionals/270_CPAL_CMSIS/CM3/CoreSupport/
vpath %.c additionals/270_CPAL_CMSIS/CM3/CoreSupport/

MAIN_OBJS += core_cm3.o

# suppress warnings about invalid wchar sizes during linker stage
LDFLAGS += --no-wchar-size-warning

END_OF_MAKEFILE
    createExtensionMakefileTail ${Name} #}
    File="../extensions/activate.${Name}.sh" #{    create activate for cpu
    createActivateScriptHead $File $0 "CPU STM32F10x"
    cat <<END_OF_ACTIVATE >>$File #{
#? rm 2>/dev/null \$Dir_ExtensionsActive/makefile.200_cpu_*

# create links into extensions.active/
createLink ../extensions/makefile.$Name \$Dir_ExtensionsActive/makefile.$Name '' QUIET

END_OF_ACTIVATE
#}
    createActivateScriptTail $File $Name
    #}

    echo "Installed successfully: $INSTALLPATH"
  fi #}
  cd ..
  rmdir 2>/dev/null $INSTALLPATH  # removes only empty install dir
#} IMPLEMENT YOUR INSTALL SCRIPT ABOVE -----------------------------------------------

exit 0
