#!/bin/bash
#
#
#  Install script for cc1190 power amplifier support 
#
#  Initial Script written by Gregor Rebel 2010-2012.
# 
#  Published under GNU Lesser General Public License
#  Check LEGAL.txt for details.
#
# Tested successfully on:
# XUbuntu 11.10 x64

source scripts/installFuncs.sh

if [ "$USER" == "root" ]; then
  echo "$0: ERROR this script should not been run as user root!"
  exit 10
fi

INSTALLPATH="400_radio_cc1190"
dir "$INSTALLPATH"
cd "$INSTALLPATH"

#{ IMPLEMENT YOUR INSTALL SCRIPT BELOW -----------------------------------------------

  Name="$INSTALLPATH"
  createExtensionMakefileHead ${Name}      #{ create makefile
  cat <<END_OF_MAKEFILE >>../extensions/makefile.$Name 

# external radio connected via SPI
MAIN_OBJS += radio_cc1190.o

# Note: Check radio_cc1190.h for a list of additional required compile options!

END_OF_MAKEFILE
  createExtensionMakefileTail ${Name} #}
  File="../extensions/activate.${Name}.sh" #{ create activate script
  createActivateScriptHead $File "$0" "Architecture independent support for Universal Synchronous Asynchronous Serial Receiver Transmitter (RADIO)"
  cat <<END_OF_ACTIVATE >>$File
# available variables (-> scripts/installFuncs.sh:createActivateScriptHead())
#
# Paths to certain directories in current project folder
#   \$Dir_Extensions        -> extensions/
#   \$Dir_ExtensionsLocal   -> extensions.local/
#   \$Dir_ExtensionsActive  -> extensions.active/
#   \$Dir_Additionals       -> additionals/

# remove activated variants of same type
# rm 2>/dev/null \$Dir_Additionals/${Name}_*

# create links into extensions.active/
createLink "\$Dir_Extensions/makefile.$Name" "\$Dir_ExtensionsActive/makefile.$Name" '' QUIET

END_OF_ACTIVATE
  createActivateScriptTail $File $Name
  #}
  echo "Installed successfully: $Name"

  for Index in 1 2 3 4 5; do    
    Name="150_board_extension_radio_${Index}_cc1190"
    createExtensionMakefileHead ${Name}      #{ create makefile
    cat <<END_OF_MAKEFILE >>../extensions/makefile.$Name

#{ TTC_RADIO${Index}_AMPLIFIER pin configuration for amplifier of first radio transceiver
#
# example configuration of a cc1190 radio amplifier as an extension board
# connect amplifier exactly like described here or define these constants in your extensions.local/makefile.700_extra_settings 

# set type of driver to use (one from ttc_radio_types.h/ttc_radio_driver_e)
COMPILE_OPTS += -DTTC_RADIO${Index}_AMPLIFIER=trd_cc1190

# pin configurations below are highly dependent on current low-level driver (here cc1190)
COMPILE_OPTS += -DTTC_RADIO${Index}_AMPLIFIER_HGM=PIN_PC8
COMPILE_OPTS += -DTTC_RADIO${Index}_AMPLIFIER_HGM_TYPE=tgm_output_push_pull # -> ttc_gpio_types.h

COMPILE_OPTS += -DTTC_RADIO${Index}_AMPLIFIER_PIN_LNA_ENABLE=PIN_PC8
COMPILE_OPTS += -DTTC_RADIO${Index}_AMPLIFIER_PIN_LNA_ENABLE_TYPE=tgm_output_push_pull # -> ttc_gpio_types.h/ttc_gpio_mode_e

COMPILE_OPTS += -DTTC_RADIO${Index}_AMPLIFIER_PIN_PA_ENABLE=PIN_PC8
COMPILE_OPTS += -DTTC_RADIO${Index}_AMPLIFIER_PIN_PA_ENABLE_TYPE=tgm_output_push_pull # -> ttc_gpio_types.h/ttc_gpio_mode_e

#}TTC_RADIO${Index}_AMPLIFIER

END_OF_MAKEFILE
    createExtensionMakefileTail ${Name} #}
    File="../extensions/activate.${Name}.sh" #{ create activate script
  createActivateScriptHead $File "$0" "Example extension for external radio amplifier cc1190"
  cat <<END_OF_ACTIVATE >>$File
# available variables (-> scripts/installFuncs.sh:createActivateScriptHead())
#
# Paths to certain directories in current project folder
#   \$Dir_Extensions        -> extensions/
#   \$Dir_ExtensionsLocal   -> extensions.local/
#   \$Dir_ExtensionsActive  -> extensions.active/
#   \$Dir_Additionals       -> additionals/

# remove activated variants of same type
# rm 2>/dev/null \$Dir_ExtensionsActive/makefile.600_example_*

activate.400_radio_cc1190.sh     QUIET \"\$0\"
activate.500_ttc_radio.sh        QUIET \"\$0\"
activate.500_ttc_gpio.sh         QUIET \"\$0\"
activate.500_ttc_spi.sh          QUIET \"\$0\"

# create links into extensions.active/
createLink "\$Dir_Extensions/makefile.$Name" "\$Dir_ExtensionsActive/makefile.$Name" '' QUIET

END_OF_ACTIVATE
  createActivateScriptTail $File $Name
  #}
    echo "Installed successfully: $Name"
  done

cd ..
#} IMPLEMENT YOUR INSTALL SCRIPT ABOVE -----------------------------------------------

exit 0
