#!/bin/bash
#
#
#  Install script for generic I2C support 
#  URL: http://code.google.com/p/fixpointlib/
#
#  Initial Script written by Gregor Rebel 2010-2012.
# 
#  Published under GNU Lesser General Public License
#  Check LEGAL.txt for details.
#
# Tested successfully on:
# openSuSE 12.1 x86
# XUbuntu 11.10 x64

source scripts/installFuncs.sh

if [ "$USER" == "root" ]; then
  echo "$0: ERROR this script should not been run as user root!"
  exit 10
fi

INSTALLPATH="500_ttc_i2c"
dir "$INSTALLPATH"
cd "$INSTALLPATH"

#{ IMPLEMENT YOUR INSTALL SCRIPT BELOW -----------------------------------------------

  Name="$INSTALLPATH"
  createExtensionMakefileHead ${Name}      #{ create makefile
  cat <<END_OF_MAKEFILE >>../extensions/makefile.$Name 

MAIN_OBJS += ttc_i2c.o

ifdef TARGET_ARCHITECTURE_STM32
  MAIN_OBJS += stm32_i2c.o
endif

END_OF_MAKEFILE
  createExtensionMakefileTail ${Name} #}
  File="../extensions/activate.${Name}.sh" #{ create activate script
  createActivateScriptHead $File "$0" "Architecture independent support for Inter-integrated circuit interface (I2C)"
  cat <<END_OF_ACTIVATE >>$File
# available variables (-> scripts/installFuncs.sh:createActivateScriptHead())
#
# Paths to certain directories in current project folder
#   \$Dir_Extensions        -> extensions/
#   \$Dir_ExtensionsLocal   -> extensions.local/
#   \$Dir_ExtensionsActive  -> extensions.active/
#   \$Dir_Additionals       -> additionals/

# remove activated variants of same type
# rm 2>/dev/null \$Dir_Additionals/${Name}_*

ArchitectureSupportAvailable=""

# check if stm32fxxx-support is available
if [ -e extensions/activate.250_stm_std_peripherals__gpio.sh ]; then
  activate.250_stm_std_peripherals__i2c.sh QUIET \"\$0\"
  activate.250_stm_std_peripherals__rcc.sh QUIET \"\$0\"
  activate.250_stm_std_peripherals__gpio.sh QUIET \"\$0\"
  ArchitectureSupportAvailable="1"
fi

if [ "\$ArchitectureSupportAvailable" == "1" ]; then
  activate.500_ttc_gpio.sh QUIET \"\$0\"
  # create links into extensions.active/
  createLink "\$Dir_Extensions/makefile.$Name" "\$Dir_ExtensionsActive/makefile.$Name" '' QUIET
else
  echo "\$0 - ERROR: No support for current architecture available!"
fi

END_OF_ACTIVATE
  createActivateScriptTail $File $Name
  #}

  echo "Installed successfully: $Name"

  for Name in 600_example_i2c_master 600_example_i2c_slave; do
    createExtensionMakefileHead ${Name}      #{ create makefile
    cat <<END_OF_MAKEFILE >>../extensions/makefile.$Name 

MAIN_OBJS += example_i2c.o

# we have to define at least one font to make ttc_font.h happy
COMPILE_OPTS += -DTTC_FONT1=font_type1_16x24

END_OF_MAKEFILE
    createExtensionMakefileTail ${Name} #}
    File="../extensions/activate.${Name}.sh" #{ create activate script
    createActivateScriptHead $File "$0" "Example of how to use architecture independent Serial Peripheral Interface (I2C) support"
    cat <<END_OF_ACTIVATE >>$File
# available variables (-> scripts/installFuncs.sh:createActivateScriptHead())
#
# Paths to certain directories in current project folder
#   \$Dir_Extensions        -> extensions/
#   \$Dir_ExtensionsLocal   -> extensions.local/
#   \$Dir_ExtensionsActive  -> extensions.active/
#   \$Dir_Additionals       -> additionals/

# remove activated variants of same type
# rm 2>/dev/null \$Dir_ExtensionsActive/makefile.600_example_*

activate.500_ttc_i2c.sh QUIET
activate.500_ttc_gfx.sh    QUIET "\$0"
activate.500_ttc_font__font_type1_16x24.sh QUIET "\$0"

# create links into extensions.active/
createLink "\$Dir_Extensions/makefile.$Name" "\$Dir_ExtensionsActive/makefile.$Name" '' QUIET

END_OF_ACTIVATE
    createActivateScriptTail $File $Name
    #}
    echo "Installed successfully: $Name"
  done
cd ..
#} IMPLEMENT YOUR INSTALL SCRIPT ABOVE -----------------------------------------------

exit 0
