#!/bin/bash
#
#
#  Install script for generic radio support 
#
#  Initial Script written by Gregor Rebel 2010-2012.
# 
#  Published under GNU Lesser General Public License
#  Check LEGAL.txt for details.
#
# Tested successfully on:
# XUbuntu 11.10 x64

source scripts/installFuncs.sh

if [ "$USER" == "root" ]; then
  echo "$0: ERROR this script should not been run as user root!"
  exit 10
fi

INSTALLPATH="999_QtCreator"
dir "$INSTALLPATH"
cd "$INSTALLPATH"

#{ IMPLEMENT YOUR INSTALL SCRIPT BELOW -----------------------------------------------

Binary=`which qtcreator`
if [ "$Binary" == "" ]; then
  Binary=`find /opt/ -name qtcreator 2>/dev/null | grep bin`
fi
if [ "$Binary" == "" ]; then
  echo "Binary qtcreator not found: installing new QtCreator.."
  
  #openSUSE
  installPackage qt-creator
  
  #ubuntu
  installPackage qtcreator

  identifyOS
  if [ "$OS_ARCHITECTURE" == "x86_32" ]; then
    getNewestFile "http://origin.releases.qt-project.org/qtcreator/current_stable/" ".*linux-x86-.*" qt-creator-linux-opensource.bin
  fi
  if [ "$OS_ARCHITECTURE" == "x86_64" ]; then
    getNewestFile "http://origin.releases.qt-project.org/qtcreator/current_stable/" ".*linux-x86_64-.*" qt-creator-linux-opensource.bin
  fi
  if [ ! -e qt-creator-linux-opensource.bin ]; then
    echo "$0 - ERROR: Cannot find a stable release of QtCreator for Architecture '$OS_ARCHITECTURE' !"
    exit 10
  else
    chmod +x qt-creator-linux-opensource.bin
    InstallDir="Source/qtcreator/bin"
    Binary="$HOME/$InstallDir/qtcreator"
    ./qt-creator-linux-opensource.bin --mode unattended --installdir $HOME/Source/qtcreator
    if [ -e "$Binary" ]; then
      addLine ../scripts/SourceMe.sh "export PATH=\"\$PATH:\$HOME/$InstallDir/\" #$0"
      echo "QtCreator successfully installed in $HOME/$InstallDir/qtcreator"
    else
      echo "$0 - ERROR: Could not install QtCreator (missing file '$Binary')"
      exit 10
    fi
  fi
else
  echo "$0 - QtCreator already installed: $Binary"
fi

#} IMPLEMENT YOUR INSTALL SCRIPT ABOVE -----------------------------------------------

cd ..

exit 0
