#!/usr/bin/perl

my $Option = shift(@ARGV);
my $OnlyPrintTotal;
if ($Option eq 'TOTAL') { $OnlyPrintTotal = 1; }

my $OBJDUMP="arm-none-eabi-objdump";

my $ObjectFiles = `ls *.o`;
my @ObjectFiles = split("\n", $ObjectFiles);

my %SumTotalSize; # sum of all sizes in all regions

foreach my $ObjectFile (@ObjectFiles) {
  
  my $BSS  = `$OBJDUMP -d -j .bss $ObjectFile`;
  my $Data = `$OBJDUMP -d -j .data $ObjectFile`;
  my @BSS1  = split("\n", $BSS);  
  my @Data1 = split("\n", $Data);
  
  my @BSS;
  foreach my $BSS (@BSS1) {   # @BSS = all non empty entries from @BSS1
    my $Stripped = strip($BSS);
    $Stripped =~ s/\.//g;
    $Stripped = strip($Stripped);
    if ($Stripped) { push @BSS, $Stripped; }
  }
  my @Data;
  foreach my $Data (@Data1) { # @Data = all non empty entries from @Data1
    my $Stripped = strip($Data);
    $Stripped =~ s/\.//g;
    $Stripped = strip($Stripped);
    if ($Stripped) { push @Data, $Stripped; }
  }

  shift(@BSS); shift(@Data);  # remove first line containing filename 

  if (scalar(@BSS) + scalar(@Data) > 0) {
    my %TotalSize; # sizes in all regions
    my @AllocationLines;
    
    my $CurrentAddress = 0;
    my $LastAddress    = 0;
    my %Regions = ( ' BSS' => \@BSS, DATA => \@Data);
    foreach my $RegionName (sort keys %Regions) {
      my $RegionRef = $Regions{$RegionName};
      if (@$RegionRef) {
        
        # count amount of characters in last line to deduce size of last item
        if (index($RegionRef->[-1], ':  ') == -1)
        { push @$RegionRef, '   0:   00 00 00 00 '; }
        my $LastLine1 = $RegionRef->[-2];
        my $LastLine2 = $RegionRef->[-1];
        my $DotPos = index($LastLine2, "           ");
        if ($DotPos > -1) { $LastLine2 = substr($LastLine2, 0, $DotPos); } # remove trailing '.word '
        my $DoubleColonPos = index($LastLine2, ':');
        if ($DoubleColonPos > -1) { $LastLine2 = substr($LastLine2, $DoubleColonPos + 1); } # remove leading 'N: '
        $LastLine2 = strip($LastLine2);
        $LastLine2 =~ s/ //g; # remove all spaces
        my $SizeOfLastEntry = length($LastLine2) / 2; # only nibble-digit left
        my $PreviousAddressHex = substr($LastLine1, 0, index($LastLine1, ' '));
        my $PreviousAddress = hex($PreviousAddressHex);
        my $NextAddress = $PreviousAddress + $SizeOfLastEntry;
        my $NewLine1 = sprintf("%08x <END_OF_OBJECT>:",         $NextAddress);
        my $NewLine2 = sprintf("  %x: 000000 .word 0x00000000", $NextAddress);
        push @$RegionRef, $NewLine1;
        push @$RegionRef, $NewLine2;

        $LastAddress = 0;
        foreach my $Line (@$RegionRef) {
          if ( (substr($Line, 0, 1) eq '0') &&  # assume that first digit of address is always zero in our uC
               (substr($Line, 1, 1) ne ':')     # exclude '0:' lines
               ) {
            my $AddressHex = substr($Line, 0, index($Line, ' '));
            $CurrentAddress = hex($AddressHex);
            my $LastSize = $CurrentAddress - $LastAddress;
            my $PosLt = index($Line, '<');
            my $PosGt = index($Line, '>');
            my $SymbolName = substr($Line, $PosLt + 1, $PosGt - $PosLt - 1);
            $TotalSize{$RegionName}    += $LastSize;
            $SumTotalSize{$RegionName} += $LastSize;
          
            push @AllocationLines, $LastSize;
            push @AllocationLines, "  ".$RegionName.": ".$AddressHex.': '.$SymbolName.''; 
            $LastAddress = $CurrentAddress;
          }
        }
        #push @AllocationLines, 0;
      }
    }
    if (@AllocationLines) {
      unless ($OnlyPrintTotal) 
      { print "\nFile: $ObjectFile allocates ".join(', ', map { $TotalSize{$_}.' bytes '.strip($_)} sort keys %TotalSize).":\n"; }
      shift(@AllocationLines); # first element is size 0
      while (@AllocationLines) {
        my $AllocationLine = shift(@AllocationLines);
        my $Size           = shift(@AllocationLines) + 0;
        unless ($OnlyPrintTotal) {
          if (index($AllocationLine, 'END_OF_OBJECT') == -1)
          { print $AllocationLine." allocates $Size bytes\n"; }
        }
        if ($Size < 0) { die("$0 - ERROR: Illegal size $Size found!"); }
      }
      if ($TotalSize{DATA} < 0) { sleep(1); die("$0 - ERROR: Illegal size ".$TotalSize{DATA}." found!"); }
    }
  }
}
my $SumTotal = 0;
map { $SumTotal += $SumTotalSize{$_}; } keys %SumTotalSize;
unless ($OnlyPrintTotal) { print "\nTotal allocations: ".join(' + ', map { $SumTotalSize{$_}.' bytes '.strip($_)} sort keys %SumTotalSize)." = ".$SumTotal." bytes.\n";  }
else { print "$SumTotal\n"; }

sub strip() {                  # removes trailing and leading whitechars from given string
  my $String = shift;
  
  unless (%s_WhiteChar) {
    $s_WhiteChar{"\n"} = 1;
    $s_WhiteChar{"\r"} = 1;
    $s_WhiteChar{"\t"} = 1;
    $s_WhiteChar{" "}  = 1;
  }
  while ( $s_WhiteChar{substr($String, 0, 1)} ) { substr($String, 0, 1) = undef; }
  while ( $s_WhiteChar{substr($String, -1, 1)} )   { substr($String, -1)   = undef; }
  
  return $String;
}
