#!/usr/bin/perl

my $URL           = shift(@ARGV);
my $Wildcard      = shift(@ARGV);
my $UrlPrefixFile = shift(@ARGV);
my $FileNameFile  = shift(@ARGV);
my $DirNameFile   = shift(@ARGV);

unless ($URL) {
  print "$0 URL WILDCARD [URLPREFIX_FILE [FILENAME_FILE] [DIRNAME_FILE]]\n";
  print "    Will look for download on website $URL that matches given wildcard and has highest version number.\n";
  print "\n";
  print "    URL             website that lists file downloads on an index page\n";
  print "    WILDCARD        regular expression identifying interesting files\n";
  print "    URLPREFIX_FILE  file where to write url-prefix of found candidate to (optional)\n";
  print "    FILENAME_FILE   file where to write filename of found candidate to (optional)\n";
  print "    DIRNAME_FILE    file where to write name of directory of lowest level (optional)\n";
  print "\n";
  print "    Examples\n";
  print "    find newest binutils*bz2 file:\n";
  print "    $0 http://ftp.gnu.org/gnu/binutils/ \"binutils-2\..*bz2\$\"\n";
  print "\n";
  print "\n";
  exit 10;
}

my $TempFile="tmp.FindNewestDownload";
system("rm -f $TempFile; wget -O \"$TempFile\" \"$URL\" >log.FindNewestDownload 2>&1");

#? if (substr($URL, -1) eq '/') { substr($URL, -1) = undef; }
my $URL_Base = $URL;
if (substr($URL_Base, -1) eq '/') { substr($URL_Base, -1) = undef; }
else {
  my $LastSlashPos = rindex($URL_Base, '/');
  substr($URL_Base, $LastSlashPos) = undef;
}

my $Page;
open(IN, "<$TempFile") or die("$0: ERROR - Cannot read file '$TempFile' downloaded from '$URL' ($!)");
while (<IN>) { $Page .= $_; }
close(IN);
$Page =~ s/<A /<a /g;
$Page =~ s/<\/A/<\/a/g;
$Page =~ s/< \/A/<\/a/g;
$Page =~ s/< \/a/<\/a/g;

my %Links;
my @Candidates;
my $LinkPos = index($Page, '<a ');
while ($LinkPos > -1) { # find all candidates matching given wildcard
  my $String = substr($Page, $LinkPos + 2);
  
  my $GtPos = index($String, '>');
  my $FirstPart = substr($String, 0, $GtPos); # 'href="...." ...'
  my $HrefPos = index(lc($FirstPart), 'href');
  if (1 && ($HrefPos > -1) ) {
    my $Quote1Pos = index($FirstPart, '"', $HrefPos);
    my $Quote2Pos = index($FirstPart, '"', $Quote1Pos + 1);
    my %Link;
    if ($Quote2Pos > $Quote1Pos) { $Link{href} = substr($FirstPart, $Quote1Pos + 1, $Quote2Pos - $Quote1Pos - 1); }
    if ($Link{href}) {
      my $EndOfTextPos = index($String, '</a', $GtPos + 1);
      my $Text = substr($String, $GtPos + 1, $EndOfTextPos - $GtPos - 1);
      $Text =~s/(<[^>]>)//g; # remove all <..> from $Text
      $Link{text} = $Text;

      my $IsCandidate = 1;
      if ($Wildcard) {
        unless ($Text =~ m/$Wildcard/) { $IsCandidate = 0; }
      }
      if ($IsCandidate) {
        push @Candidates, \%Link;
      }
    }
  }
  
  $LinkPos = index($Page, '<a', $LinkPos + 1);
};

@Candidates = sort { # sort candidates by version number
  my $A = strip($a->{text});
  my $B = strip($b->{text});
  
  my $Compare = 0;
  for (my $Index = 0; $Index < length($A); $Index++) {
    if (isNumber($A, $Index) && isNumber($B, $Index)) {
      (my $NumberA, my $LenNumberA) = getNumber($A, $Index);
      (my $NumberB, my $LenNumberB) = getNumber($B, $Index);
      $Compare = ($NumberB <=> $NumberA);
      $Index += $LenNumberA;
    }
    else { 
      $Compare = (substr($B, $Index,1) cmp substr($A, $Index,1) );
    }
    if ($Compare != 0) { last; }
  }
  if ($Compare == 0) {
    $Compare = length($A) <=> length($B);
  }
  #X if ($Compare < 0)    { print "$A < $B\n"; }
  #X elsif ($Compare > 0) { print "$A > $B\n"; }
  #X else                 { print "$A = $B\n"; }
  
  $Compare;
} @Candidates;
if (@Candidates) {
  my $Candidate = $Candidates[0];
  my $Link = $Candidate->{href};
  my $CandidateURL;
  if ( index(lc($Link), 'http://')  > -1 ) { $CandidateURL = $Link; }
  if ( index(lc($Link), 'https://') > -1 ) { $CandidateURL = $Link; }
  if ( index(lc($Link), 'ftp://')   > -1 ) { $CandidateURL = $Link; }
  unless ($CandidateURL) {
    $CandidateURL = $URL_Base.'/'.$Link; 
  }
  print $CandidateURL."\n";
  
  my $LastSlashPos = rindex($CandidateURL, '/');
  if ($UrlPrefixFile) {
    open(OUT, ">$UrlPrefixFile") or die("$0 - ERROR: Cannot write to file '$UrlPrefixFile' ($!)");
    if ($LastSlashPos > -1) { print OUT substr($CandidateURL, 0, $LastSlashPos + 1)."\n"; }
    close(OUT);
  }
  if ($FileNameFile) {
    open(OUT, ">$FileNameFile") or die("$0 - ERROR: Cannot write to file '$FileNameFile' ($!)");
    print OUT substr($CandidateURL, $LastSlashPos + 1)."\n";
    close(OUT);
  }
  if ($DirNameFile) {
    open(OUT, ">$DirNameFile") or die("$0 - ERROR: Cannot write to file '$DirNameFile' ($!)");
    while (substr($CandidateURL, -1) eq '/') { substr($CandidateURL, -1) = undef; }
    my $LastSlashPos = rindex($CandidateURL, '/');
 
    my $LastDir = substr($CandidateURL, $LastSlashPos + 1);
    
    print OUT $LastDir."\n";
    close(OUT);
  }
}

if (0) { # print all candidates
  map {
    my $LinkRef = $_;
    print "Entry: ".join(" ", map { $_.'="'.$LinkRef->{$_}.'"'; } sort keys %{$LinkRef})."\n";
  } @Candidates;
}

sub findNextDifference {
  my $A = shift;
  my $B = shift;
  my $StartPos = shift() + 0;
  
  for (; $StartPos < length($A); $StartPos++) {
    if ( substr($A, $StartPos, 1) ne substr($B, $StartPos, 1) ) {
      return $StartPos;
    }
    $StartPos++;
  }
  return $StartPos;
}
sub strip {
  my $String = shift;
  
  while ( isWhiteChar(substr($String, 0, 1)) ) 
  { substr($String, 0, 1) = undef; }
  while ( isWhiteChar(substr($String, -1)) ) 
  { substr($String, -1) = undef; }
  
  return $String;
}
sub isWhiteChar {
  my $C = shift;
  
  if ( ($C eq ' ')  ||
       ($C eq "\n") ||
       ($C eq "\r") ||
       ($C eq "\t")
     )
  { return 1; }
  
  return 0;
}
sub isNumber {
  my $String = shift;
  my $Pos    = shift;

  my %IsNumber = ('0' => 1, '1' => 1, '2' => 1, '3' => 1, '4' => 1, '5' => 1, '6' => 1, '7' => 1, '8' => 1, '9' => 1);
  
  my $C = substr($String, $Pos, 1);
  
  return $IsNumber{$C};
}
sub getNumber { # extracts number starting at given position in given string
  my $String = shift;
  my $Pos    = shift;
  
  my $Number;
  
  while ( ($Pos < length($String)) && isNumber($String, $Pos) ) {
    $Number .= substr($String, $Pos, 1);
    $Pos++;
  }
  
  return ($Number + 0, length($Number));
}